/* NMM - Network-Integrated Multimedia Middleware
 *
 * Copyright (C) 2002-2005
 *                    NMM work group,
 *                    Computer Graphics Lab,
 *                    Saarland University, Germany
 *                    http://www.networkmultimedia.org
 *
 * Maintainer:         
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or (at your option) any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 * 
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307
 * USA
 */

#ifndef __NMMXMLPARSER_HPP
#define __NMMXMLPARSER_HPP

#include "../main/application.hpp"

#include "nmm/interfaces/video/osd/manager/IBitmapReader.hpp"

class XMLTree;
class XMLNode;

namespace NMM {

  class MMBoxApplication;
  class MMBoxState;

  typedef list<MMBoxState*> state_list_t;

  class XmlParser {

  private:
    /* a list of all added states */
    state_list_t     state_list;

    XMLTree* xml;
    XMLNode* rootnode;

    string path;

    /**
     * Parse menu structure and adds AutoMenu instances to state_list
     * \param bmreader Bitmapreader that is needed to load the button icon
     * \param app pointer to MMBoxApplication that is passed to the Automenu
     * \returns true, if menu could be parsed
     */
    bool parseMenu( IBitmapReader* bmreader, MMBoxApplication* app );

    /**
     * Parse Playlist structure and adds Playlist instances to state_list
     * \param app pointer to MMBoxApplication that is passed to the Automenu
     * \returns true, if playlist could be parsed
     */
    bool parsePlaylist( IBitmapReader* bmreader, MMBoxApplication* app );
  
    /* parse XML data for cd state */
    bool parseCDPlay( IBitmapReader* bmreader, MMBoxApplication* app );

    /* parse XML data for mp3 state */
    bool parseMP3Play( IBitmapReader* bmreader, MMBoxApplication* app );

    /* parse XML data for slideshow */
    bool parseSlideShow( IBitmapReader* bmreader, MMBoxApplication* app );

     /* parse XML data for transcode */
     bool parseTranscode( IBitmapReader* bmreader, MMBoxApplication* app );

    /* parse XML data for dvd state */
    bool parseDVDPlay( IBitmapReader* bmreader, MMBoxApplication* app );

    /* parse XML data for dvb state */
    bool parseDVBPlay( IBitmapReader* bmreader, MMBoxApplication* app );

    /* parse XML data for dvbrecord state */
    bool parseDVBRecord( IBitmapReader* bmreader, MMBoxApplication* app );

    /* parse XML data for epg state */
    bool parseEPG( IBitmapReader* bmreader, MMBoxApplication* app );

    /* parse XML data for epg-search state */
    bool parseEPGSearch( IBitmapReader* bmreader, MMBoxApplication* app );

    /* parse XML data for search-results in epg-search state */
    bool parseEPGSearchResult( IBitmapReader* bmreader, MMBoxApplication* app );

    /* parse XML data for cd-grabber state */
    bool parseCDGrabber( IBitmapReader* bmreader, MMBoxApplication* app );

    /* parse XML data for dvd-grabber state */
    bool parseDVDGrabber( IBitmapReader* bmreader, MMBoxApplication* app );

    bool parseCamera( IBitmapReader* bmreader, MMBoxApplication* app );

    /* parse XML data for tvtimer state */
    bool parseTvTimer( IBitmapReader* bmreader, MMBoxApplication* app );

    /* parse XML data for config state */
    bool parseConfig( IBitmapReader* bmreader, MMBoxApplication* app );

    /* parse XML data for taskmgr state */
    bool parseTaskMgr( IBitmapReader* bmreader, MMBoxApplication* app );

    /* get resource path  */
    bool parseResourcePath();
    
    /* get config  */
    bool parseSettings( MMBoxApplication* app );
    
    /* parse XML data for osd symbols like play, pause, forward, backward */
    bool parseOSDSymbols( IBitmapReader* bmreader, MMBoxApplication* app );

    /**
     * Parse osd buttons into bitmapreader
     * \return true if successful
     */
    bool parseOSD( IBitmapReader* bmreader, MMBoxApplication* app  );

    /**
     * convert a string that contains 4 int values (0-255) seperated by spaces to an integer value
     */
    unsigned int strToInt( string s );

  public:
    XmlParser( string filename = string("") );

    /**
     * \note: if the class is deleted, only the list of MMBoxState pointers is deleted, but
     * NOT the instances itselves
     */
    ~XmlParser();

    /**
     * Read XML File, create root node
     * \param filename name of XMl file
     * \return if file could be parsed successfully
     */
    bool open( string filename );

    /**
     * Parse all known structures menu, playlist, ...
     * \param bmreader Bitmapreader that is needed to load graphics
     * \param app pointer to MMBoxApplication that is passed to the states
     * \returns pointer to list of MMBoxState instances
     */
    state_list_t* parse( IBitmapReader* bmreader, MMBoxApplication* app );

    /**
     * Parse standard state tags: background, lazy_background, needaudio
     * \param state pointer to state
     * \param node pointer to node that descibes state
     */
    void parseStdState( XMLNode* node, MMBoxState* state);

    const string getResourcePath();
    void setResourcePath(const string);
  };

}

#endif // __NMMXMLParser_HPP
