/* NMM - Network-Integrated Multimedia Middleware
 *
 * Copyright (C) 2002-2005
 *                    NMM work group,
 *                    Computer Graphics Lab,
 *                    Saarland University, Germany
 *                    http://www.networkmultimedia.org
 *
 * Maintainer:         
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or (at your option) any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 * 
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307
 * USA
 */


#ifndef __DVDAPPLICATION__
#define __DVDAPPLICATION__

#include "../../main/application.hpp"
#include "../../main/mmboxstate.hpp"

// used interfaces
#include "nmm/interfaces/base/INode.hpp"
#include "nmm/interfaces/device/dvd/IDVD.hpp"
#include "nmm/interfaces/device/dvd/IDVDNav.hpp"

namespace NMM {
  
  /** DVD Play State.
   * MMBox State for playing a DVD with menu navigation.
   * @bug DVD Subtitles sometimes stay visible after chapter seeking
   * @bug No seeking or pause symbols are displayed during seek and pause mode
   * @bug Selecting a submenu and pressing return does not update the display, only after pressing a navigation-key (up/down/left/right)
   */
  class DvdState : public MMBoxState {
  public:
  
    DvdState( string,  MMBoxApplication* );
    ~DvdState();
  
    Result receiveMessage( string message );
    Result doCreate( string );
    Result doKill();
    Result doInitialize( string );
    Result doUninitialize();
  
    /* here all used "events" must be entered */
    Result Up();
    Result Down();
  
    Result Back();
    Result Return();
  
    Result Left();
    Result Right();
  
    Result Stop();
  
    Result Forward();
    Result Rewind();
  
    Result Pause();
  
    Result specialKey01();
    void disconnectSinkSynchronizer(ISinkSynchronizer* tmp_sync);
    void connectSinkSynchronizer(ISinkSynchronizer* tmp_sync);
  private:
    GraphDescription graph;
    ISinkSynchronizer_var sink_synchronizer;
    INode* display;
    INode* audio_play;
    INode* osd_node;
    NodeDescription* playback_nd;
    NodeDescription* osdNode_nd;
    // own nodes
    INode* deinter;
    INode* dvd;
    INode* mpg2yuv;
    INode* demux;
    INode* ac3_dec;
    INode* overlay;
    INode* spudec;
    
    // interfaces
    IDVD* dvd_interface;
    IDVDNav* dvdnav;
    
    // options
    bool ac3_passthrough;
    bool pause;
    string device;
    
    void showAlert(string title, string message);
    
    void requestNodes(string);
    void cleanup();
  };
}

#endif //__DVDAPPLICATION__
