/* NMM - Network-Integrated Multimedia Middleware
 *
 * Copyright (C) 2002-2005
 *                    NMM work group
 *                    Computer Graphics Lab,
 *                    Saarland University, Germany
 *                    http://www.networkmultimedia.org
 *
 * Maintainer:        Christoph Wellner <chwellner@graphics.cs.uni-sb.de>
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or (at your option) any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307
 * USA
 */

#ifndef MMBOX_EPG_HPP
#define MMBOX_EPG_HPP MMBOX_EPG_HPP

#include <string>
#include <time.h>

// the app
#include "../../main/application.hpp"
#include "../../main/mmboxstate.hpp"

// stored Interfaces
#include "nmm/interfaces/video/osd/manager/ITextView.hpp"
#include "nmm/interfaces/misc/tvguide/ITVGuide.hpp"

#include "nmm/utils/DirReader.hpp"
#include "nmm/plugins/video/osd/manager/OSDElements/tableunit.hpp"

#include "nmm/base/proxy/tcp/TCPStrategy.hpp"
#include "nmm/misc/tvguide/TVGuide.hpp"

#include "../tvtimer/tvtimer.hpp"


namespace NMM {

  class EPG : public MMBoxState {

  private:
    string rootpath;
    string now;
    string today;

    int currentDay;
    int currentChannel;

    string epgsource;

    ThreadMutex mutex;

    list<ProgramDescription> shows;
    int currentEventIndex;
    time_t currenttime;

    ITVTimer_var itvtimer;

    //For DVBEPG
    /*
    TVManager *tvmanager;
    GraphDescription *gd;
    NodeDescription *null_nd;
    */
  public:

    typedef struct {
      unsigned int  xpos;
      unsigned int  ypos;
      unsigned int  width;
      unsigned int  height;
      unsigned int  win_active_color;
      unsigned int  win_inactive_color;
      unsigned int  sel_active_color;
      unsigned int  sel_active_color2;
      unsigned int  sel_inactive_color;
    } win_properties_t;

    enum wintype_e {DAYS=0, CHANNELS, SCHEDULE, EXTINFO, UPDATE};
    enum weekdays_e {MO=1, DI, MI, DO, FR, SA, SO};
    /**
     * constructor needs a string id that identifies the class
     * \param id the menu id string
     */
    EPG(string, MMBoxApplication*);
    /** Destructor */
    ~EPG();

    Result Up();
    Result Down();
    Result Left();
    Result Right();
    Result Return();
    Result Back();
    Result specialKey01();
    Result specialKey02();
    Result colorKeyRed();
    Result colorKeyGreen();
    Result colorKeyYellow();
    Result colorKeyBlue();
    Result Record();
    Result doCreate(string);
    Result doKill();
    Result doInitialize(string);
    Result doUninitialize();

    Result showOSD();
    Result hideOSD();

    Result newChannel();
    Result newShow(string& channelID);
    
    void setWinProperties( int, win_properties_t* );

    ITVGuide *itvgserver;

  protected:

    IOSDManager* manager;
    
    /** update and refresh OSD */
    void updateOSD();

    /** window with input focus */
    wintype_e win_focus;

     /** ListUnit for the three windows */
    TableUnit* epgWins[4];

    /** properties of the windows */
    win_properties_t  winprops[4];

    /** delete crated windows */
    void deleteWindows();
    /** create windows for the guide */
    void createWindow(wintype_e w, string h);

    /** select the window with ID id*/
    void selectWindow(int id);

    /** create initial content of the windows */
    void createContent();
    /** update the content with respect to the selection */
    void updateContent();

    /** converts the time-string from the XML-File to a more
        readable format */
    string convertTime(string time, bool longtime = false);
    /** Updates The content of the ChannelWindow */
    void updateChannels();
    /** updates the content of the Show-Window according to selection */
    void updateShows();
    /** updates the content of the Extended-Information-Window
        according to the selectins */
    void updateExtInfo();

    TEDObject0<EPG> newChannel_listener;
    TEDObject1<EPG, string> newShow_listener;
  private:
    TCPAddress parseHostString(const string s);

    /** returns a string wich contains the date, wich differs by div days from the specified date 
        format of the input-string must be yyyymmdd */
    string getDate(string date, int div);
    
  };
}

#endif //MMBOX_EPG_HPP
