/* NMM - Network-Integrated Multimedia Middleware
 *
 * Copyright (C) 2002-2005
 *                    NMM work group
 *                    Computer Graphics Lab,
 *                    Saarland University, Germany
 *                    http://www.networkmultimedia.org
 *
 * Maintainer:        Christoph Wellner <chwellner@graphics.cs.uni-sb.de>
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or (at your option) any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307
 * USA
 */

#ifndef MMBOX_EPGSEARCH_HPP
#define MMBOX_EPGSEARCH_HPP MMBOX_EPGSEARCH_HPP

#include <string>
#include <time.h>

// the app
#include "../../main/application.hpp"
#include "../../main/mmboxstate.hpp"

// stored Interfaces
#include "nmm/interfaces/video/osd/manager/ITextView.hpp"
#include "nmm/interfaces/misc/tvguide/ITVGuide.hpp"

#include "nmm/utils/DirReader.hpp"
#include "nmm/plugins/video/osd/manager/OSDElements/listunit.hpp"
#include "nmm/plugins/video/osd/manager/OSDElements/textinput.hpp"
#include "nmm/utils/thread/ThreadedObject.hpp"

#include "nmm/misc/tvguide/TVGuide.hpp"

namespace NMM {

  class EPGSearch : public MMBoxState {

  private:

    list<ProgramDescription> shows;
    map<string, ProgramFilter> bookmarkList;

    ThreadMutex mutex;

    void initializeWindows();
    void updateExtResult();
    void readBookmarks();
    void saveBookmarks();
    void addBookmark(string name);

    void fillFromBookmarks(int i);
    string formatTimeString(string s);
    string formatTimeSearchString(string s);
    string dateInput(int w, string s);
    bool lineMarked(list<unsigned int> l, unsigned int i);

    void transform(string &s){
      for (unsigned int i = 0; i <= s.length(); i++)
        s[i] = toupper(s[i]);
    }

  public:


    typedef struct {
      unsigned int  xpos;
      unsigned int  ypos;
      unsigned int  width;
      unsigned int  height;
      unsigned int  win_active_color;
      unsigned int  win_inactive_color;
      unsigned int  sel_active_color;
      unsigned int  sel_active_color2;
      unsigned int  sel_inactive_color;
    } win_properties_t;

    enum wintype_e {BOOKMARKS=0, STARTTIME, ENDTIME, PHRASE, CHANNEL, CATINPUT, CATEGORY, BOOKMARKNAME};
    /**
     * constructor needs a string id that identifies the class
     * \param id the menu id string
     */
    EPGSearch(string, MMBoxApplication*);
    /** Destructor */
    ~EPGSearch();

    Result Up();
    Result Down();
    Result Left();
    Result Right();
    Result Return();
    Result Back();
    Result Record();
    Result specialKey01();
    Result specialKey02();
    Result numKey0();
    Result numKey1();
    Result numKey2();
    Result numKey3();
    Result numKey4();
    Result numKey5();
    Result numKey6();
    Result numKey7();
    Result numKey8();
    Result numKey9();
    Result Rewind();

    Result colorKeyRed();
    Result colorKeyGreen();
    Result colorKeyYellow();
    Result colorKeyBlue();

    Result doCreate(string);
    Result doKill();
    Result doInitialize(string);
    Result doUninitialize();

    Result showFilterOSD();
    Result hideFilterOSD();
    Result showResultOSD();

    Result newChannel();

    void setWinProperties( int, win_properties_t* );

  protected:
    ITVGuide* itvgserver;

    IOSDManager* manager;
    /* Windos for Filter-Mask */
    ListUnit* bookmarks;
    TextInput* starttime;
    TextInput* endtime;
    TextInput* phrase;
    ListUnit* channel;
    TextInput* categoryInput;
    ListUnit* category;

    /* for entering bookmark-name */
    TextInput* bookmarkName;
    
    /** window with input focus */
    wintype_e win_focus;

    TEDObject0<EPGSearch> newChannel_listener;

    /** properties of the windows */
    win_properties_t  winprops[8];

    /** update and refresh OSD */
    void updateOSD();

    /** delete created windows */
    void deleteWindows();
    /** create windows for the guide */
    void createWindow(wintype_e w, string h);

  };
}

#endif //MMBOX_EPGSEARCH_HPP
