/* NMM - Network-Integrated Multimedia Middleware
 *
 * Copyright (C) 2002, 2003
 *                    Network-Multimedia Workgroup,
 *                    Computer Graphics Lab,
 *                    Saarland University, Germany
 *                    http://www.networkmultimedia.org
 *
 * Maintainer:        David Maass (dama@graphics.cs.uni-sb.de) 
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or (at your option) any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307
 * USA
 */

#include "JobManager.hpp"
#include "TranscodingJob.hpp"
#include "TranscodingGroup.hpp"
namespace NMM {

  JobManager::JobManager(TranscodeTemplateFunctions* s) :condition(mutex),isrunning(true),state(s),
							 last_update(),
							 second(1,0)
  {
    activateThread();
  }

  JobManager::~JobManager() {
    mutex.lock();
    isrunning=false;
    condition.notify();
    mutex.unlock();
    deactivateThread();
  }

  void JobManager::update() {
    updatemutex.lock();
    NMM::Time temp = clock.getTime();
    //just update once a second - redrawing the complete osd is expensive on not so fast systems
    if (temp-last_update>second) {
      state->updateInfo();
      last_update=temp;
    }
    updatemutex.unlock();
  }

  void JobManager::stopJob(TranscodingJob* job,TranscodingGroup* group) {
    mutex.lock();
    stop_queue.push(pair<TranscodingJob*,TranscodingGroup*>(job,group));
    condition.notify();
    mutex.unlock();
  }

  void JobManager::run() {
    while(isrunning) {
      mutex.lock();
      while (!stop_queue.empty()) {
        (stop_queue.front().first)->eof(); 
	(stop_queue.front().second)->jobDone();
        (stop_queue.front().second)->processJobs();
        
	update();
        stop_queue.pop();
      }
      condition.wait();
      mutex.unlock();
    }
  }
}
