/* NMM - Network-Integrated Multimedia Middleware
 *
 * Copyright (C) 2002, 2003
 *                    Network-Multimedia Workgroup,
 *                    Computer Graphics Lab,
 *                    Saarland University, Germany
 *                    http://www.networkmultimedia.org
 *
 * Maintainer:        David Maass (dama@graphics.cs.uni-sb.de) 
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or (at your option) any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307
 * USA
 */
#ifndef __JOBMANAGER__
#define __JOBMANAGER__

#include <queue>

#include "apps/mmbox/main/TranscodeTemplateFunctions.hpp"
#include "nmm/utils/thread/ThreadedObject.hpp"
#include "nmm/base/sync/Clock.hpp"

namespace NMM {
  class TranscodingJob;
  class TranscodingGroup;
  class JobManager : public ThreadedObject {

  private:
    queue<pair<TranscodingJob*,TranscodingGroup*> > stop_queue;
    ThreadMutex mutex;
    ThreadMutex updatemutex;
    ThreadCondition condition;
    bool isrunning;
    TranscodeTemplateFunctions* state;
    Clock clock;
    NMM::Time last_update;
    Interval second;


  public:
    JobManager(TranscodeTemplateFunctions*);
    ~JobManager();

    void stopJob(TranscodingJob*,TranscodingGroup*);
    void update();

  protected:
    virtual void run();
  };
}

#endif
