/* NMM - Network-Integrated Multimedia Middleware
 *
 * Copyright (C) 2002-2005
 *                    NMM work group,
 *                    Computer Graphics Lab,
 *                    Saarland University, Germany
 *                    http://www.networkmultimedia.org
 *
 * Maintainer:        David Maass <dama@graphics.cs.uni-sb.de>
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or (at your option) any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307
 * USA
 */

#ifndef _NMM_WINDOWSELECTORPLAYLIST_HPP
#define _NMM_WINDOWSELECTORPLAYLIST_HPP

#include "nmm/plugins/video/osd/manager/OSDElements/OSDElementComposit.hpp"
#include "nmm/plugins/video/osd/manager/OSDElements/listunit.hpp"

#include "nmm/interfaces/base/INode.hpp"
#include "nmm/base/graph/CompositeNode.hpp"
#include "../main/PlaylistTemplateFunctions.hpp"
#include "../main/application.hpp"

#include "nmm/utils/DirReader.hpp"
#include "nmm/base/utils/GraphURL.hpp"

#include <stack>

namespace NMM {
  class WindowSelectorPlaylist :
    public OSDElementComposit {

  public:
    WindowSelectorPlaylist(IOSDManager* manager, IWidgetFactory* factory, PlaylistTemplateFunctions* state);
    ~WindowSelectorPlaylist();

    virtual Result selectLeft();
    virtual Result selectRight();
    virtual Result selectUp();
    virtual Result selectDown();
    virtual Result select();

    virtual Result colorKeyRed();
    virtual Result colorKeyGreen();
    virtual Result colorKeyYellow();
    virtual Result colorKeyBlue();

    void setRootpath(string);
    void setCDRomPath(string path) {cdrompath=path;}
    void setApplication(MMBoxApplication*);
    void refreshFiles();

    void setWinProperties(int nr, win_properties_t* prop);
    virtual void createWindows();
    void deleteWindows();
    void selectWindow(int winid);
    void getAllPlaylistFiles(vector<GraphURL>& content);
    void getAllPlaylistNames(vector<string>&);
    string getPreviousPlaylistfile();
    string getPreviousPlaylistName();
    string getNextPlaylistfile();
    string getNextPlaylistName();
    string getActualFile();
    void displaySymbol(string sym);
    void addprefix(bool);
    void userplaylistfile(string plist) {playlistfile=plist;}
    virtual void updateColorBar();

  protected:
    void   dirToText();
    string getSimpleName(string name);
    bool   loadPlayList(string liste);
    void   savePlayList(string liste);
    void   add_all_files(string dirname);

    
    win_properties_t  winprops[3];
    ListUnit* playlist;
    ListUnit* track;
    ListUnit* selection;
    vector<DirReader::directory_t> dir;

    DirReader dirreader;
    CompositeNode* cn;
    INode* cn_node;
    PlaylistTemplateFunctions* state;
    MMBoxApplication* app;
    stack<int> dir_position;
    bool useprefix;
    bool mounted;
    string prefix;
    string rootpath;
    string cdrompath;
    string actual_file;
    string playlistfile;
    enum pmode {NORMAL=0, EDIT} playlist_mode;
  };
}

#endif
