/* NMM - Network-Integrated Multimedia Middleware
 *
 * Copyright (C) 2002-2005
 *                    NMM work group,
 *                    Computer Graphics Lab,
 *                    Saarland University, Germany
 *                    http://www.networkmultimedia.org
 *
 * Maintainer:         
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or (at your option) any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307
 * USA
 */


#include "application.hpp"
#include "nmm/base/NMMApplication.hpp"

namespace NMM {

Result MMBoxState::Up()                    { return( SUCCESS ); };
Result MMBoxState::Down()                  { return( SUCCESS ); };
Result MMBoxState::Left()                  { return( SUCCESS ); };
Result MMBoxState::Right()                 { return( SUCCESS ); };
Result MMBoxState::Return()                { return( SUCCESS ); };
Result MMBoxState::Back()                  { return( SUCCESS ); };
Result MMBoxState::receiveMessage( string ){ return( SUCCESS ); };
Result MMBoxState::doInitialize( string )  { return( SUCCESS ); };
Result MMBoxState::doUninitialize()        { return( SUCCESS ); };
Result MMBoxState::doCreate( string )      { return( SUCCESS ); };
Result MMBoxState::doKill()                { return( SUCCESS ); };
Result MMBoxState::Pause()                 { return( SUCCESS ); };
Result MMBoxState::Rewind()                { return( SUCCESS ); };
Result MMBoxState::Forward()               { return( SUCCESS ); };
Result MMBoxState::Record()                { return( SUCCESS ); };
Result MMBoxState::Stop()                  { return( SUCCESS ); };
Result MMBoxState::OSDKey()                  { return( SUCCESS ); };
Result MMBoxState::specialKey01()          { return( SUCCESS ); };
Result MMBoxState::specialKey02()          { return( SUCCESS ); };
Result MMBoxState::colorKeyRed()           { return( SUCCESS ); };
Result MMBoxState::colorKeyGreen()         { return( SUCCESS ); };
Result MMBoxState::colorKeyYellow()        { return( SUCCESS ); };
Result MMBoxState::colorKeyBlue()          { return( SUCCESS ); };
Result MMBoxState::numKey0()               { return( SUCCESS ); };
Result MMBoxState::numKey1()               { return( SUCCESS ); };
Result MMBoxState::numKey2()               { return( SUCCESS ); };
Result MMBoxState::numKey3()               { return( SUCCESS ); };
Result MMBoxState::numKey4()               { return( SUCCESS ); };
Result MMBoxState::numKey5()               { return( SUCCESS ); };
Result MMBoxState::numKey6()               { return( SUCCESS ); };
Result MMBoxState::numKey7()               { return( SUCCESS ); };
Result MMBoxState::numKey8()               { return( SUCCESS ); };
Result MMBoxState::numKey9()               { return( SUCCESS ); };
Result MMBoxState::REVEAL()                { return( SUCCESS ); };

MMBoxState::MMBoxState( string id, MMBoxApplication* _app ) {
  /* save the id */
  stateid  = id;
  app=_app;

  setLazyBackground( false );
  setNeedAudio( false );

  setBackgroundFree( true );
  setAudioFree( true );

  setFocus( false );
}

MMBoxState::~MMBoxState() {
  
}

void MMBoxState::setBackground( string b ) {
  background = b;
}

string MMBoxState::getBackground() {
  return( background );
}


void MMBoxState::setLazyBackground( bool s ) {
  lazy_background = s;

  //cerr << getID() << ": lazy background=" << s << endl;
}

void MMBoxState::setNeedVideo( bool s ) {
  lazy_background = !s;
}

void MMBoxState::setNeedAudio( bool s ) {
  need_audio = s;
}

bool MMBoxState::isLazyBackground() {
  return( lazy_background );
}

bool MMBoxState::isNeedAudio() {
  return( need_audio );
}

bool MMBoxState::isNeedVideo() {
  return( ! lazy_background );
}

void MMBoxState::setBackgroundFree( bool b ) {
  background_free = b;
}

void MMBoxState::setAudioFree( bool b ) {
  audio_free = b;
}

bool MMBoxState::isBackgroundFree() {
  return( background_free );
}

bool MMBoxState::isAudioFree() {
  return( audio_free );
}

string MMBoxState::getID() {
  return( stateid );
}

bool MMBoxState::hasFocus() {
  return( has_focus );
}

void MMBoxState::setFocus( bool b ) {
  has_focus = b;
}

Result MMBoxState::initialize( string message ) {

  /**
  * save the last state id, but do not overwrite the existing one
  * this is why the initialize method is called if a child state
  * switches to the last state and if we overwrite the last state id,
  * we can't switch to the last state anymore
  */
  if ( lastid == "" ) lastid = app->last_stateid;

  /* set last_stateid for the following child states */
  app->last_stateid = stateid;

  setFocus( true );

  if ( doInitialize( message ) == SUCCESS ) {

    return( SUCCESS );
  } else {

    /* here we should call the initialize method from the previous state */
    return( FAILURE );
  }

};

Result MMBoxState::uninitialize() {
  setFocus( false );
  return( doUninitialize() );
};

void MMBoxState::deactivate() {
  GLOBAL_DEBUG_STREAM("MMBoxState: Deleting current state from StateList." << endl);
  app->delFromActiveStates( this );
  GLOBAL_DEBUG_STREAM("MMBoxState: Deleted current state from StateList." << endl);
}

Result MMBoxState::create( string message ) {

  /**
  * save the last state id, but do not overwrite the existing one
  * this is why the initialize method is called if a child state
  * switches to the last state and if we overwrite the last state id,
  * we can't switch to the last state anymore
  */
  if ( lastid == "" ) lastid = app->last_stateid;

  /* set last_stateid for the following child states */
  app->last_stateid = stateid;

  setFocus( true );

  app->addToActiveStates( this );
  if ( doCreate( message ) == SUCCESS ) {

    return( SUCCESS );
  } else {
    GLOBAL_DEBUG_STREAM("Deleting " << getID() << " from active States!" << endl);
    app->delFromActiveStates( this );
    GLOBAL_DEBUG_STREAM("Deleted " << getID() << " from active States!" << endl);

    /* here we should call the initialize method from the previous state */
    return( FAILURE );
  }

};

Result MMBoxState::kill() 
{
  // release all additionally created shared sessions running on remote hosts that were created using the task mgr state
  for(  SharedStateMap_t::iterator i = shared_states.begin();
	i != shared_states.end();
	++i)
  {
    try {
      GraphDescription& graph = (*i).second.first;
      graph.disconnectSinkSynchronizer();
      app->getNMMApplication()->getRegistry().releaseGraph(graph);
      IMMBoxApplication* remote_mmbox_app = (*i).second.second;
      if(remote_mmbox_app) {
	remote_mmbox_app->deleteDistState("DistTaskState");
      }
    } catch(const Exception& e){
      GLOBAL_ERROR_STREAM("Exception while releasing graph" << std::endl);
      GLOBAL_ERROR_STREAM(e << endl);
    }
  }
  shared_states.clear();


  app->delFromActiveStates( this );
  setFocus( false );
  GLOBAL_DEBUG_STREAM("Returning doKill()" << endl);
  return( doKill() );
};

Result MMBoxState::goToLastState( string message ) {
  if ( lastid != "" ) {
    if ( changeState( lastid, message ) == FAILURE ) {
      GLOBAL_DEBUG_STREAM( "id: " << lastid << " not found" << endl );
      return( FAILURE );
    }
    /* now we left the state and we have got no lastid anymore */
    lastid = "";
    return( SUCCESS );
  } else {
    return( FAILURE );
  }
}

Result MMBoxState::changeState( string stateid, string message ) {
  return( app->changeState( stateid, message ) );
};

  const GraphDescription& MMBoxState::getCurrentGraphDescription() const
  {
    return current_graph_description;
  }
  
  void MMBoxState::setCurrentGraphDescription(const GraphDescription& current_graph_description, string session)
  {
    this->current_graph_description = current_graph_description;
    this->session_location = session;
  }
  
  bool MMBoxState::hasCurrentGraphDescription()
  {
    if(current_graph_description.getNodeDescriptionList().size()) {
      return true;
    }
    else {
      return false;
    }
  }
  
  bool MMBoxState::hasSharedState(const string& host, const string& state) const 
  {
    pair<string, string> key = make_pair(host, state);
    SharedStateMap_t::const_iterator found = shared_states.find(key);
    
    return (found != shared_states.end());
  }
  
  GraphDescription MMBoxState::getSharedGraphDescription(const string& host, const string& state) const
  {
    pair<string, string> key = make_pair(host, state);
    SharedStateMap_t::const_iterator found = shared_states.find(key);

    if(found != shared_states.end()) {
      return ( (*found).second.first );
    }
    else {
      GraphDescription graph;
      return graph;
    }
  }

  void MMBoxState::addSharedState(const string& host, const string& state,
				  const GraphDescription& graph, IMMBoxApplication* dist_mmbox /* = 0 */ )
  {
    pair<string, string>                       key   = make_pair(host, state);
    pair<GraphDescription, IMMBoxApplication*> value = make_pair(graph, dist_mmbox);
    shared_states.insert( make_pair(key, value) );
  }
  
  Result MMBoxState::releaseSharedState(const string& host, const string& state)
  {
    pair<string, string> key = make_pair(host, state);
    SharedStateMap_t::iterator i = shared_states.find(key);

    if(i != shared_states.end() ) {
      try {
	GraphDescription& graph = (*i).second.first;
	graph.disconnectSinkSynchronizer();
	app->getNMMApplication()->getRegistry().releaseGraph(graph);

	IMMBoxApplication* remote_mmbox_app = (*i).second.second;
	if(remote_mmbox_app) {
	  remote_mmbox_app->deleteDistState("DistTaskState");
	}
      } catch(const Exception& e){
	GLOBAL_ERROR_STREAM("Exception while releasing graph" << std::endl);
	GLOBAL_ERROR_STREAM(e << endl);
      }
      shared_states.erase(i);

      GLOBAL_DEBUG_STREAM("releaseSharedGraphDescription: " << host << " " << state << " : successful!" << endl);
      return SUCCESS;
    }
    else {
      GLOBAL_WARNING_STREAM("releaseSharedGraphDescription: " << host << " " << state << " : could find graph description" << endl);
      return FAILURE;
    }
  }

  string MMBoxState::getSessionLocation()
  {
    return this->session_location;
  }

  
}
