/* NMM - Network-Integrated Multimedia Middleware
 *
 * Copyright (C) 2002-2005
 *                    NMM work group,
 *                    Computer Graphics Lab,
 *                    Saarland University, Germany
 *                    http://www.networkmultimedia.org
 *
 * Maintainer:        Michael Repplinger <repplinger@cs.uni-sb.de>
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version 2
 * of the License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307
 * USA
 */

#ifndef QGRAPHLIST_H
#define QGRAPHLIST_H

#include <qwidget.h>
#include <list>
#include <map>

using namespace std;

/**
  *@author Michael Repplinger
  */

namespace NMM {
  class INode;
}

class QListView;
class QGraphWidget;
class QGridLayout;
class QListViewItem;
class QPopupMenu;

class QGraphList : public QWidget  {
   Q_OBJECT
public: 
	QGraphList(QWidget *parent=0, const char *name=0, const std::list<QString>& = list<QString>());
  ~QGraphList();

  void addNodeGraph(QGraphWidget* widget) { __graph_list.push_back(widget); updateView(); }

  const std::list<QGraphWidget*>& getWidgetList() const { return __graph_list; }

signals:
  void clicked(QGraphWidget* );
  void clicked(NMM::INode* );

protected:
        std::list<QGraphWidget*>& getWidgetList()       { return __graph_list; }
  void  updateView();
  void  updateGraphWidget(QListViewItem* item, QGraphWidget* widget);
private:
  void  allocate();
  void  deallocate();
  void  initConnect();
private slots:
  void  showGraphMenu(QListViewItem*, const QPoint &, int);
  void  emitClicked(QListViewItem*);
private:
  typedef std::list<QGraphWidget* >::iterator InternalIt;
  typedef std::list<QGraphWidget* >::const_iterator CInternalIt;

  QGridLayout* __top_layout;
  QListView*   __list_view;
  QPopupMenu*  __popup_menu;
  std::list<QString>  __header_list;
  std::list< QGraphWidget*> __graph_list;
  
  std::map<QListViewItem*, NMM::INode* > __node_map;
  std::map<QListViewItem*, QGraphWidget* > __graph_map;
};

#endif
