/* NMM - Network-Integrated Multimedia Middleware
 *
 * Copyright (C) 2002-2005
 *                    NMM work group,
 *                    Computer Graphics Lab,
 *                    Saarland University, Germany
 *                    http://www.networkmultimedia.org
 *
 * Maintainer:        Michael Repplinger <repplinger@cs.uni-sb.de>
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version 2
 * of the License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307
 * USA
 */

#include "qgraphwidget.h"
#include <qmessagebox.h>
#include "nmm/interfaces/base/INode.hpp"

QGraphWidget::QGraphWidget(const list<QString>& header_list)
  : __header_list(header_list)
{

}

QGraphWidget::~QGraphWidget(){

}


void QGraphWidget::init() {
  try {
    for (InternalIt it = getNodeList().begin(); it != getNodeList().end(); ++it) {
      (*it)->init();
    }
  } catch (const NMM::Exception& exception) {
//    QMessageBox::critical(this, "Init NodeGraph", exception.getComment().c_str());
  }
}

void QGraphWidget::deinit() {
  try {
    for (InternalIt it = getNodeList().begin(); it != getNodeList().end(); ++it) {
      (*it)->deinit();
    }
  } catch (const NMM::Exception& exception) {
//    QMessageBox::critical(this, "Deinit NodeGraph", exception.getComment().c_str());
  }
}

void QGraphWidget::activate() {
  try {
    for (InternalIt it = getNodeList().begin(); it != getNodeList().end(); ++it) {
      (*it)->activate();
    }
  } catch (const NMM::Exception& exception) {
//    QMessageBox::critical(this, "activate NodeGraph", exception.getComment().c_str());
  }
}

void QGraphWidget::deactivate(){
  try {
    for (InternalIt it = getNodeList().begin(); it != getNodeList().end(); ++it) {
      (*it)->deactivate();
    }
  } catch (const NMM::Exception& exception) {
//    QMessageBox::critical(this, "Deactivate NodeGraph", exception.getComment().c_str());
  }
}

void QGraphWidget::start(){
  try {
    for (InternalIt it = getNodeList().begin(); it != getNodeList().end(); ++it) {
      (*it)->start();
    }
  } catch (const NMM::Exception& exception) {
//    QMessageBox::critical(this, "start NodeGraph", exception.getComment().c_str());
  }
}

void QGraphWidget::stop(){
  try {
    for (InternalIt it = getNodeList().begin(); it != getNodeList().end(); ++it) {
      (*it)->stop();
    }
  } catch (const NMM::Exception& exception) {
//    QMessageBox::critical(this, "stop NodeGraph", exception.getComment().c_str());
  }
}

//void QGraphWidget::connect() {
//  try {
//    for (InternalIt it = getNodeList().begin(); it != getNodeList().end(); ++it) {
//      InternalIt next_it = ++it;
//      if (next_it == getNodeList.end())
//        break;
//
//      (*connect(it, **next_it));
//    }
//  } catch (const Exception& exception) {
//    QMessageBox::critical(this, "stop NodeGraph", exception.getComment().c_str());
//  }
//}
//
//void QGraphWidget::disconnect() 
