/*
 * Qwt Widget Library (http://qwt.sf.net)
 * Copyright (C) 1997   Josef Wilgen
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version 2
 * of the License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, 
 * USA.
 */


#ifndef QWT_SCLDRAW_H
#define QWT_SCLDRAW_H

#include <qstring.h>
#include <qwt_dimap.h>
#include <qwt_scldiv.h>
#include <qfont.h>
#include <qrect.h>
#include <qcolor.h>

class QPainter;
class QwtAutoScale;

class QwtScaleDraw
: public QwtDiMap
{
	
public:
	enum Orientation { Bottom, Top, Left, Right, Round };

private:

	QwtScaleDiv d_scldiv;
	
 protected:
	
	static const int minLen;
	
	Orientation d_orient;
	
	int d_xorg;
	int d_yorg;
	int d_len;
	
	int d_hpad;
	int d_vpad;
	
	int d_medLen;
	int d_majLen;
	int d_minLen;

	int d_minAngle;
	int d_maxAngle;

	char d_fmt;
	int d_prec;
	
 private:

	void drawTick(QPainter *p, double val, int len) const;
	void drawBackbone(QPainter *p) const;
	void drawLabel(QPainter *p, double val) const;
	void calcTicks();
	
 public:

	// *STRUCTORS
	QwtScaleDraw();
  virtual ~QwtScaleDraw(){}
	// MANIPULATORS

	void setScale(double vmin, double vmax, int maxMajIntv, int maxMajIntv,
		      double step = 0.0, int logarithmic = 0);
	void setScale(const QwtScaleDiv &s);
	void setGeometry(int xorigin, int yorigin, int length, Orientation o);
	void setAngleRange(double angle1, double angle2);
	void setLabelFormat(char f, int prec);
  
	const QwtScaleDiv& scalediv() const { return d_scldiv; }

	int orientation() const { return d_orient; }
	
	
	QRect maxBoundingRect(QPainter *p) const;
	int maxWidth(QPainter *p, bool worst = TRUE) const;
	int maxHeight(QPainter *p) const;
	int maxLabelWidth(QPainter *p, int worst) const;
	const QwtScaleDiv& scaleDiv() const;
	virtual void draw(QPainter *p) const;

};


//------------------------------------------------------------
//
//.S	QwtScaleDraw::scaleDiv
//
//
//.s	Parameters
//.p	
//
//.s	Return Type
//		const QwtScaleDiv &
//
//.s	Return Value
//
//.s	Description
//
//.s	Syntax
//.f	const QwtScaleDiv & QwtScaleDraw::scaleDiv()
//
//------------------------------------------------------------
inline const QwtScaleDiv & QwtScaleDraw::scaleDiv() const
{
    return d_scldiv;
}
#endif







