/*
 * Qwt Widget Library (http://qwt.sf.net)
 * Copyright (C) 1997   Josef Wilgen
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version 2
 * of the License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, 
 * USA.
 */



#ifndef QWT_SLDBASE_H
#define QWT_SLDBASE_H

#include <qwt_drange.h>
#include <qframe.h> 
#include <qpixmap.h>
#include <qdatetime.h>

class QwtSliderBase : public QWidget, public QwtDblRange
{

	Q_OBJECT 

 public:

	enum { ScrNone, ScrMouse, ScrTimer, ScrDirect };
	
 protected:

	int d_scrollMode;
	double d_mouseOffset;
	int d_direction;
	int d_tracking;
	
 private:
	
	int d_tmrID;
	int d_updTime;
	int d_timerTick;
	QTime d_time;
	double d_speed;
	double d_mass;

	void buttonReleased();

 public:
	
	QwtSliderBase( QWidget *parent = 0, const char *name = 0 );
	virtual ~QwtSliderBase();

	void setUpdateTime(int t);
//	void incValue(double nSteps);
	void stopMoving();
	void setTracking(bool enable);
	
  protected:
	
	virtual void setMass(double val);
	void setPosition(const QPoint &p);
	virtual void valueChange();
	virtual double mass() const;

	void timerEvent(QTimerEvent *e);
	void mousePressEvent(QMouseEvent *e);
	void mouseReleaseEvent(QMouseEvent *e);
	void mouseMoveEvent(QMouseEvent *e);
	   
 public slots:

	void moveTo(int val);
	void adjustTo(int val);
	
 signals:

	void valueChanged(int);
	void sliderPressed();
	void sliderReleased();
	void sliderMoved(int);

 protected:

	virtual double getValue(const QPoint & p) = 0;
	virtual void getScrollMode( const QPoint &p,
				   int &scrollMode, int &direction) = 0;

};


inline double QwtSliderBase::mass() const
{
	return d_mass;
}

#endif












