/* NMM - Network-Integrated Multimedia Middleware
 *
 * Copyright (C) 2002-2005
 *                    NMM work group,
 *                    Computer Graphics Lab,
 *                    Saarland University, Germany
 *                    http://www.networkmultimedia.org
 *
 * Maintainer:        NMM work group,
 *                    http://www.networkmultimedia.org
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version 2
 * of the License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307
 * USA
 */
#include <iostream>
#include "nmm/base/connect.hpp"
#include "nmm/base/ProxyObject.hpp"

// include used interfaces
#include "nmm/interfaces/video/display/IVideoDisplay.hpp"
#include "nmm/interfaces/base/sync/ISynchronizedSink.hpp"

// include the registry
#include "nmm/base/ProxyApplication.hpp"
#include "nmm/base/registry/ClientRegistry.hpp"
#include "nmm/interfaces/base/registry/IServerRegistry.hpp"
#include "nmm/utils/utils.hpp"

/* Besipiel eines Eventhandlers*/

using namespace NMM;


int main(int argc, char **argv)
{
  bool rgb;
  bool noise;
  bool gl;
  bool big;
  bool back;



  if (argc <= 1) {
    GLOBAL_ERROR_STREAM( "usage: cameratest rgb|yuv [noise] [gl] [big]" << endl);
    exit(1);
  }



  rgb = true; noise = false; gl = false; big = false; back = false;
  for (int i=1; i<argc; i++) {
    if(!strcmp(argv[i],"yuv")) rgb = false;
    else if(!strcmp(argv[i],"noise")) noise = true;
    else if(!strcmp(argv[i],"gl")) gl = true;
    else if(!strcmp(argv[i],"big")) big = true;
    else if(!strcmp(argv[i],"back")) back = true;
  }

  int x_resolution = 320;
  int y_resolution = 240;
  double framerate = 15.0;
  if (big) {
    x_resolution = 640;
    y_resolution = 480;
    framerate = 15.0;
  }

//  NMM::NamedObject::getGlobalInstance().setWarningStream(NULL, NamedObject::ALL_LEVELS);
  NMM::NamedObject::getGlobalInstance().setMessageStream(NULL, NamedObject::ALL_LEVELS);
  NMM::NamedObject::getGlobalInstance().setDebugStream(NULL, NamedObject::ALL_LEVELS);

  /* Allocate _var variables for the INode's*/
  INode_var camera;
  INode_var converter;
  INode_var display;

  NMMApplication* app = ProxyApplication::getApplication(argc, argv);

  try {

    Format camera_out_format;
    camera_out_format.setFormatType("video/raw");
    camera_out_format.addIntParamValue(Format::x_resolution_param,x_resolution);
    camera_out_format.addIntParamValue(Format::y_resolution_param,y_resolution);
    //camera_out_format.addIntParamValue(Format::ratio_x_param, x_resolution);
    //camera_out_format.addIntParamValue(Format::ratio_y_param, y_resolution);
    //camera_out_format.addStringParamValue(Format::channelorder_param,"uyvy");
    //camera_out_format.addStringParamValue(Format::endian_param, "little");
    //camera_out_format.addStringParamValue(Format::colorspace_param,"yuv422");
    //camera_out_format.addIntParamValue(Format::bitperpixel_param,16);
    //camera_out_format.addStringParamValue(Format::format_param, "packet");
    camera_out_format.addFloatParamValue(Format::framerate_param, framerate);

    Format camera_out_format2;
    camera_out_format2.addIntParamValue(Format::x_resolution_param,x_resolution);
    camera_out_format2.addIntParamValue(Format::y_resolution_param,y_resolution);
    //camera_out_format2.addIntParamValue(Format::ratio_x_param, x_resolution);
    //camera_out_format2.addIntParamValue(Format::ratio_y_param, y_resolution);
    //camera_out_format2.addStringParamValue(Format::colorspace_param,"yuv422");
    camera_out_format2.addFloatParamValue(Format::framerate_param, framerate);

    Format converter_out_format;
    converter_out_format.setFormatType("video/raw");
    converter_out_format.addIntParamValue(Format::x_resolution_param,x_resolution);
    converter_out_format.addIntParamValue(Format::y_resolution_param,y_resolution);
    converter_out_format.addIntParamValue(Format::ratio_x_param, x_resolution);
    converter_out_format.addIntParamValue(Format::ratio_y_param, y_resolution);
    converter_out_format.addStringParamValue(Format::channelorder_param,"rgb");
    converter_out_format.addStringParamValue(Format::endian_param, "little");
    converter_out_format.addStringParamValue(Format::colorspace_param,"rgb24");
    converter_out_format.addIntParamValue(Format::bitperpixel_param,32);
    converter_out_format.addStringParamValue(Format::format_param, "packet");
    converter_out_format.addFloatParamValue(Format::framerate_param, framerate);

    Format converter_out_format2;
    converter_out_format2.addIntParamValue(Format::x_resolution_param,x_resolution);
    converter_out_format2.addIntParamValue(Format::y_resolution_param,y_resolution);
    converter_out_format2.addIntParamValue(Format::ratio_x_param, x_resolution);
    converter_out_format2.addIntParamValue(Format::ratio_y_param, y_resolution);
    converter_out_format2.addStringParamValue(Format::colorspace_param,"rgb24");
    converter_out_format2.addFloatParamValue(Format::framerate_param, framerate);

    /*Begin> Create request for the camera*/
    NodeDescription camera_request;
      /* Add a specific output format */
      //camera_request.addOutputFormat(camera_out_format);
      /* The should be a SOURCE-Node */
      //camera_request.setNodeType(INode::SOURCE);
      /* If no WhiteNoise node is requested */
      if (!noise) {
        //        camera_request.addInterface("ICamera"/*ICamera::ICamera_type*/);
        camera_request.setNodeName("Camera1394");
      } else {
        camera_request.setNodeName("WhiteNoiseNode");
      }
   /*End< */

    NodeDescription converter_request;
    if (rgb) {
      /* add the converters input format */
      converter_request.addInputFormat(camera_out_format);
      /* add the converters output format */
      converter_request.addOutputFormat(converter_out_format);
      /* The node must be of type converter */
      converter_request.setNodeType(INode::CONVERTER);
    }

    /*Begin> Create request for the CameraNode*/
    NodeDescription display_request;
    display_request.setNodeType(INode::SINK);
    display_request.addInterface(IVideoDisplay::IVideoDisplay_type);
    if (rgb)
      display_request.addInputFormat(converter_out_format);
    else
      display_request.addInputFormat(camera_out_format);
    if (gl) {
      display_request.setNodeName("GLDisplayNode");
    } else {
      display_request.setNodeName("XDisplayNode");
    }



    /*Begin> Get the ClientRegistry and request Nodes */
    ClientRegistry& registry = app->getRegistry();

    registry.initTransaction();


    /* Begin > initiliaze a request using the ClientRegistry */

    /* search all registries that can be accessed through the ClientRegistry */
    list<Response> response1 = registry.initRequest(camera_request);
    list<Response> response2;
    if (rgb) {
      /* search a specific registry. In this case it is the "localhost" */
      converter_request.setLocation("localhost");
      response2 = registry.initRequest(converter_request);
    }

    /* For the 3. request we create a list of ServerRegistries that should searched */
    list<RegistryName> registry_list;
    registry_list.push_back(RegistryName(getHostname(), ClientRegistry::default_port));

    /* If we init a request with the list of ServerRegistries, a ResponseMap is returned.
     * A ResponseMap is a typedef to map<string, list<Response> >;
     * For each string the method creates a key that stores the list<Response> */
    ResponseMap resp3 = registry.initRequest(display_request, registry_list);

    /* get the response-list from "localhost" */
    list<Response> response3 = resp3[RegistryName(getHostname(), ClientRegistry::default_port)];


    if (response1.empty() || response3.empty() || (response2.empty() && rgb)) {
      GLOBAL_ERROR_STREAM("Size of 1. response " << response1.size() << endl);
      if (rgb) {
        GLOBAL_ERROR_STREAM("Size of 2. response " << response2.size() << endl);
      }
      GLOBAL_ERROR_STREAM("Size of 3. response " << response3.size() << endl);
      throw Exception("A required node is not available");
    }

    /* End< initiliaze a request using the ClientRegistry */




    /* Begin> request the nodes */
    camera.reset(registry.requestNode(response1.front()));
    if (rgb)
      converter.reset(registry.requestNode(response2.front()));
    display.reset(registry.requestNode(response3.front()));
    registry.releaseTransaction();
    /* End< */


    camera->init();
    if (rgb)
      converter->init();
    display->init();

    camera->initOutput();
    /* Connect the nodes (this returns a CommunicationChannel*), this automatically
     * sets the transport strategy, so you only need the communication channel in
     * case you want to change something lateron
     */
    if (rgb)    {
      cerr << ">Connect camera with converter" << endl;
      connect(camera, converter, &camera_out_format2);
      cerr << "Connect camera with converter" << endl;
    }
    else {
      cerr << ">Connect camera with display" << endl;
      connect(camera, display, &camera_out_format2);
      cerr << "Connect camera with display" << endl;
    }

    camera->activate();

    if (rgb) {
      converter->initOutput();
      connect(converter, display, &converter_out_format2);
      converter->activate();
    }

    display->initOutput();
    display->activate();
    
    ISynchronizedSink_var display_sync(display->getParentObject()->getCheckedInterface<ISynchronizedSink>());
    display_sync->setSynchronized(false);
    
    // ... and finally start them
    camera->start();
    if (rgb)
      converter->start();
    display->start();

    IVideoDisplay_var vdisplay(display->getParentObject()->getCheckedInterface<IVideoDisplay>());

    bool fullscreen = vdisplay->getFullscreen();
    // play until 'q' pressed, 'n' jumps to next track.
    string p = "";
    while(true) {
      cin >> p;
      if(p == "q") {
        break;
      }
      if(p == "f") {
        if (vdisplay->hasFullscreen()) {
          if (!vdisplay->isFullscreenOnly()) {
            fullscreen = !fullscreen;
            vdisplay->setFullscreen(fullscreen);
          } else {
            cerr << "Fullscreen-only display!" << endl;
          }
        } else {
          cerr << "Fullscreen not supported!" << endl;
        }
      }
    }

    // stop nodes, reset and destroy them
    GLOBAL_DEBUG_STREAM("Stopping all Nodes" << endl);
    camera->stop();
    if (converter.get())
      converter->stop();
    display->stop();

    GLOBAL_DEBUG_STREAM("Flushing all Nodes" << endl);

    camera->flush();
    if (converter.get())
      converter->flush();
    display->flush();


    GLOBAL_DEBUG_STREAM("Deactivating all Nodes" << endl);
    camera->deactivate();
    if (rgb)
      converter->deactivate();
    display->deactivate();

    display->deinitOutput();
    if (rgb) {
      disconnect_output(converter);
      converter->deinitOutput();
    }
    disconnect_output(camera);
    camera->deinitOutput();


    GLOBAL_DEBUG_STREAM("Deinitializing all Nodes" << endl);
    camera->deinit();
    if (rgb)
      converter->deinit();
    display->deinit();

    /*Release the reserved nodes*/
    GLOBAL_DEBUG_STREAM("Releasing all Nodes" << endl);
    registry.releaseNode(*camera);
    if (converter.get())
      registry.releaseNode(*converter);
    registry.releaseNode(*display);
    GLOBAL_DEBUG_STREAM("All nodes successfully released" << endl);
  }
  // catch NMM-exception and print it
  catch(const Exception& e) {
    GLOBAL_ERROR_STREAM(e << endl);
    
    /* release the tranaction */
    app->getRegistry().releaseTransaction();

    /*Release the reserved nodes*/
    if (camera.get())
      app->getRegistry().releaseNode(*camera);
    if (converter.get())
      app->getRegistry().releaseNode(*converter);
    if (display.get())
      app->getRegistry().releaseNode(*display);
    return 1;
  }
  // any other exception
  catch(...) {
    cerr << "Something went wrong..." << endl;
    
    /* release the tranaction */
    app->getRegistry().releaseTransaction();

    /*Release the reserved nodes*/
    if (camera.get())
      app->getRegistry().releaseNode(*camera);
    if (converter.get())
      app->getRegistry().releaseNode(*converter);
    if (display.get())
      app->getRegistry().releaseNode(*display);
    
    return 1;
  }

  return 0;
}

