/* NMM - Network-Integrated Multimedia Middleware
 *
 * Copyright (C) 2002-2005
 *                    NMM work group,
 *                    Computer Graphics Lab,
 *                    Saarland University, Germany
 *                    http://www.networkmultimedia.org
 *
 * Maintainer:        
 *                    NMM work group,
 *                    http://www.networkmultimedia.org
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or (at your option) any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 * 
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307
 * USA
 */


#ifndef __DVBAPPLICATION__
#define __DVBAPPLICATION__

#include "nmm/plugins/video/display/X/XDisplayNode.hpp"
#include "nmm/plugins/video/tv/dvb/DVBReadNode.hpp"
#include "nmm/plugins/file/timeshifting/MPEGTimeShiftingNode.hpp"
#include "nmm/plugins/av/mpeg/MPEGDemuxNode.hpp"
#include "nmm/plugins/video/mpegdec/MPEGVideoDecodeNode.hpp"
#include "nmm/plugins/audio/mpegdec/MPEGAudioDecodeNode.hpp"
#include "nmm/plugins/audio/PlaybackNode.hpp"
#include "nmm/base/sync/AudioVideoSynchronizer.hpp"
#include "nmm/plugins/stats/DevNullNode.hpp"

using namespace NMM;

class DvbState {
  public:

    DvbState();
    ~DvbState();

    Result doInitialize();
    Result doUninitialize();

    /* here all used "events" must be entered */
    Result Up();
    Result Down();

    Result Return(); // Return means Play
    Result Left();   // a/v-skew--
    Result Right();  // a/v-skew++

    Result Pause();
    Result Forward();
    Result Rewind();
    Result Stop();  // switch to LIVE-mode  (and clear filebuffer)
    Result Record();

  private:

    /* Application stuff: */

    int  fast_forward_speed;
    bool is_in_raw_mode;
    bool was_in_raw_mode;  // true if PAUSE from LIVE-Mode
    int rewind_speed;
    void tuneChannel(int dist);

    bool use_membuf;
    bool sync;
    bool fullscreen;
    int av_skew; //> a/v delay

      //video nodes
    DVBReadNode*          video_mpeg;
    DVBReadNode*           video_raw;
    MPEGTimeShiftingNode*    ts_node;
    MPEGDemuxNode*             demux;
    MPEGVideoDecodeNode*     mpg2yuv;
    GenericSyncSinkNode*     display;

			//audio nodes
    MPEGAudioDecodeNode*     mpa_dec;
    PlaybackNode*         audio_play;

			// a/v
    AudioVideoSynchronizer* sync_node;
    DevNullNode*            null_node;

		//needed formats
    Format* null_format;
    Format* display_format;
    Format* audio_format;
    Format* video_mpeg_output_format;
    Format* video_raw_output_format;

    void prepareMPEGFlush();
    void flushMPEG();
    void restartMPEG();
    void playMemBuf();
    bool stopYUV();
    bool startYUV();
    void disconnectYUV();
    void connectYUV();
    bool stopMPEG();
    bool startMPEG();
    void disconnectMPEG();
    void connectMPEG();
    void switchMPG2YUV();
    void switchYUV2MPG();
    void setupFormats();
    void setupNodes();
    Result dvbMute(bool mode);
    void dvbLive();
    void disconnectNodes();

};
#endif //__DVBAPPLICATION__
