#include "MyStruct.hpp"
#include "nmm/base/serialize/XMLValueStream.hpp"
#include "nmm/base/serialize/MagicNumberValueStream.hpp"
#include "nmm/base/serialize/StdStream.hpp"
#include "nmm/base/registry/LocalRegistry.hpp"
#include "nmm/base/Event.hpp"
#include <fstream>

using namespace MyNamespace;
using namespace NMM;

void eventExample() {
  CEvent cevent(Event("Test_event_1", new TInValue<int>(5)), Event("Test_event_2", new TInValue<string>("Hello World")) );

  XMLOValueStream xov_1;
  xov_1 << cevent;

  string serialized_1 = xov_1.toString();
  cout << serialized_1 << endl;

  XMLIValueStream xiv(serialized_1);
  CEvent new_cevent;
  xiv >> new_cevent;

  XMLOValueStream xov_2;
  xov_2 << new_cevent;
  string serialized_2 = xov_2.toString();
  cout << serialized_2 << endl;

  if (serialized_1 != serialized_2)
    cerr << "Error in CEevent de/serialization process" << endl;
  else
    cout << "CEvent de/srialization successfully completed" << endl;


}

void myStructExample() {

  MyStruct my_struct1;
  my_struct1.getName() = "my_struct1";
  my_struct1.setCounter(54);

  for (unsigned int i = 0; i < 10; ++i)
    my_struct1.getValueList().push_back(34.0 + i);


  /*Begin> serialize the object */
  XMLOValueStream xov; //create a XML based OValueStream
  xov << my_struct1;   //serialize the object

  string s = xov.toString();
  cout << s << endl;
  /*End< */


  /*Begin> deserialize the object*/
  XMLIValueStream xiv(s); //create a XML based IValueStream
  MyStruct my_struct2;
  xiv >> my_struct2;
  /*End<*/

  /*Begin> compare both objects*/
  if (my_struct1.getName() != my_struct2.getName()) {
    cerr << "deserialization failed" << endl;
    return ;
  }

  if (my_struct1.getCounter() != my_struct2.getCounter()) {
    cerr << "deserialization failed" << endl;
    return ;
  }

  if (my_struct1.getValueList() != my_struct2.getValueList()) {
    cerr << "deserialization failed" << endl;
    return ;
  }
  /*End<*/

  cout << "De/serialisation successfully completed" << endl;
}

int main(int, char**)
{
  try {
    CEvent cevent(Event("Test_event_1", new TInValue<int>(5)), Event("Test_event_2", new TInValue<string>("Hello World")) );
    Timestamp t;
    t.sync_time = Time(233, 32);
    t.stream_counter = 400;
    cevent.setTimestamp(t);

    int i = 0;
    while ( i < 10000) {
      strstream w_stream;
      StdOStream stream(&w_stream);

      MagicNumberOValueStream ov;
      ov.setOPartnerStream(&stream);

      ov << cevent;
      ov.writeData();

      MagicNumberIValueStream miv;
      StdIStream i_s(&w_stream);
      miv.setIPartnerStream(&i_s);
      miv.readData();

      CEvent ce;
      ce.createFromStream(miv);
      ++i;
      cerr << ".";
    }
  } catch (const Exception& e) {
    cerr << e;
  }
//  SLocalRegistry::getInstance();
//  eventExample();
//  myStructExample();
//  return 1;
}
