#include "nmm/base/serialize/XMLValueStream.hpp"
#include "nmm/base/serialize/MagicNumberValueStream.hpp"
#include "nmm/base/serialize/StdStream.hpp"
#include "nmm/base/registry/LocalRegistry.hpp"
#include "nmm/base/CompositeStrategy.hpp"
#include "nmm/base/Event.hpp"
//#include "nmm/base/NMMApplication.hpp"
#include "nmm/base/ProxyApplication.hpp"
#include "nmm/base/memorymgr/Buffer.hpp"
#include "nmm/base/proxy/tcp/TCPStrategy.hpp"
#include "nmm/base/proxy/tcp/TCPAcceptor.hpp"
#include "nmm/base/serialize/net/BufferedNetStream.hpp"
#include "nmm/base/serialize/net/UnbufferedNetStream.hpp"
#include "nmm/base/ProxyObject.hpp"
#include "nmm/base/memorymgr/IExternalBufferManager.hpp"
#include "nmm/interfaces/base/INamed.hpp"
#include "nmm/base/serialize/MagicNumberValueStream.hpp"
#include "nmm/plugins/stats/RawNode.hpp"
#include "nmm/base/connect.hpp"
#include "nmm/interfaces/stats/IBufferCounter.hpp"
#include "nmm/interfaces/stats/IMethodCall.hpp"
#include "nmm/interfaces/stats/IMethodCallImpl.hpp"
#include "nmm/utils/net/SockStream.hpp"
#include "nmm/base/ConnectionManager.hpp"
#include "nmm/base/ConnectorAddress.hpp"
#include "nmm/base/registry/ReferenceManager.hpp"
#include <sys/time.h>
#include <fstream>
#include <getopt.h>
#include <cassert>

using namespace NMM;

static const int second = 1000000;

string host;
string host2;
int port = ClientRegistry::default_port;
unsigned int iteration = 1000;
int benchmark = 0;
int buffer_size = 1024;
unsigned int loss_interval = 0;
unsigned int jitter_samples = 0;
unsigned int jitter_first_sample = 100;
unsigned int max_rate = 0;
unsigned int event_interval = 0;

unsigned int string_size = 0;

bool mnsa2005_paper_benchmark = false;
bool constant_rate = false;

double min_event_interval = 0;//1.0 / 200;
double max_event_interval = 1.0 / 2;
double inc_event_interval = (max_event_interval - min_event_interval) / 30;
double real_event_interval = min_event_interval;

ThreadMutex mutex;
ThreadCondition condition(mutex);

class TSSignalReceiver : public ISignalReceiver {
public:
  TSSignalReceiver() : blocked(false) {}
  
  virtual void receiveSignal(Signal signal) {
    if (signal == FREE_QUEUE_SLOT) {
      mutex.lock();
      blocked = false;
      condition.notify();
      mutex.unlock();
    } else {
      cerr << "UNKNOWN SIGNAL " << signal << endl;
    }
  }
  
  bool blocked;
};


void serializationBenchmark() {
  try {
    timeval start_time, end_time, duration;

    Buffer b1(1024, 0);
    Buffer b2(1024, 0);
    Timestamp t;
    t.sync_time = Time(233, 32);
    t.stream_counter = 400;
    b1.setTimestamp(t);

    gettimeofday(&start_time, 0);

    unsigned int i = 0;
    while ( i < iteration) {
      strstream w_stream;
      StdOStream stream(&w_stream);

      MagicNumberOValueStream ov;
      ov.setOPartnerStream(&stream);

      ov << b1;
      ov.writeData();

      MagicNumberIValueStream miv;
      StdIStream i_s(&w_stream);
      miv.setIPartnerStream(&i_s);
      miv.readData();

      b2.createFromStream(miv);
      ++i;
    }


    gettimeofday(&end_time, 0);
    duration.tv_usec = end_time.tv_usec - start_time.tv_usec;
    if (duration.tv_usec < 0) {
      duration.tv_usec = second + duration.tv_usec ;
      duration.tv_sec  = end_time.tv_sec - start_time.tv_sec - 1;
    } else
      duration.tv_sec  = end_time.tv_sec - start_time.tv_sec ;

    cout << endl;
    cout << "Serialization/Deserialization of " << i << " Buffers " << endl ;
    cout << "Loop count   : " << i << endl;
    cout << "Duration sec : " << duration.tv_sec << endl;
    cout << "Duration usec: " << duration.tv_usec << endl << endl;


  } catch (const Exception& e) {
    cout << e << endl;
  }
}

void TCPStrategyBenchmark() {
  try {
    timeval start_time, end_time, duration;
    gettimeofday(&start_time, 0);

    unsigned int i = 0;
    while ( i < iteration) {
      TCPStrategy strategy;
      strategy.setAddress(TCPAddress(host, port));
      strategy.closeConnection(strategy.getStreamSocket().getSocket());
      ++i;
    }


    gettimeofday(&end_time, 0);
    duration.tv_usec = end_time.tv_usec - start_time.tv_usec;
    if (duration.tv_usec < 0) {
      duration.tv_usec = second + duration.tv_usec;
      duration.tv_sec  = end_time.tv_sec - start_time.tv_sec - 1;
    } else
      duration.tv_sec  = end_time.tv_sec - start_time.tv_sec ;

    cout << endl;
    cout << "Connection of " << i << " TCPStrategies " << endl ;
    cout << "Loop count   : " << i << endl;
    cout << "Duration sec : " << duration.tv_sec << endl;
    cout << "Duration usec: " << duration.tv_usec << endl << endl;


  } catch (const Exception& e) {
    cout << e << endl;
  }
}

void connectionBenchmark() {
  try {
    timeval start_time, end_time, duration;
    gettimeofday(&start_time, 0);

    unsigned int i = 0;
    while ( i < iteration) {
      TCPStrategy* strategy = new TCPStrategy();
      strategy->setAddress(TCPAddress(host, port));
      strategy->closeConnection(strategy->getStreamSocket().getSocket());
      ++i;
    }


    gettimeofday(&end_time, 0);
    duration.tv_usec = end_time.tv_usec - start_time.tv_usec;
    if (duration.tv_usec < 0) {
      duration.tv_usec = second + duration.tv_usec;
      duration.tv_sec  = end_time.tv_sec - start_time.tv_sec - 1;
    } else
      duration.tv_sec  = end_time.tv_sec - start_time.tv_sec ;

    cout << endl;
    cout << "Connection of " << i << " TCPStrategies " << endl ;
    cout << "Loop count   : " << i << endl;
    cout << "Duration sec : " << duration.tv_sec << endl;
    cout << "Duration usec: " << duration.tv_usec << endl << endl;


  } catch (const Exception& e) {
    cout << e << endl;
  }
}

void interfaceConnectionBenchmark(int argc, char** argv) {
  try {
    NMMApplication& app = *ProxyApplication::getApplication(argc, argv);
    ClientRegistry& reg = app.getRegistry();

    if (!host.empty())
      reg.addRegistry(TCPAddress(host, port));

    NodeDescription nd("RawNode");
    nd.setLocation(host);

    unsigned int size = iteration;
    list<Response> resp = reg.initRequest(nd);
    vector<INode*> i_node_array(size);

    for (unsigned int i = 0; i < size; ++i)  {
      i_node_array[i] = reg.requestNode(resp);
    }

    cout << "Finished requesting " << iteration << " nodes " << endl;

    timeval start_time, end_time, duration;
    gettimeofday(&start_time, 0);

    unsigned int i = 0;
    while ( i < size) {
      try {
        i_node_array[i]->getParentObject()->establishBinding();
        delete i_node_array[i];
        ++i;
      } catch (const Exception& e) {
        cout << e << endl;
        break;
      }

    }

    gettimeofday(&end_time, 0);
    duration.tv_usec = end_time.tv_usec - start_time.tv_usec;
    if (duration.tv_usec < 0) {
      duration.tv_usec = second + duration.tv_usec;
      duration.tv_sec  = end_time.tv_sec - start_time.tv_sec - 1;
    } else
      duration.tv_sec  = end_time.tv_sec - start_time.tv_sec ;

    cout << endl;
    cout << "Connection of " << i << " different object-interfaces with their distributed parent-object " << endl ;
    cout << "Loop count   : " << i << endl;
    cout << "Duration sec : " << duration.tv_sec << endl;
    cout << "Duration usec: " << duration.tv_usec << endl << endl;

  } catch (const Exception& e) {
    cout << e << endl;
  }

}

/** Receiver object for method call benchmark.
 */
class MethodCallReceiver
  : public TimedElement,
    public IMethodCallImpl
{
public:
  MethodCallReceiver()
    : __own_string(string_size, '$'),
      __method_call_counter(0)
  {}
  
  virtual void noArgumentNoReturn() {
    ++__method_call_counter;
  }
  
  virtual int noArgumentIntReturn() {
    ++__method_call_counter;
    return 0;
  }
  
  virtual int oneArgumentInIntReturn(const int& ) {
    
    ++__method_call_counter;
    return 0;
  }
  
  virtual int twoArgumentInIntReturn(const int& , const int& ){
    
    ++__method_call_counter;
    return 0;
  }
  
  virtual int threeArgumentInIntReturn(const int& , const int& , const int& ){  
    ++__method_call_counter;
    return 0;
  }
  
  virtual int fourArgumentInIntReturn(const int& , const int& , const int& , const int& ){
    
    ++__method_call_counter;
    return 0;
  }
  
  virtual int fiveArgumentInIntReturn(const int& , const int& , const int& , const int& , const int& ) {
    
    ++__method_call_counter;
    return 0;
  }
  
  virtual int oneArgumentOutIntReturn(int& ) {
    ++__method_call_counter;
    return 0;
  }
  
  virtual int twoArgumentOutIntReturn(int& , int& ) {
    ++__method_call_counter;
    return 0;
  }
  
  virtual int threeArgumentOutIntReturn(int& , int& , int& ) {  
    ++__method_call_counter;
    return 0;
  }
  
  virtual int fourArgumentOutIntReturn(int& , int& , int& , int& ) {
    ++__method_call_counter;
    return 0;
  }
  
  virtual int fiveArgumentOutIntReturn(int& , int& , int& , int& , int& ) {
    ++__method_call_counter;
    return 0;
  }
  
  virtual int oneArgumentInOutIntReturn(int& ) {
    ++__method_call_counter;
    return 0;
  }
  
  virtual int twoArgumentInOutIntReturn(int& , int& ) {
    ++__method_call_counter;
    return 0;
  }
  
  virtual int threeArgumentInOutIntReturn(int& , int& , int& ) {
    ++__method_call_counter;
    return 0;
  }
  
  virtual int fourArgumentInOutIntReturn(int& , int& , int& , int& ) {
    ++__method_call_counter;
    return 0;
  }
  
  virtual int fiveArgumentInOutIntReturn(int& , int& , int& , int& , int& ) {
    ++__method_call_counter;
    return 0;
  }
  
  virtual void stringInArgument(const string& ) {
    ++__method_call_counter;
  }

  virtual void stringOutArgument(string& s) {
    ++__method_call_counter;
    
    /* return a string of string_size */
    s = __own_string;
  }

  virtual void stringInOutArgument(string& ) {
    ++__method_call_counter;
  }
  
  virtual void resetMethodCallCounter() {
    __method_call_counter = 0;
  }
  
  virtual int getMethodCallCounter() {
    return __method_call_counter;
  }
  
private:
  string __own_string;
  unsigned int __method_call_counter;
};

void methodCallBenchmarkReceiver( int port ) {
  TCPAcceptor tcp_acceptor(port);
  
  MethodCallReceiver recv;
  SReferenceManager::getInstance().registerService("Benchmark", &recv);
  
  try {
    while (1) {
      sleep (15);
    }
  } catch (const Exception& e ) {
    cout << e << endl;
  }
}

void methodCallBenchmark(bool local = false, bool strings = false) {
  try {
    unsigned int size = iteration;   /* number of iterations */
    MethodCallReceiver local_recv;   /* local receiver object */
    NMMObject* object = &local_recv; /* pointer to local receiver for virtual function call benchmark */
    IMethodCall_var imethod;         /* method call interface */
    
    Clock clock;                     /* clock for time measurements */
    Time t1, t2;                     /* start and end time */
    Interval duration;               /* duration */
    int64_t time_per_call;           /* time in nanoseconds per call */
    unsigned int i;                  /* loop counter */
    
    /* BEGIN> Connect to receiver */
    if(!host.empty()) {
      /* BEGIN> Connect to remote receiver */
      cout << endl << "Performing distributed method calls ----------------->" << endl << endl;
      
      TCPAddress address(host, port);
      
      ProxyObject* proxy_object = new ProxyObject();
      proxy_object->connectToObject(address);
      
      IObjectAcceptor_var acceptor(proxy_object->getCheckedInterface<IObjectAcceptor>());
      
      acceptor->connectToService("Benchmark");
      
      imethod.reset(proxy_object->getCheckedInterface<IMethodCall>());
      /* END< */
    } else {
      /* BEGIN> Connect to local receiver */
      if(local) {
        cout << endl << "Performing C++ virtual method calls ------------------>" << endl << endl;
      } else {
        cout << endl << "Performing local NMM method calls ------------------->" << endl << endl;
      }
      
      imethod.reset(local_recv.getCheckedInterface<IMethodCall>());
      /* END< */
    }
    
    /* establish the binding */
    imethod->getParentObject()->establishBinding();  
    /* END< */
    
    if(strings) {
      /* BEGIN> in string benchmark */
      string in_string(string_size, '#');      
      t1 = clock.getTime();
      i = 0;
      if(!local) {
        while ( i < size) {
          imethod->stringInArgument(in_string);
          ++i;
        }
      } else {
        while ( i < size) {
          dynamic_cast<MethodCallReceiver*>(object)->stringInArgument(in_string);
          ++i;
        }
      }
      t2 = clock.getTime();
      
      duration = t2 - t1;
      time_per_call = (nano_per_sec * duration.sec + duration.nsec) / i;
      
      cout << endl;
      cout << "Method call with string in-argument argument(s) of size " << string_size << " and no return value " << endl;
      cout << "Loop count    : " << i << endl;
      cout << "Duration      : " << duration << endl;
      cout << "Time per call : " << time_per_call << " ns" << endl;
      
//      cout << "Read Throughput : " << dynamic_cast<TCPStrategy*>(imethod->getParentObject()->getOutOfBandChannel().getTransportStrategy())->getNetStream().getReadCounter() / (1024) << "MByte " << endl;
//      cout << "Write Throughput : " << dynamic_cast<TCPStrategy*>(imethod->getParentObject()->getOutOfBandChannel().getTransportStrategy())->getNetStream().getWriteCounter() / (1024) << "MByte" << endl << endl;
//      dynamic_cast<TCPStrategy*>(imethod->getParentObject()->getOutOfBandChannel().getTransportStrategy())->getNetStream().resetReadCounter();
//      dynamic_cast<TCPStrategy*>(imethod->getParentObject()->getOutOfBandChannel().getTransportStrategy())->getNetStream().resetWriteCounter();      
      /* END< in string benchmark */      
      
      /* BEGIN> out string benchmark */
      string out_string(string_size, '#');
      t1 = clock.getTime();
      i = 0;
      if(!local) {
        while ( i < size) {
          imethod->stringOutArgument(out_string);
          ++i;
        }
      } else {
        while ( i < size) {
          dynamic_cast<MethodCallReceiver*>(object)->stringOutArgument(out_string);
          ++i;
        }
      }
      t2 = clock.getTime();
      
      duration = t2 - t1;
      time_per_call = (nano_per_sec * duration.sec + duration.nsec) / i;
      
      cout << endl;
      cout << "Method call with string out-argument argument(s) of size " << string_size << " and no return value " << endl;
      cout << "Loop count    : " << i << endl;
      cout << "Duration      : " << duration << endl;
      cout << "Time per call : " << time_per_call << " ns" << endl;
      
//      cout << "Read Throughput : " << dynamic_cast<TCPStrategy*>(imethod->getParentObject()->getOutOfBandChannel().getTransportStrategy())->getNetStream().getReadCounter() / (1024) << "MByte " << endl;
//      cout << "Write Throughput : " << dynamic_cast<TCPStrategy*>(imethod->getParentObject()->getOutOfBandChannel().getTransportStrategy())->getNetStream().getWriteCounter() / (1024) << "MByte" << endl << endl;
//      dynamic_cast<TCPStrategy*>(imethod->getParentObject()->getOutOfBandChannel().getTransportStrategy())->getNetStream().resetReadCounter();
//      dynamic_cast<TCPStrategy*>(imethod->getParentObject()->getOutOfBandChannel().getTransportStrategy())->getNetStream().resetWriteCounter();      
      /* END< out string benchmark */
    } else {
      /* BEGIN> Benchmark for methood void noArgumentNoReturn() */
      t1 = clock.getTime();
      i = 0;
      if(!local) {
        while ( i < size) {
          imethod->noArgumentNoReturn();
          ++i;
        }
      } else {
        while ( i < size) {
          dynamic_cast<MethodCallReceiver*>(object)->noArgumentNoReturn();
          ++i;
        }
      }
      t2 = clock.getTime();
      
      duration = t2 - t1;
      time_per_call = (nano_per_sec * duration.sec + duration.nsec) / i;
      
      cout << endl;
      cout << "Method call with 0 argument(s) and no return value " << endl;
      cout << "Loop count    : " << i << endl;
      cout << "Duration      : " << duration << endl;
      cout << "Time per call : " << time_per_call << " ns" << endl;
      
//       cout << "Read Throughput : " << dynamic_cast<TCPStrategy*>(imethod->getParentObject()->getOutOfBandChannel().getTransportStrategy())->getNetStream().getReadCounter() / (1024) << "MByte " << endl;
//       cout << "Write Throughput : " << dynamic_cast<TCPStrategy*>(imethod->getParentObject()->getOutOfBandChannel().getTransportStrategy())->getNetStream().getWriteCounter() / (1024) << "MByte" << endl << endl;
//       dynamic_cast<TCPStrategy*>(imethod->getParentObject()->getOutOfBandChannel().getTransportStrategy())->getNetStream().resetReadCounter();
//       dynamic_cast<TCPStrategy*>(imethod->getParentObject()->getOutOfBandChannel().getTransportStrategy())->getNetStream().resetWriteCounter();
      /* END< Benchmark for methood void noArgumentNoReturn() */
      
      /* BEGIN> Benchmark for method int noArgumentIntReturn() */
      t1 = clock.getTime();
      i = 0;
      if(!local) {
        while ( i < size) {
          imethod->noArgumentIntReturn();
          ++i;
        }
      } else {
        while ( i < size) {
          dynamic_cast<MethodCallReceiver*>(object)->noArgumentIntReturn();
          ++i;
        }
      }
      t2 = clock.getTime();
      
      duration = t2 - t1;
      time_per_call = (nano_per_sec * duration.sec + duration.nsec) / i;
      
      cout << endl;
      cout << "Method call with 0 argument(s) and 1 return value " << endl;
      cout << "Loop count    : " << i << endl;
      cout << "Duration      : " << duration << endl;
      cout << "Time per call : " << time_per_call << " ns" << endl;
      
//       cout << "Read Throughput : " << dynamic_cast<TCPStrategy*>(imethod->getParentObject()->getOutOfBandChannel().getTransportStrategy())->getNetStream().getReadCounter() / (1024) << "MByte " << endl;
//       cout << "Write Throughput : " << dynamic_cast<TCPStrategy*>(imethod->getParentObject()->getOutOfBandChannel().getTransportStrategy())->getNetStream().getWriteCounter() / (1024) << "MByte" << endl << endl;
//       dynamic_cast<TCPStrategy*>(imethod->getParentObject()->getOutOfBandChannel().getTransportStrategy())->getNetStream().resetReadCounter();
//       dynamic_cast<TCPStrategy*>(imethod->getParentObject()->getOutOfBandChannel().getTransportStrategy())->getNetStream().resetWriteCounter();
      /* END< Benchmark for method int noArgumentIntReturn() */
      
      /* BEGIN> Benchmark for method int oneArgumentInIntReturn(in int) */
      t1 = clock.getTime();
      i = 0;
      if(!local) {
        while ( i < size) {
          imethod->oneArgumentInIntReturn(1);
          ++i;
        }
      } else {
        while ( i < size) {
          dynamic_cast<MethodCallReceiver*>(object)->oneArgumentInIntReturn(1);
          ++i;
        }
      }
      t2 = clock.getTime();

      duration = t2 - t1;
      time_per_call = (nano_per_sec * duration.sec + duration.nsec) / i;
      
      cout << endl;
      cout << "Method call with 1 in argument(s) and 1 return value " << endl;
      cout << "Loop count    : " << i << endl;
      cout << "Duration      : " << duration << endl;
      cout << "Time per call : " << time_per_call << " ns" << endl;
      
//       cout << "Read Throughput : " << dynamic_cast<TCPStrategy*>(imethod->getParentObject()->getOutOfBandChannel().getTransportStrategy())->getNetStream().getReadCounter() / (1024) << "MByte " << endl;
//       cout << "Write Throughput : " << dynamic_cast<TCPStrategy*>(imethod->getParentObject()->getOutOfBandChannel().getTransportStrategy())->getNetStream().getWriteCounter() / (1024) << "MByte" << endl << endl;
//       dynamic_cast<TCPStrategy*>(imethod->getParentObject()->getOutOfBandChannel().getTransportStrategy())->getNetStream().resetReadCounter();
//       dynamic_cast<TCPStrategy*>(imethod->getParentObject()->getOutOfBandChannel().getTransportStrategy())->getNetStream().resetWriteCounter();
      /* END< Benchmark for method int oneArgumentInIntReturn(in int) */
      
      /* BEGIN> Benchmark for method int twoArgumentInIntReturn(in int, in int) */
      t1 = clock.getTime();
      i = 0;
      if(!local) {
        while ( i < size) {
          imethod->twoArgumentInIntReturn(1,1);
          ++i;
        }
      } else {
        while ( i < size) {
          dynamic_cast<MethodCallReceiver*>(object)->twoArgumentInIntReturn(1,1);
          ++i;
        }
      }
      t2 = clock.getTime();
      
      duration = t2 - t1;
      time_per_call = (nano_per_sec * duration.sec + duration.nsec) / i;
      
      cout << endl;
      cout << "Method call with 2 in argument(s) and 1 return value " << endl;
      cout << "Loop count    : " << i << endl;
      cout << "Duration      : " << duration << endl;
      cout << "Time per call : " << time_per_call << " ns" << endl;
      
//       cout << "Read Throughput : " << dynamic_cast<TCPStrategy*>(imethod->getParentObject()->getOutOfBandChannel().getTransportStrategy())->getNetStream().getReadCounter() / (1024) << "MByte " << endl;
//       cout << "Write Throughput : " << dynamic_cast<TCPStrategy*>(imethod->getParentObject()->getOutOfBandChannel().getTransportStrategy())->getNetStream().getWriteCounter() / (1024) << "MByte" << endl << endl;
//       dynamic_cast<TCPStrategy*>(imethod->getParentObject()->getOutOfBandChannel().getTransportStrategy())->getNetStream().resetReadCounter();
//       dynamic_cast<TCPStrategy*>(imethod->getParentObject()->getOutOfBandChannel().getTransportStrategy())->getNetStream().resetWriteCounter();
      /* END< Benchmark for method int twoArgumentInIntReturn(in int, in int) */
      
      /* BEGIN> Benchmark for method int oneArgumentOutIntReturn(out int) */
      t1 = clock.getTime();
      i = 0;
      if(!local) {
        while ( i < size) {
          int v;
          imethod->oneArgumentOutIntReturn(v);
          ++i;
        }
      } else {
        while ( i < size) {
          int v;
          dynamic_cast<MethodCallReceiver*>(object)->oneArgumentOutIntReturn(v);
          ++i;
        }
      }
      t2 = clock.getTime();

      duration = t2 - t1;
      time_per_call = (nano_per_sec * duration.sec + duration.nsec) / i;
      
      cout << endl;
      cout << "Method call with 1 out argument(s) and 1 return value " << endl;
      cout << "Loop count    : " << i << endl;
      cout << "Duration      : " << duration << endl;
      cout << "Time per call : " << time_per_call << " ns" << endl;
      
//       cout << "Read Throughput : " << dynamic_cast<TCPStrategy*>(imethod->getParentObject()->getOutOfBandChannel().getTransportStrategy())->getNetStream().getReadCounter() / (1024) << "MByte " << endl;
//       cout << "Write Throughput : " << dynamic_cast<TCPStrategy*>(imethod->getParentObject()->getOutOfBandChannel().getTransportStrategy())->getNetStream().getWriteCounter() / (1024) << "MByte" << endl << endl;
//       dynamic_cast<TCPStrategy*>(imethod->getParentObject()->getOutOfBandChannel().getTransportStrategy())->getNetStream().resetReadCounter();
//       dynamic_cast<TCPStrategy*>(imethod->getParentObject()->getOutOfBandChannel().getTransportStrategy())->getNetStream().resetWriteCounter();
      /* END< Benchmark for method int oneArgumentOutIntReturn(out int) */
      
      /* BEGIN> Benchmark for method int twoArgumentOutIntReturn(out int, out int) */
      t1 = clock.getTime();
      i = 0;
      if(!local) {
        while ( i < size) {
          int v1, v2;
          imethod->twoArgumentOutIntReturn(v1,v2);
          ++i;
        }
      } else {
        while ( i < size) {
          int v1, v2;
          dynamic_cast<MethodCallReceiver*>(object)->twoArgumentOutIntReturn(v1,v2);
          ++i;
        }
      }
      t2 = clock.getTime();

      duration = t2 - t1;
      time_per_call = (nano_per_sec * duration.sec + duration.nsec) / i;
      
      cout << endl;
      cout << "Method call with 2 out argument(s) and 1 return value " << endl;
      cout << "Loop count    : " << i << endl;
      cout << "Duration      : " << duration << endl;
      cout << "Time per call : " << time_per_call << " ns" << endl;
      
//       cout << "Read Throughput : " << dynamic_cast<TCPStrategy*>(imethod->getParentObject()->getOutOfBandChannel().getTransportStrategy())->getNetStream().getReadCounter() / (1024) << "MByte " << endl;
//       cout << "Write Throughput : " << dynamic_cast<TCPStrategy*>(imethod->getParentObject()->getOutOfBandChannel().getTransportStrategy())->getNetStream().getWriteCounter() / (1024) << "MByte" << endl << endl;
//       dynamic_cast<TCPStrategy*>(imethod->getParentObject()->getOutOfBandChannel().getTransportStrategy())->getNetStream().resetReadCounter();
//       dynamic_cast<TCPStrategy*>(imethod->getParentObject()->getOutOfBandChannel().getTransportStrategy())->getNetStream().resetWriteCounter();
      /* END< Benchmark for method int twoArgumentInOutIntReturn(inout int, inout int) */

      /* BEGIN> Benchmark for method int oneArgumentInOutIntReturn(inout int) */
      t1 = clock.getTime();
      i = 0;
      if(!local) {
        while ( i < size) {
          int v;
          imethod->oneArgumentInOutIntReturn(v);
          ++i;
        }
      } else {
        while ( i < size) {
          int v;
          dynamic_cast<MethodCallReceiver*>(object)->oneArgumentInOutIntReturn(v);
          ++i;
        }
      }
      t2 = clock.getTime();

      duration = t2 - t1;
      time_per_call = (nano_per_sec * duration.sec + duration.nsec) / i;
      
      cout << endl;
      cout << "Method call with 1 inout argument(s) and 1 return value " << endl;
      cout << "Loop count    : " << i << endl;
      cout << "Duration      : " << duration << endl;
      cout << "Time per call : " << time_per_call << " ns" << endl;
      
//       cout << "Read Throughput : " << dynamic_cast<TCPStrategy*>(imethod->getParentObject()->getOutOfBandChannel().getTransportStrategy())->getNetStream().getReadCounter() / (1024) << "MByte " << endl;
//       cout << "Write Throughput : " << dynamic_cast<TCPStrategy*>(imethod->getParentObject()->getOutOfBandChannel().getTransportStrategy())->getNetStream().getWriteCounter() / (1024) << "MByte" << endl << endl;
//       dynamic_cast<TCPStrategy*>(imethod->getParentObject()->getOutOfBandChannel().getTransportStrategy())->getNetStream().resetReadCounter();
//       dynamic_cast<TCPStrategy*>(imethod->getParentObject()->getOutOfBandChannel().getTransportStrategy())->getNetStream().resetWriteCounter();
      /* END< Benchmark for method int oneArgumentOutIntReturn(out int) */
      
      /* BEGIN> Benchmark for method int twoArgumentInOutIntReturn(inout int, inout int) */
      t1 = clock.getTime();
      i = 0;
      if(!local) {
        while ( i < size) {
          int v1, v2;
          imethod->twoArgumentInOutIntReturn(v1,v2);
          ++i;
        }
      } else {
        while ( i < size) {
          int v1, v2;
          dynamic_cast<MethodCallReceiver*>(object)->twoArgumentInOutIntReturn(v1,v2);
          ++i;
        }
      }
      t2 = clock.getTime();
      
      duration = t2 - t1;
      time_per_call = (nano_per_sec * duration.sec + duration.nsec) / i;
      
      cout << endl;
      cout << "Method call with 2 inout argument(s) and 1 return value " << endl;
      cout << "Loop count    : " << i << endl;
      cout << "Duration      : " << duration << endl;
      cout << "Time per call : " << time_per_call << " ns" << endl;
      
//       cout << "Read Throughput : " << dynamic_cast<TCPStrategy*>(imethod->getParentObject()->getOutOfBandChannel().getTransportStrategy())->getNetStream().getReadCounter() / (1024) << "MByte " << endl;
//       cout << "Write Throughput : " << dynamic_cast<TCPStrategy*>(imethod->getParentObject()->getOutOfBandChannel().getTransportStrategy())->getNetStream().getWriteCounter() / (1024) << "MByte" << endl << endl;
//       dynamic_cast<TCPStrategy*>(imethod->getParentObject()->getOutOfBandChannel().getTransportStrategy())->getNetStream().resetReadCounter();
//       dynamic_cast<TCPStrategy*>(imethod->getParentObject()->getOutOfBandChannel().getTransportStrategy())->getNetStream().resetWriteCounter();
      /* END< Benchmark for method int twoArgumentInOutIntReturn(inout int, inout int) */
    }
    
    cout << "Performed " << imethod->getMethodCallCounter() << " method calls" << endl;    
  } catch (const Exception& e) {
    cout << e << endl;
  }
}

void methodCallBenchmarkLocal() {
  methodCallBenchmark(true);
}

void methodCallBenchmarkWithStrings() {
  methodCallBenchmark(false, true);
}

void nodeConnectionBenchmark(int argc, char** argv) {
  try {
    NMMApplication& app = *ProxyApplication::getApplication(argc, argv);
    ClientRegistry& reg = app.getRegistry();
    reg.addRegistry(TCPAddress(host, port));

    NodeDescription nd("DevNullNode");
    nd.setLocation(host);

    unsigned int size = 10;
    list<Response> resp = reg.initRequest(nd);
    vector<INode*> raw_node_array(size);
    vector<INode*> devnull_node_array(size);

    for (unsigned int i = 0; i < size; ++i) {
      Node* n = new RawNode();
      raw_node_array[i] = n->getInterface<INode>();
      raw_node_array[i]->init();
      raw_node_array[i]->initOutput();
      devnull_node_array[i] = reg.requestNode(resp);
      devnull_node_array[i]->init();
    }


    timeval start_time, end_time, duration;
    gettimeofday(&start_time, 0);

    for (unsigned int i = 0; i < size; ++i) {
      NMM::connect(raw_node_array[i],devnull_node_array[i]);
    }

    gettimeofday(&end_time, 0);
    duration.tv_usec = end_time.tv_usec - start_time.tv_usec;
    if (duration.tv_usec < 0) {
      duration.tv_usec = second + duration.tv_usec;
      duration.tv_sec  = end_time.tv_sec - start_time.tv_sec - 1;
    } else
      duration.tv_sec  = end_time.tv_sec - start_time.tv_sec ;

    cout << endl;
    cout << "Method call with 2 inout argument(s) and 1 return value ";
    cout << "Loop count   : " << size << endl;
    cout << "Duration sec : " << duration.tv_sec << endl;
    cout << "Duration usec: " << duration.tv_usec << endl << endl;
    /**/

  } catch (const Exception& e) {
    cout << e << endl;
  }
}

void distNodeConnectionBenchmark(int argc, char** argv) {
  try {
    NMMApplication& app = *ProxyApplication::getApplication(argc, argv);
    ClientRegistry& reg = app.getRegistry();
    reg.addRegistry(TCPAddress(host, port));

    NodeDescription nd("DevNullNode");
    nd.setLocation(host);

    NodeDescription nd2("RawNode");
    nd2.setLocation(host);

    unsigned int size = 10;
    list<Response> resp = reg.initRequest(nd);
    list<Response> resp2 = reg.initRequest(nd2);
    vector<INode*> raw_node_array(size);
    vector<INode*> devnull_node_array(size);

    for (unsigned int i = 0; i < size; ++i) {
      raw_node_array[i] = reg.requestNode(resp2);
      raw_node_array[i]->init();
      raw_node_array[i]->initOutput();
      devnull_node_array[i] = reg.requestNode(resp);
      devnull_node_array[i]->init();
    }


    timeval start_time, end_time, duration;
    gettimeofday(&start_time, 0);

    for (unsigned int i = 0; i < size; ++i) {
      NMM::connect(raw_node_array[i],devnull_node_array[i]);
    }

    gettimeofday(&end_time, 0);
    duration.tv_usec = end_time.tv_usec - start_time.tv_usec;
    if (duration.tv_usec < 0) {
      duration.tv_usec = second + duration.tv_usec;
      duration.tv_sec  = end_time.tv_sec - start_time.tv_sec - 1;
    } else
      duration.tv_sec  = end_time.tv_sec - start_time.tv_sec ;

    cout << endl;
    cout << "Method call with 2 inout argument(s) and 1 return value ";
    cout << "Loop count   : " << size << endl;
    cout << "Duration sec : " << duration.tv_sec << endl;
    cout << "Duration usec: " << duration.tv_usec << endl << endl;
    /**/

  } catch (const Exception& e) {
    cout << e << endl;
  }
}

void readFromNetwork( int port ) {
  SockAcceptor s_accept;
  INetAddr addr;
  addr.setPort(port);
  s_accept.open(addr);
  SockStream sock;
  UnbufferedNetStream stream(sock);
  s_accept.accept(stream.getSock());
  char data[buffer_size];
  try {
    while (1) {
      stream.readBuffer(&data[0], buffer_size);
    }
  } catch (const Exception& e ) {
    cout << e << endl;
  }
  cout << stream.getReadCounter()  << " Bytes read " << endl;
}

void readFromNetwork2( int port ) {
  cout << "Using readFromNetwork2 : " << endl;
  SockAcceptor s_accept;
  INetAddr addr;
  addr.setPort(port);
  s_accept.open(addr);
  SockStream sock;
  BufferedNetStream stream(sock);
  s_accept.accept(stream.getSock());
  Buffer *b = new Buffer(buffer_size, 0);
  MagicNumberIValueStream iv;
  iv.setIPartnerStream(&stream);
  try {
    while (1) {
      iv.readData();
      iv >> (*b);
    }
  } catch (const Exception& e ) {
    cout << e << endl;
  }
  cout << stream.getReadCounter()  << " Bytes read " << endl;
}


void readFromNetwork3( int port ) {
  SockAcceptor s_accept;
  INetAddr addr;
  addr.setPort(port);
  s_accept.open(addr);

  TCPStrategy strategy;
  s_accept.accept(strategy.getStreamSocket());

  ITCPControl_var itcp (strategy.getInterface<ITCPControl>());
  itcp->enableNoDelay(true);
  IExternalBufferManager b_manager;
  strategy.setBufferManager(&b_manager);
  
  /* register the Socket by the NetReactor */
  SNetReactor::getInstance().registerSocket( strategy.getStreamSocket().getSocket(), &strategy);
  
  try {
    while (1) {
      sleep (15);
    }
  } catch (const Exception& e ) {
    cout << e << endl;
  }
  //cout << stream.getReadCounter()  << " Bytes read " << endl;
}


class BenchmarkReceiver : public NMMObject, public IMessageReceiver, public TimedElement {
public:
  enum Mode {
    SEQUENCE_NUMBERS,
    AVERAGE,
    VARIANCE,
  };
  
public:
  BenchmarkReceiver(Mode mode = AVERAGE)
    : __first_message(true),
      __mode(mode)
  {
    getEventDispatcher().registerEvent("connectToBenchmark",new TEDObject0<BenchmarkReceiver, void (BenchmarkReceiver::*)(), void>(this, &BenchmarkReceiver::connectToBenchmark));
  }
  
  virtual Result receiveMessage(Message* message, const IMessageSender* /*sender*/) {
    /* the time when the message was received by the application */
    Time forward_time = getTime();
    
    /* the message time */
    Time msg_time = forward_time;
    if(__first_message ){
      __start_time = msg_time;
      __last_time = msg_time;
      __first_message = false;
    }
    
    if(message->getType() == Message::BUFFER) {
      if(__mode == SEQUENCE_NUMBERS) {
        /* output pairs (sequence number, delta reception time) */
        cout << message->getTimestamp().stream_counter << " " << double(msg_time - __last_time) << endl;
      } else {
        /* sample delta reception time */
        if(message->getTimestamp().stream_counter >= static_cast<int>(jitter_first_sample) &&
           message->getTimestamp().stream_counter < static_cast<int>(jitter_first_sample + jitter_samples)) {
          __jitter_samples.push_back(msg_time - __last_time);
        }
      }
      
      __last_time = msg_time;
    }
    
    /* compute transmission delay for buffers */
    // XXX: uncomment to use delay
//     if(message->getType() == Message::BUFFER &&
//        message->getTimestamp().stream_counter >= static_cast<int>(jitter_first_sample) &&
//        message->getTimestamp().stream_counter < static_cast<int>(jitter_first_sample + jitter_samples)) {
//       Interval delay = forward_time - message->getTimestamp().sync_time;
//       __jitter_samples.push_back(delay);
//     }
    
    if(__mode != SEQUENCE_NUMBERS && jitter_samples > 0 &&
       message->getTimestamp().stream_counter == static_cast<int>(jitter_first_sample + jitter_samples)) {
      /* compute average and variance */
      TAverage<Interval> average;
      average.setAverageInterval(__jitter_samples.size());
      for(unsigned int i=0; i<__jitter_samples.size(); ++i) {
        average.put(__jitter_samples[i]);
      }
      if(!mnsa2005_paper_benchmark) {
    // XXX: uncomment to use delay
//         cout << "Average message transmission delay: " << average.get() << endl;
//         cout << "Jitter                            : " << average.getVariance() << endl;
        cout << "Average difference in buffer reception time    : " << average.get() << endl;
        cout << "Variance of difference in buffer reception time: " << average.getVariance() << endl;
      } else {
        cout << real_event_interval * 100.0;
        if(__mode == AVERAGE) {
          cout << " " << double(average.get()) << endl;
        } else if(__mode == VARIANCE){
          cout << " " << double(average.getVariance()) << endl;
        }
        
        real_event_interval += inc_event_interval;
      }
      
      __jitter_samples.clear();
    }
    /* END< jitter benchmark */
    
    message->release();
    
    return SUCCESS;
  }
  
  virtual void connectToBenchmark() {
    TransportStrategy* ts = dynamic_cast<TransportStrategy*>(getCurrentSender());
    assert(ts);
    
    ts->setMessageReceiver(this);
    
    CompositeStrategy* cs = dynamic_cast<CompositeStrategy*>(ts);
    if(cs) {
      TransportStrategy* buffer_ts = cs->getBufferStrategy();
      buffer_ts->setBufferManager(new IExternalBufferManager());
    } else {
      ts->setBufferManager(new IExternalBufferManager());
    }
  }
  
  vector<Interval> __jitter_samples;
  Time __start_time;
  Time __last_time;
  bool __first_message;
  Mode __mode;
};

void readFromNetwork4( int port, ICompositeStrategy::Mode, BenchmarkReceiver::Mode jitter_mode) {
  TCPAcceptor tcp_acceptor(port);
  
  BenchmarkReceiver recv(jitter_mode);
  SReferenceManager::getInstance().registerService("Benchmark", &recv);
  
  try {
    while (1) {
      sleep (15);
    }
  } catch (const Exception& e ) {
    cout << e << endl;
  }
}

void writeToNetwork1( ) {
  /* Uses a SockStream */
  SockConnector s_connect;
  INetAddr addr;
  addr.setPort(port);
  addr.setAddress(host.c_str());

  SockStream sock;
  BufferedNetStream stream(sock);
  s_connect.connect(stream.getSock(), addr);
  char data[buffer_size];
  memset (data, buffer_size, 'r');

  timeval start_time, end_time, duration;
  gettimeofday(&start_time, 0);


  for ( unsigned int i = 0; i < iteration; ++i)
    {
      stream.writeBuffer(&data[0], buffer_size);
    }
  sock.close();

  gettimeofday(&end_time, 0);
  duration.tv_usec = end_time.tv_usec - start_time.tv_usec;
  if (duration.tv_usec < 0) {
    duration.tv_usec = second + duration.tv_usec;
    duration.tv_sec  = end_time.tv_sec - start_time.tv_sec - 1;
  } else {
    duration.tv_sec  = end_time.tv_sec - start_time.tv_sec ;
  }
  
  cout << endl;
  cout << "Sending  Buffer ";
  cout << "Duration sec : " << duration.tv_sec << endl;
  cout << "Duration usec: " << duration.tv_usec << endl << endl;
  double dur = duration.tv_sec + (double)(duration.tv_usec) / 1000000;
  double m_bytes = (double)(stream.getWriteCounter())/(double)(1024*1024);
  cout << "Duration compll " << dur << endl;
  cout << "Amount in MB " << m_bytes << endl;
  cout << m_bytes / dur   << " MBytes/s written " << endl;
}

void writeToNetwork2( ) {
  /* USes a MagicNumberValueStream */
  SockConnector s_connect;
  INetAddr addr;
  addr.setPort(port);
  addr.setAddress(host.c_str());

  SockStream sock;
  BufferedNetStream stream(sock);
  s_connect.connect(stream.getSock(), addr);
  char data[buffer_size];
  memset (data, buffer_size, 'r');

  timeval start_time, end_time, duration;

  MagicNumberOValueStream ov;
  ov.setOPartnerStream(&stream);
  Buffer *b = new Buffer(buffer_size,0);

  gettimeofday(&start_time, 0);
  for ( unsigned int i = 0; i < iteration; ++i) {
    ov << *b;
    ov.writeData();
  }
  gettimeofday(&end_time, 0);
  sock.close();
  
  duration.tv_usec = end_time.tv_usec - start_time.tv_usec;
  if (duration.tv_usec < 0) {
    duration.tv_usec = second + duration.tv_usec;
    duration.tv_sec  = end_time.tv_sec - start_time.tv_sec - 1;
  } else
    duration.tv_sec  = end_time.tv_sec - start_time.tv_sec ;
  
  cout << endl;
  cout << "Sending  Buffer ";
  cout << "Duration sec : " << duration.tv_sec << endl;
  cout << "Duration usec: " << duration.tv_usec << endl << endl;
  double dur = duration.tv_sec + (double)(duration.tv_usec) / 1000000;
  double m_bytes = (double)(stream.getWriteCounter())/(double)(1024*1024);
  cout << "Duration compll " << dur << endl;
  cout << "Amount in MB " << m_bytes << endl;
  cout << m_bytes / dur   << " MBytes/s written " << endl;
}


void writeToNetwork3( ) {
  /* Uses a TCPStrategy */
  TCPAddress addr(host, port);
  TCPStrategy strategy;
  strategy.setAddress(addr);

  timeval start_time, end_time, duration;
  gettimeofday(&start_time, 0);

  Buffer *b = new Buffer(1024,0);
  b->increaseCount(iteration +2 );
  TSSignalReceiver ts_receiver;
  strategy.setSignalReceiver(&ts_receiver);

  gettimeofday(&start_time, 0);
  for ( unsigned int i = 0; i < iteration; ++i)
    {
      mutex.lock();
      strategy.receiveMessage(b, 0);
      condition.wait();
      mutex.unlock();
    }

  gettimeofday(&end_time, 0);
  duration.tv_usec = end_time.tv_usec - start_time.tv_usec;
  if (duration.tv_usec < 0) {
    duration.tv_usec = second + duration.tv_usec;
    duration.tv_sec  = end_time.tv_sec - start_time.tv_sec - 1;
  } else
    duration.tv_sec  = end_time.tv_sec - start_time.tv_sec ;

  sleep(2);
  cout << endl;
  cout << "Sending  Buffer " << strategy.getNetStream().getWriteCounter() << endl;
  cout << "Duration sec : " << duration.tv_sec << endl;
  cout << "Duration usec: " << duration.tv_usec << endl << endl;
  double dur = duration.tv_sec + (double)(duration.tv_usec) / 1000000;
  double m_bytes = (double)(strategy.getNetStream().getWriteCounter())/(double)(1024*1024);
  cout << "Duration compll " << dur << endl;
  cout << "Amount in MB " << m_bytes << endl;
  cout << m_bytes / dur   << " MBytes/s written " << endl;
}


void writeToNetwork4(bool composite, ICompositeStrategy::Mode mode) {
  /* Uses a TCPStrategy */
  TCPStrategy *buffer_strategy, *event_strategy;
  TransportStrategy *c_strategy;

  /* need a TCPAcceptor for back channel */
  TCPAcceptor tcp_acceptor;
  
  if(composite) {
    /* get ConnectionManager interface */
    TCPAddress icm_address(host, port);
    Interface* temp = SConnectionManager::getInstance().getService(icm_address, "ConnectionManager");
    IConnectionManager* icm = temp->getParentObject()->getInterface<IConnectionManager>();

    /* build composite address */
    TCPAddress buffer_addr(host, port);
    TCPAddress event_addr(host, port);
    ConnectorAddress master_addr("CompositeStrategy", 0);
    master_addr.setConnectionManager(*icm);
    CompositeAddress comp_addr(0, master_addr, buffer_addr, event_addr);
    
    /* build and connect composite strategy */
    CompositeStrategy* cs = new CompositeStrategy();
    cs->setMode(mode);
    cs->setLossInterval(loss_interval);
    cs->setAddress(comp_addr);
    
    buffer_strategy = dynamic_cast<TCPStrategy*>(cs->getBufferStrategy());
    event_strategy = dynamic_cast<TCPStrategy*>(cs->getEventStrategy());
    c_strategy = cs;
  } else {
    TCPAddress addr(host, port);
    TCPStrategy* strategy = new TCPStrategy();
    strategy->setAddress(addr);
    buffer_strategy = strategy;
    event_strategy = strategy;
    c_strategy = strategy;
  }
  
  /* connect to benchmark receiver */
  Event event1(IObjectAcceptor::connectToService_event, new TInValue<string>("Benchmark"));
  if(SUCCESS != c_strategy->receiveReliableEvent(event1, 0)) {
    throw Exception("Failed to send connectToService event");
  }
  if(event1.getException()) {
    throw *event1.getException();
  }
  Event event2("connectToBenchmark");
  if(SUCCESS != c_strategy->receiveReliableEvent(event2, 0)) {
    throw Exception("Failed to send connectToBenchmark event");
  }
  if(event2.getException()) {
    throw *event2.getException();
  }
  
  Clock clock;
  Time start_time, end_time;
  Interval duration;
  
  Buffer *b1 = new Buffer(buffer_size,0);
  Buffer *b2 = new Buffer(buffer_size,0);
  b1->setUsed(buffer_size);
  b2->setUsed(buffer_size);
  Event e("TestEvent");
  CEvent *ce1 = new CEvent(e);
  CEvent *ce2 = new CEvent(e);
  
  TSSignalReceiver ts_receiver;
  c_strategy->setSignalReceiver(&ts_receiver);

  Buffer* b = b1;
  CEvent* ce = ce1;

  Timestamp ts;
  ts.stream_counter = 1;
  
  start_time = clock.getTime();
  unsigned int buffer_count = 0;
  
  ThreadMutex sleep_mutex;
  ThreadCondition sleep_cond(sleep_mutex);
  
  double decision = 0;
  for ( unsigned int i = 0; i < iteration; ++i) {
    /* send an event every n buffers */
    bool buffer = true;
    if(mnsa2005_paper_benchmark) {
      decision += real_event_interval;
      if(decision > 0) {
        decision -= 1;
        buffer = false;
      }
    } else {
      buffer = (event_interval == 0 || (i+5) % event_interval != 0);
    }
    if(buffer) {
      /* BEGIN> maintain maximum transmission rate */
      if(max_rate != 0) {
        for(;;) {
          Time current_time = clock.getTime();
          Interval wait_time;
          
          if(!constant_rate) {
            /* compute amount of bytes already sent */
            unsigned int sent_bytes = buffer_strategy->getNetStream().getWriteCounter();
            // XXX: uncomment to include events as well
//             if(buffer_strategy != event_strategy) {
//               sent_bytes += event_strategy->getNetStream().getWriteCounter();
//             }
            
            /* compute current duration */
            duration = current_time - start_time;
            
            /* compute amount of time to send the same amount of bytes at max_rate */
            Interval min_duration(Rational(sent_bytes, max_rate));
            wait_time = min_duration - duration;
            
          } else {
            /* compute the time when this buffer should be sent with constant rate */
            Interval interval(Rational(buffer_count * buffer_size, max_rate));
            Time send_time = start_time + interval;
            wait_time = send_time - current_time;
          }
          
          /* wait */
          if(wait_time >= Interval(0)) {
            sleep_mutex.lock();
            sleep_cond.relativeTimedWait(wait_time.sec, wait_time.nsec);
            sleep_mutex.unlock();
          } else {
            break;
          }
        }
      }
      /* END< maintain maximum transmission rate */
      
      ++buffer_count;
      
      /* without a composite strategy, set the timestamp */
      if(!composite) {
        ts.sync_time = clock.getTime();
      }
      
      b->setTimestamp(ts);
      ++ts.stream_counter;
      b->increaseCount(100);
      mutex.lock();
      ts_receiver.blocked = true;
      
      /* if we are not using a CompositeStrategy, we simulate data loss by not sending a buffer */
      bool simulate_loss = !composite && loss_interval != 0 && ts.stream_counter % loss_interval == 0;
      //cout << double(clock.getTime() - start_time) << " " << 6 << endl;
      if(!simulate_loss) {
        if(BLOCKED != c_strategy->receiveMessage(b, 0)) {
          ts_receiver.blocked = false;
        }
      }
      
      mutex.unlock();
      if(b == b1) {
        b = b2;
      } else {
        b = b1;
      }
    } else {
      ce->setTimestamp(ts);
      ++ts.stream_counter;
      ce->increaseCount(100);
      mutex.lock();
      ts_receiver.blocked = true;
      if(BLOCKED != c_strategy->receiveMessage(ce, 0)) {
        ts_receiver.blocked = false;
      }
      mutex.unlock();
      if(ce == ce1) {
        ce = ce2;
      } else {
        ce = ce1;
      }
    }
    mutex.lock();
    if(ts_receiver.blocked) {
      condition.wait();
    }
    mutex.unlock();
  }

  end_time = clock.getTime();
  duration = end_time - start_time;
  
  sleep(2);
  if(!mnsa2005_paper_benchmark) {
    cout << endl;
    cout << "Sending  Buffer " << buffer_strategy->getNetStream().getWriteCounter() << endl;
    cout << "Duration     : " << duration << endl;
  }
  double dur = duration.sec + double(duration.nsec) / nano_per_sec;
  double m_bytes = (double)(buffer_strategy->getNetStream().getWriteCounter())/(double)(1024*1024);
  if(event_strategy != buffer_strategy) {
    m_bytes += (double)(event_strategy->getNetStream().getWriteCounter())/(double)(1024*1024);
  }
  if(!mnsa2005_paper_benchmark) {
    cout << "Duration compll " << dur << endl;
    cout << "Amount in MB " << m_bytes << endl;
    cout << m_bytes / dur   << " MBytes/s written " << endl;
  } else {
    cout << 100 * real_event_interval << " " << m_bytes / dur   << endl;
  }
}

void graphConnectionBenchmark(int argc, char** argv, const string& host1, const string& host2, bool disable_registry_optimization) {
  cerr << host1 << host2 << endl;
  GraphDescription gd;
  vector<NodeDescription> nd_vector;

  NodeDescription source("RawNode");
  NodeDescription sink("DevNullNode");
  NodeDescription nd("IdNode");

  nd_vector.push_back(source);

  for ( unsigned int i = 0; i < iteration-2; ++i) {
    nd_vector.push_back(NodeDescription(nd));
  }

  nd_vector.push_back(sink);

  for ( unsigned int i = 0; i < nd_vector.size(); ++i) {
    if ( ( i % 2 ) == 0 )
      nd_vector[i].setLocation(host1);
    else
      nd_vector[i].setLocation(host2);
  }

  NMMApplication* app = ProxyApplication::getApplication(argc, argv);
  ClientRegistry& reg = app->getRegistry();
  if(disable_registry_optimization) {
    cout << "Disable registry optimization -> you need a running serverregistry on every host!" << endl;
    reg.setOptimization (false);
  }

  try {
    reg.addRegistry(TCPAddress(host1, ClientRegistry::default_port));
  }  catch (...) {

  }
  try {
    reg.addRegistry(TCPAddress(host2, ClientRegistry::default_port));
  } catch (...) {

  }
  for ( unsigned int i = 0; i < nd_vector.size()-1; ++i) {
    gd.addEdge(nd_vector[i], nd_vector[i+1]);
  }

  Clock clock;
  Time t1 = clock.getTime();
  reg.requestGraph(gd);
  Time t2 = clock.getTime();
  gd.realizeGraph();
  Time t3 = clock.getTime();

  cout << "requesting " << nd_vector.size() << " nodes (requestGraph) : " << (t2 - t1) << endl;
  cout << "connecting " << nd_vector.size() << " nodes (realizeGraph) : " << (t3 - t2) << endl;
  cout << "together : " << (t3 - t1) << endl;
}

void usage(char* name) {
  cout << "Usage:" << endl
       << "  " << name << " [options] <benchmark number>" << endl
       << endl
       << "Options:" << endl
       << "  -i : Sets the number of iteration of a benchmaerk" << endl
       << "  -e : Sets the event interval for benchmarks 10-15" << endl
       << "  -l : Sets the interval for simulated data loss interval for benchmarks 10-15" << endl
       << "  -j : Enable jitter and delay measurements for benchmarks 10-15" << endl
       << "  -J : Set mode of jitter benchmark. Possible values:" << endl
       << "       s = delta reception time over sequence numbers" << endl
       << "       a = average delta reception time" << endl
       << "       v = variance of delta reception time" << endl
       << "  -r : Set maximum rate in MBytes per second 10-15" << endl
       << "  -c : Use together with -r to send buffers at constant rate if possible" << endl
       << "  -H : Sets the host of the distrributed server registry" << endl
       << "  -p : Sets the port of the distrributed server registry" << endl
       << "  -o : Disable optimization for local registry (e.g. for benchmark 16)" << endl
       << "  -s : Sets the size of the string to transmit in benchmark 33 and 31" << endl
       << "  -v : increase verbositiy (may be given multiple times)" << endl
       << "  -h : Show this help" << endl << endl
       << "Benchmark Nr.: " << endl
       << " 1 : benchmarks the serialization/deserialization of an NMM Buffer with 1024 Bytes data" << endl
       << " 2 : benchmarks the time to connect an interface to its distributed object." << endl
       << " 3 : benchmarks the time for a distributed method invocation. the server must run benchmark 31" << endl
       << "31 : server for benchmark 3 and 33" << endl
       << "32 : benchmarks the time for a local virtual method invocation." << endl
       << "33 : similiar to benchmark 3, but transmits a string of -s size" << endl
       << " 4 : benchmarks for receiving char arrays. The sender must use benchmark nr. 7 to send data" << endl
       << " 5 : benchmarks for receiving NMM buffers using a MagicNumberValueStream. The sender must use benchmark nr. 8 to send data" << endl
       << " 6 : benchmarks for receiving NMM buffers using a TCPStrategy. The sender must use benchmark nr. 9 to send data" << endl
       << " 7 : benchmarks for sending char arrays. The receiver must use benchmark nr. 4 to send data" << endl
       << " 8 : benchmarks for sending NMM buffers using a MagicNumberValueStream. The receiver must use benchmark nr. 5 to send data" << endl
       << " 9 : benchmarks for sending NMM buffers using a TCPStrategy. The receiver must use benchmark nr. 6 to send data" << endl
       << "10 : benchmarks for sending NMM buffers and events using a R/R CompositeStrategy. The sender must use benchmark nr. 11 to send data" << endl
       << "11 : benchmarks for sending NMM buffers and events using a R/R CompositeStrategy. The receiver must use benchmark nr. 10 to receive data" << endl
       << "12 : benchmarks for sending NMM buffers and events using a R/U CompositeStrategy. The sender  must use benchmark nr. 13 to send data" << endl
       << "13 : benchmarks for sending NMM buffers and events using a R/U CompositeStrategy. The receiver must use benchmark nr. 12 to receive data" << endl
       << "14 : benchmarks for sending NMM buffers and events using a TCPStrategy. The sender  must use benchmark nr. 15 to send data" << endl
       << "15 : benchmarks for sending NMM buffers and events using a TCPStrategy. The receiver must use benchmark nr. 14 to receive data" << endl
       << endl;
}

int main(int argc, char** argv) {
  try {
    int verbose = 0;
    BenchmarkReceiver::Mode jitter_mode = BenchmarkReceiver::AVERAGE;
    
    bool disable_registry_optimization = false;
    
    const char *opt_string = "vhi:p:H:I:b:e:l:j:J:r:Ps:oc";
    struct option options [] = {
      {"iteration",   1,NULL,'i'},
      {"port",        1,NULL,'p'},
      {"host",        1,NULL,'H'},
      {"host2",       1,NULL,'I'},
      {"buffer",      1,NULL,'b'},
      {"loss",        1,NULL,'l'},
      {"event",       1,NULL,'e'},
      {"paper",       1,NULL,'P'},
      {"jitter",      1,NULL,'j'},
      {"jitter-mode", 1,NULL,'J'},
      {"rate",        1,NULL,'r'},
      {"stringsize",  1,NULL,'s'},
      {"disable-optimization", 0,NULL,'o'},
      {"constant",    0,NULL,'c'},
      {"verbose",     0,NULL,'v'},
      {"help",        0,NULL,'h'},
      {NULL,          0,NULL,0}
    };
    int ch,long_option_index;
    
    while((ch=getopt_long(argc,argv,opt_string,options,&long_option_index))!=EOF){
      // which option did we get?
      switch (ch) {
      case 'v':
        verbose++;
        break;
      case 'p':
        port = atoi(optarg);
        break;
      case 'i':
        iteration = atoi(optarg);
        break;
      case 'H':
        host = optarg;
        break;
      case 'I':
	host2= optarg;
	break;
      case 'h':
        usage(argv[0]);
        exit(0);
      case 'b':
        buffer_size = atoi(optarg);
        break;
      case 's':
        string_size = atoi(optarg);
        break;
      case 'e':
        event_interval = atoi(optarg);
        break;
      case 'l':
        loss_interval = atoi(optarg);
        break;
      case 'j':
        jitter_samples = atoi(optarg);
        break;
      case 'J':
        switch(optarg[0]) {
        case 's':
          jitter_mode = BenchmarkReceiver::SEQUENCE_NUMBERS;
          break;
        case 'a':
          jitter_mode = BenchmarkReceiver::AVERAGE;
          break;
        case 'v':
          jitter_mode = BenchmarkReceiver::VARIANCE;
          break;
        default:
          usage(argv[0]);
          throw Exception("Invalid jitter benchmark mode");
        }
        break;
      case 'o':
        disable_registry_optimization = true;
        break;
      case 'r':
        max_rate = atoi(optarg) * 1024 * 1024;
        break;
      case 'P':
        mnsa2005_paper_benchmark = true;
        break;
      case 'c':
        constant_rate = true;
        break;
      default:
        usage(argv[0]);
        throw Exception("Invalid option");
      }
    }

    /* handle *Stream output */
    if (verbose < 1) {
      NamedObject::getGlobalInstance().setErrorStream(0, NamedObject::ALL_LEVELS);
    }
    if (verbose < 2) {
      NamedObject::getGlobalInstance().setWarningStream(0, NamedObject::ALL_LEVELS);
    }
    if (verbose < 3) {
      NamedObject::getGlobalInstance().setMessageStream(0, NamedObject::ALL_LEVELS);
    }
    if (verbose < 4) {
      NamedObject::getGlobalInstance().setDebugStream(0, NamedObject::ALL_LEVELS);
    }

    // check for filename
    if ( argc - optind < 1 ) {
      usage(argv[0]);
      throw Exception("No benchmark specified " );
    }

    benchmark = atoi(argv[optind]);

    switch (benchmark) {
    case 1:
      cerr << "Start serialization benchmark" << endl;
      serializationBenchmark();
      break;
    case 2:
      cerr << "Start connection benchmark" << endl;
      interfaceConnectionBenchmark(argc, argv);
      break;
    case 3:
      cerr << "Start method call benchmark" << endl;
      methodCallBenchmark();
      break;
    case 31:
      cerr << "Start method call benchmark server" << endl;
      methodCallBenchmarkReceiver(port);
      break;
    case 32:
      cerr << "Start virtual method call benchmark" << endl;
      methodCallBenchmarkLocal();
      break;
    case 33:
      cerr << "Start method call benchmark with strings" << endl;
      methodCallBenchmarkWithStrings();
      break;
    case 4:
      cerr << "Start listening on port " << port << endl;
      cerr << "The sender must use benchmark number 7" << endl;
      readFromNetwork(port);
      break;
    case 5:
      cerr << "Start listening on port " << port << endl;
      cerr << "The sender must use benchmark number 8" << endl;
      readFromNetwork2(port);
      break;
    case 6:
      cerr << "Start listening on port " << port << endl;
      cerr << "The sender must use benchmark number 9" << endl;
      readFromNetwork3(port);
      break;
    case 7:
      cerr << "Start write to port " << port << endl;
      writeToNetwork1();
      break;
    case 8:
      cerr << "Start write to port " << port << endl;
      writeToNetwork2();
      break;
    case 9:
      cerr << "Start write to port " << port << endl;
      writeToNetwork3();
      break;
    case 10:
      cerr << "Start listening on port " << port << " and " << port +1 << endl;
      cerr << "The sender must use benchmark number 11" << endl;
      readFromNetwork4(port, ICompositeStrategy::MODE_RELIABLE_RELIABLE, jitter_mode);
      break;
    case 11:
      cerr << "Start writing on port " << port << " and " << port +1 << endl;
      cerr << "The receiver must use benchmark number 10" << endl;
      if(mnsa2005_paper_benchmark) {
        for(real_event_interval=min_event_interval; real_event_interval<=max_event_interval; real_event_interval += inc_event_interval) {
          event_interval = static_cast<unsigned int>(1.0 / real_event_interval);
          writeToNetwork4(true, ICompositeStrategy::MODE_RELIABLE_RELIABLE);
        }
      } else {
        writeToNetwork4(true, ICompositeStrategy::MODE_RELIABLE_RELIABLE);
      }
      break;
    case 12:
      cerr << "Start listening on port " << port << " and " << port +1 << endl;
      cerr << "The sender must use benchmark number 13" << endl;
      readFromNetwork4(port, ICompositeStrategy::MODE_RELIABLE_UNRELIABLE, jitter_mode);
      break;
    case 13:
      cerr << "Start writing on port " << port << " and " << port +1 << endl;
      cerr << "The receiver must use benchmark number 12" << endl;
      if(mnsa2005_paper_benchmark) {
        for(real_event_interval=min_event_interval; real_event_interval<=max_event_interval; real_event_interval += inc_event_interval) {
          event_interval = static_cast<unsigned int>(1.0 / real_event_interval);
          writeToNetwork4(true, ICompositeStrategy::MODE_RELIABLE_UNRELIABLE);
        }
      } else {
        writeToNetwork4(true, ICompositeStrategy::MODE_RELIABLE_UNRELIABLE);
      }
      break;
    case 14:
      cerr << "Start listening on port " << port << " and " << port +1 << endl;
      cerr << "The sender must use benchmark number 15" << endl;
      readFromNetwork4(port, ICompositeStrategy::MODE_RELIABLE_RELIABLE, jitter_mode);
      break;
    case 15:
      cerr << "Start writing on port " << port << " and " << port +1 << endl;
      cerr << "The receiver must use benchmark number 14" << endl;
      if(mnsa2005_paper_benchmark) {
        for(real_event_interval=min_event_interval; real_event_interval<=max_event_interval; real_event_interval += inc_event_interval) {
          event_interval = static_cast<unsigned int>(1.0 / real_event_interval);
          writeToNetwork4(false, ICompositeStrategy::MODE_RELIABLE_RELIABLE);
        }
      } else {
        writeToNetwork4(false, ICompositeStrategy::MODE_RELIABLE_RELIABLE);
      }
      break;
    case 16:
      cerr << "Start connecting " << iteration  << " nodes on " << host << " and " << host2 << endl;
      graphConnectionBenchmark(argc, argv, host, host2, disable_registry_optimization);
      break;
    default:
      cerr << "Unknown benchmark " << endl;
      usage(argv[0]);
      break;
    }
  } catch (const Exception& e) {
    cerr << e << endl;
  }
}
