/* NMM - Network-Integrated Multimedia Middleware
 *
 * Copyright (C) 2002-2005
 *                    NMM work group,
 *                    Computer Graphics Lab,
 *                    Saarland University, Germany
 *                    http://www.networkmultimedia.org
 *
 * Maintainer:        Florian Winter <fw@graphics.cs.uni-sb.de>
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or (at your option) any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307
 * USA
 */


#include "FormatStrategyMap.hpp"

namespace NMM {
  
  FormatStrategyMap::FormatStrategyMap()
  {}
  
  FormatStrategyMap::~FormatStrategyMap()
  {}
  
  void FormatStrategyMap::addSupportedFormat(const string& strategy_type, const Format& format) {
    __format_map.push_back(pair<Format, string>(format, strategy_type));
  }
  
  list<string> FormatStrategyMap::getSupportedStrategies(const Format& format) const {
    list<string> result;
    
    for (FormatMapConstIterator i(__format_map.begin());
         i != __format_map.end();
         ++i) {      
      Format* intersection = format.createIntersectionFormat(&i->first);
      if (intersection) {
        result.push_back(i->second); 
      }
    }
    
    result.unique();
    return result;
  }

  list<Format*> FormatStrategyMap::getSupportedFormats(const string& strategy_type) {
    list<Format*> result;
    
    for (FormatMapIterator i(__format_map.begin());
         i != __format_map.end();
         ++i) {
      if (i->second == strategy_type) {
	result.push_back(&i->first); 
      }
    }
    
    result.unique();
    return result;
  }
  
  bool FormatStrategyMap::isSupported(const Format& format, const string& strategy_type) {
    for (FormatMapConstIterator i(__format_map.begin());
         i != __format_map.end();
         ++i) {
      if (i->second == strategy_type) {
        Format* intersection = format.createIntersectionFormat(&i->first);
        if (intersection) {
          return true;
        }
      }
    }
    
    return false;
  }
  
  void FormatStrategyMap::removeStrategyType(const string& strategy_type) {
    for (FormatMapIterator i(__format_map.begin());
         i != __format_map.end();
         ++i) {
      if (i->second == strategy_type) {
        i = __format_map.erase(i);
      }
    }
  }
  
  void FormatStrategyMap::display(ostream& out) {
    /* Begin> Find all formats */
    Format top("*/*");
    for (FormatMapConstIterator i(__format_map.begin());
         i != __format_map.end();
         ++i) {
      out << "==== Format: " << endl
          << i->first << endl
          << "==== Supports " << i->second << endl;
    }
    /* End< */
  }
}
