/* NMM - Network-Integrated Multimedia Middleware
 *
 * Copyright (C) 2002-2005
 *                    NMM work group,
 *                    Computer Graphics Lab,
 *                    Saarland University, Germany
 *                    http://www.networkmultimedia.org
 *
 * Maintainer:        Michael Repplinger <repplinger@cs.uni-sb.de>
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or (at your option) any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307
 * USA
 */


#ifndef __NMM_PROXYCHANNEL_HPP
#define __NMM_PROXYCHANNEL_HPP

#include "CommunicationChannel.hpp"

#include "nmm/interfaces/base/ITransportStrategy.hpp"

namespace NMM {

  /** This Communication Channel represents the connection between two Jacks.
   */
  class InstreamChannel : public CommunicationChannel {
  public:
    /** @name Constructor & Destructor
     */
    //@{
    /** Constructor
     */
    InstreamChannel();

    /** Constructor that accepts an Interface to Input and Output Jack
     */
    InstreamChannel(IInputJack& input_jack, IOutputJack& output_jack)
      throw (const BindingException&);

    /** Destructor
     */
    ~InstreamChannel();

    /** Copy-Constructor
     */
    InstreamChannel(const InstreamChannel&);

    /** virtual Copy-Constructor
     */
    InstreamChannel* copy() const {return new InstreamChannel(*this); }
    //@}

    /** @name Configure the Communication Channel
     */
    //@{
    /** @name Configure the Communication Channel
     */
    //@{
    /** This method returns a list of all TransportStrategies availble for the connection
     */
    virtual const list<string>& getStrategyList() const;
  
    /** This method returns a list of all TransportStrategies availble for the connection
     */
    virtual       list<string>& getStrategyList();
  
    /** This method sets the TransportStrategy that should be used.
     */
    virtual void  setTransportStrategy(const string& strategy_type);
    
    /** This method sets the TransportStrategy that should be used.
     * Creates a CompositeStrategy with different strategies for buffers and events.
     */
    virtual void  setTransportStrategy(const string& event_strategy_type, const string& buffer_strategy_type);
    
    /** This method sets the ValueStream that should be used in the TransportStrategy
     */
    virtual void  setValueStream(const IValueStream&, const OValueStream& );
    //@}

    /** @name Access Communication Channel information
     */
    //@{
    /** This method retursn the Interfaces of the InputJack's TranportStrategy
     */
    virtual list<Interface*>& getInterfaceList();

    /** This method retursn the Interfaces of the OutputJack's TranportStrategy
     */
    virtual list<Interface*>& getPartnerInterfaceList();
    //@}

  private: //private methods
    void registerAttributes();
    void createStrategyList();
    void updateAttributes();
    void updateIStrategies();
    void registerStrategyType(TSAddress::BindingType);
  
  private:
    TValue<IInputJack*>     __input_jack;             ///< Interface to the InputJack
    TValue<IOutputJack*>    __output_jack;            ///< Interface to the OutputJack
    ITransportStrategy_var __input_jack_strategy;     ///< Interface to the InputJack's transport strategy
    ITransportStrategy_var __output_jack_strategy;    ///< Interface to the OutputJack's transport strategy
    bool __i_strategies_valid;                        ///< is true if all interfaces are valid
    bool __strategy_list_valid;                       ///< is true if the strategy list is valid
  
  public:
    /** This method returns the TypeInfo for this class.
     */
    static TypeInfo getStaticTypeInfo();
  };
  
}

#endif
