/* NMM - Network-Integrated Multimedia Middleware
 *
 * Copyright (C) 2002-2005
 *                    NMM work group,
 *                    Computer Graphics Lab,
 *                    Saarland University, Germany
 *                    http://www.networkmultimedia.org
 *
 * Maintainer:        Michael Repplinger <repplinger@cs.uni-sb.de>
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or (at your option) any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307
 * USA
 */

#include <unistd.h> // for close()

#include "ObjectAcceptor.hpp"
#include "nmm/base/registry/ReferenceManager.hpp"
#include "nmm/base/registry/LocalRegistry.hpp"
#include "nmm/base/registry/PluginManager.hpp"
#include "nmm/base/TSFactory.hpp"
#include "nmm/utils/utils.hpp"

namespace NMM {

ObjectAcceptor::ObjectAcceptor() {

}

void ObjectAcceptor::connectToObject(const NMMReference& reference)
    throw (const BindingException& )
{
  /* search the NMMObject with given reference */
  NMMObject* object = SReferenceManager::getInstance().getNMMObject(reference);
  if (!object)
    throw BindingException("Unknwon reference");

  /* The current sender must be a TransportStrategy */
  TransportStrategy* strategy = dynamic_cast<TransportStrategy*>(getCurrentSender());
  if (!strategy)
    throw BindingException("Unkown sender");

  /* Sets the sender to the specific NMMObject */
  strategy->setEventReceiver(object);
}

list<string> ObjectAcceptor::getServiceList() const
{
  return SReferenceManager::getInstance().getServiceList();
}

void ObjectAcceptor::connectToService(const string& service)
    throw (const BindingException&)
{
  /* search the NMMObject with given service name */
  NMMObject* object = SReferenceManager::getInstance().getService(service);
  if (!object)
    throw BindingException("Unknwon service " + service);

  /* The current sender must be a TransportStrategy */
  TransportStrategy* strategy = dynamic_cast<TransportStrategy*>(getCurrentSender());
  if (!strategy)
    throw BindingException("Unkown sender");

  /* Sets the sender to the specific NMMObject */
  strategy->setEventReceiver(object);
}


}//end namespace


