/* NMM - Network-Integrated Multimedia Middleware
 *
 * Copyright (C) 2002-2005
 *                    NMM work group,
 *                    Computer Graphics Lab,
 *                    Saarland University, Germany
 *                    http://www.networkmultimedia.org
 *
 * Maintainer:        NMM work group,
 *                    http://www.networkmultimedia.org
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or (at your option) any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 * 
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307
 * USA
 */


#ifndef _NMM_CPARAMETER_HPP
#define _NMM_CPARAMETER_HPP

#include "Parameter.hpp"
#include "Format.hpp"

namespace NMM 
{
  
/**
   This class stores only pointers to other Formats.
   With this class you can describe recursive or "compound" formats like MPEG4.
**/

  class CParameter : public Parameter 
  {

  protected:
    //! The STL-List of all the recursive included formats.
    list<Format *> m_value_list;
    //! Private Variable for browsing the STL-List Format_List
    mutable format_list::const_iterator m_counter;
    
    void addValueInternal(Format* format);
    
  public:
    /**
       The default Constructor.
       \param string pName: the name of the parameter.
    **/
    CParameter(const string & parameter_name = "");
	
    /**
       Constructor with a parameter that is directly added to the Format_List.
       \param string pName: the name of the parameter.
       \param Format * : pointer to a format.
    **/
    CParameter(const string & parameter_name , Format * format);
	
    /**
       The default Destructor.
    */
    virtual ~CParameter();
	
    virtual void addValue(void * pValue);
    
    virtual void addValue(Format *value);

    virtual void showParameter() const;
    
    virtual ostream& showParameter(ostream&) const;
    
    virtual ostream& showParameterVerbose(ostream&) const;

    virtual bool isValueIncluded(void *pValue) const;
    
    virtual bool isValueIncluded(const Format *value) const;
    
    virtual void *nextValue() const;
    
    virtual uint getNumberOfValues() const;

    virtual void * getFirstValue() const;

    virtual Parameter * duplicate() const;

    virtual float getWeight(void *value) const;

    virtual Parameter *getIntersection(const Parameter *reference) const;

//     virtual CParameter *getIntersectionFormat(const Parameter *reference) const;

    virtual Format * getFormat(const string & name) const;

    virtual bool isParameterIdentical(const Parameter *reference) const;

    virtual bool isParameterMatching(const Parameter * refernce) const;

    virtual float getStandardizedWeight() const;

    virtual const format_list & getFPointerValueList() const;

    virtual void fromStream(IValueStream& );
    virtual void toStream(OValueStream&) const  ;
  };
}


#endif
