/* NMM - Network-Integrated Multimedia Middleware
 *
 * Copyright (C) 2002-2005
 *                    NMM work group,
 *                    Computer Graphics Lab,
 *                    Saarland University, Germany
 *                    http://www.networkmultimedia.org
 *
 * Maintainer:        NMM work group,
 *                    http://www.networkmultimedia.org
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or (at your option) any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 * 
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307
 * USA
 */


#ifndef _NMM_SPARAMETER_HPP
#define _NMM_SPARAMETER_HPP

#include "Parameter.hpp"

namespace NMM {

/**
   This class stores values for STRING Parameter.
**/

  class SParameter : public Parameter 
  {
    
  private:
    //! The STL-List to store the STRINGs.
    map <string, float> m_value_list;
    //! Private Variable for browsing in the STL-List Value_List
//     mutable STRING_Iterator m_value_counter;
//     mutable STRING_Iterator m_pair_counter;
    mutable map<string, float>::const_iterator m_value_counter;
    mutable map<string, float>::const_iterator m_pair_counter;

    //! The default value
    string m_default_value;
    
  public:
	
    /**
       The default Constructor.
       \param string parameter_name: the name of the parameter.
    */
    SParameter(const string & parameter_name = "");
    
    /**
       Constructor with a parameter pValue that is directly 
       added to the m_value_list.
       \param string parameter_name: the name of the parameter.
       \param string pValue: a value.
       \param float pWeight: the weight of the value.
    */
    SParameter(const string & parameter_name , string pValue, float pWeight = DEFAULT_WEIGHT);
    
    /**
       The default Destructor.
    */
    virtual ~SParameter() {};
    
    virtual void addValue(void *pValue);

    virtual void addValue(void *pValue, float pWeight = DEFAULT_WEIGHT);
    
    virtual void addValue(const string & value, float weight = DEFAULT_WEIGHT);

    virtual void showParameter() const;
    
    virtual ostream& showParameter(ostream&) const;
    
    virtual ostream& showParameterVerbose(ostream&) const;

    virtual bool isValueIncluded(void *pValue) const;

    virtual bool isValueIncluded(const string & value) const;
    
    virtual void * nextValue() const;

    virtual map<string, float>::const_iterator nextPair() const;

    virtual Parameter * duplicate() const;

    virtual float getWeight(void *pValue) const;

    virtual uint getNumberOfValues() const;

    virtual bool isParameterIdentical(const Parameter *reference) const;

    virtual bool isParameterMatching(const Parameter *reference) const;

    virtual void *getFirstValue() const;

    virtual Parameter *getIntersection(const Parameter *reference) const;

    virtual float getStandardizedWeight() const;

    virtual void setDefaultValue(const string & value);

    virtual const string & getDefaultValue() const;

    virtual const string_list getStringValueList() const;

    virtual void fromStream(IValueStream& ) ;
    virtual void toStream(OValueStream&) const  ;
  };
}
#endif
