/* NMM - Network-Integrated Multimedia Middleware
 *
 * Copyright (C) 2002-2005
 *                    NMM work group,
 *                    Computer Graphics Lab,
 *                    Saarland University, Germany
 *                    http://www.networkmultimedia.org
 *
 * Maintainer:        Florian Winter <fw@graphics.cs.uni-sb.de>
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or (at your option) any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307
 * USA
 */


#include "RTPGeneric.hpp"

#include <live/liveMedia.hh>
#include <live/GroupsockHelper.hh>
#include <live/BasicUsageEnvironment.hh>

namespace NMM {
  
  RTPGeneric::RTPGeneric() 
    : __cname("NMM"),
      __ttl(30),
      __buffer_size(65536),
      __max_frame_size(8192),
      __recv_buffer_size(2000000), // XXX: for testing
      __bandwidth(160),
      __scheduler(NULL),
      __env(NULL),
      __rtp_sock(NULL),
      __rtcp_sock(NULL)
  {
    __scheduler = BasicTaskScheduler::createNew();
    __env = BasicUsageEnvironment::createNew(*__scheduler);
  }
  
  RTPGeneric::~RTPGeneric() {
    /* new version of libLive claims to do that automatically */
//     if (__env) {
//       delete __env;
//       __env = NULL;
//     }
    
    if (__scheduler) {
      delete __scheduler;
      __scheduler = NULL;
    }
  }
  
  void RTPGeneric::setCNAME(const string& cname) {
    __cname = cname;
  }
  
  string RTPGeneric::getCNAME() const {
    return __cname;
  }
  
  void RTPGeneric::setTTL(const unsigned int& ttl) {
    __ttl = ttl;
  }
  
  unsigned int RTPGeneric::getTTL() const {
    return __ttl;
  }
  
  void RTPGeneric::setBufferSize(const size_t& buffer_size) {
    __buffer_size = buffer_size;
  }
  
  unsigned int RTPGeneric::getBufferSize() const {
    return __buffer_size;
  }
  
  void RTPGeneric::setMaxFrameSize(const size_t& max_frame_size) {
    __max_frame_size = max_frame_size;
  }
  
  size_t RTPGeneric::getMaxFrameSize() const {
    return __max_frame_size;
  }
  
  void RTPGeneric::setSessionBandwidth(const unsigned int& bandwidth) {
    __bandwidth = bandwidth;
  }
  
  size_t RTPGeneric::getSessionBandwidth() const {
    return __bandwidth;
  }
  
  void RTPGeneric::setReceiveBufferSize(const size_t& recv_buffer_size) {
    __recv_buffer_size = recv_buffer_size;
  }
  
  unsigned int RTPGeneric::getReceiveBufferSize() const {
    return __recv_buffer_size;
  }
  
  void RTPGeneric::setRTPSocket(Groupsock* sock) {
    if (__rtp_sock) {
      delete __rtp_sock;
    }
    
    __rtp_sock = sock;
  }
  
  void RTPGeneric::setRTCPSocket(Groupsock* sock) {
    if (__rtcp_sock) {
      delete __rtcp_sock;
    }
    
    __rtcp_sock = sock;
  }
}
