/* NMM - Network-Integrated Multimedia Middleware
 *
 * Copyright (C) 2002-2005
 *                    NMM work group,
 *                    Computer Graphics Lab,
 *                    Saarland University, Germany
 *                    http://www.networkmultimedia.org
 *
 * Maintainer:        Michael Repplinger <repplinger@cs.uni-sb.de>
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or (at your option) any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307
 * USA
 */


#ifndef NMM_TCP_ACCEPTOR_HPP
#define NMM_TCP_ACCEPTOR_HPP

#include "nmm/utils/net/NetReactor.hpp"
#include "nmm/utils/net/SockAcceptor.hpp"
#include "nmm/base/ObjectAcceptor.hpp"
#include "nmm/base/registry/ClientRegistry.hpp"

namespace NMM {


class TCPAcceptor : public ObjectAcceptor, public DispatchedNetHandler, protected SockAcceptor {
public:
/** @name Constructor & Destructor
  */
//@{
/** Constructor, that accepts a port to listen for incoming connections.
  */
  TCPAcceptor(unsigned int port = ClientRegistry::default_port);

/** Constructor, that accepts a port-range. The Acceptor tries to listen one of these ports.
  */
  TCPAcceptor(unsigned int begin_port, unsigned int end_port);

/** Destructor
  */
  virtual ~TCPAcceptor();
//@}

/** @name Connection
  */
//@{
/** If this mehtod is called a new connection is established
  */
  virtual NetResult receiveData(int);
//@}

private:
  INetAddr addr;                      ///< The INetAddr, that is listen for new connections 
};

}

#endif


