/* NMM - Network-Integrated Multimedia Middleware
 *
 * Copyright (C) 2002-2005
 *                    NMM work group,
 *                    Computer Graphics Lab,
 *                    Saarland University, Germany
 *                    http://www.networkmultimedia.org
 *
 * Maintainer:        Michael Repplinger <repplinger@cs.uni-sb.de>
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or (at your option) any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 * 
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307
 * USA
 */


#include <iostream>
#include "LTDll.hpp"

namespace NMM {

LTDll::LTDll() : handle(0) {
  int retval = lt_dlinit();
  if (retval != 0) {
    cerr << retval << " error occured" << endl;
    __is_valid = false;
  } else {
    __is_valid = true;
  }
}

LTDll::~LTDll(){
  try {
    close();
  } catch (const Exception& e) {
    GLOBAL_ERROR_STREAM(e << endl);
  }
  lt_dlexit();
}

void LTDll::open(const string& n, int )
  throw (const Exception&)
{
  if (handle) {
    try {
      close();
    } catch (const Exception& e) {
      GLOBAL_ERROR_STREAM(e << endl);
    }
  }

  name = n;
  handle = lt_dlopen(name.c_str());

  if (!handle) {
    throw Exception("Error while loading shared library '" + name +"'. Error message :" + lt_dlerror());
  }
}

void* LTDll::resolve(const string& symbol) {
  if (handle != 0)
    return lt_dlsym(handle, const_cast<char*> (symbol.c_str()));
  return 0;
}

void LTDll::close()
  throw (const Exception&)
{
  if (!handle)
    return;

  if (lt_dlclose(handle) != 0) {
    handle = 0;
    throw Exception("Error while closing shared library '" + name +"'. Error message :" + lt_dlerror());
  }

  handle = 0;
}
} //end namespace
