/* NMM - Network-Integrated Multimedia Middleware
 *
 * Copyright (C) 2002-2005
 *                    NMM work group,
 *                    Computer Graphics Lab,
 *                    Saarland University, Germany
 *                    http://www.networkmultimedia.org
 *
 * Maintainer:        
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or (at your option) any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307
 * USA
 */

#ifndef _NMM_RM_CONFIGMANAGER_HPP_
#define _NMM_RM_CONFIGMANAGER_HPP_

#include "nmm/utils/NamedObject.hpp"

#include <string>
#include <list>
#include <map>

namespace NMM {

class RMConfigManager : public NamedObject {

    string net_file;
    string mesh_file;

    string path;
    string filename;

    //   node        type   RMMesh
    map<string, map<string, string > > config;  // meshes

    //   host        speed interface
    map<string, pair<float,uint> >  net_config; // network
//    map<string, float>  net_config; // network

    // interface speeds
    map<uint, list<float> > interface_speeds;

    map<uint,float> max_speeds; // used for caching max_speeds

    uint interface_n;  // number of interfaces

    void testDirs(); // looks for the needed directories and creates them if they don't exist

  public:

    RMConfigManager();

    void setMesh(string node_name, string format_id, string mesh);
    string getMesh(string node_name, string format_id);
    list<pair<string,string> > getMeshes(string node_name);

    float getBandwidth(string host);
    void setBandwidth(string host, float speed, uint interf);
    bool save(); // save to ~/.nmm/HOSTNAME/ network and meshes
    bool load(); // load from ~/.nmm/HOSTNAME/ network and meshes

    // get number of available interfaces on this host
    uint getInterfaces();
    void setInterfaces(uint interf_n);

    // get maximum Bandwidth of interface interf
    float getMaxBW(uint interf);


};

}

#endif
