/* NMM - Network-Integrated Multimedia Middleware
 *
 * Copyright (C) 2002-2005
 *                    NMM work group,
 *                    Computer Graphics Lab,
 *                    Saarland University, Germany
 *                    http://www.networkmultimedia.org
 *
 * Maintainer:        
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or (at your option) any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307
 * USA
 */

#ifndef _NMM_SLOPE_HPP_
#define _NMM_SLOPE_HPP_

#include <string>
#include <map>
#include "nmm/utils/NamedObject.hpp"
#include "Range.hpp"
#include "RMVertex.hpp"

namespace NMM {

/**
 * This class contains slopes for Ranges in different quality dimensions
 */

class Slope  : public NMM::SerializedValue, public NamedObject {

      //The attributes
      //              quality-dimension  at value    slope, no of samples
      NMM::TValue<map<string,        map<Range    ,pair<float,uint> > > >  slopes;

    public:

      Slope() :  NMM::SerializedValue("Slope", "NMM") , NamedObject("Slope") {
        registerValues();
      }

      Slope(const Slope& s);

      Slope* copy() const { return new Slope(*this); } //virtual CopyConstructor

      Result addSlope(const string& dim, Range start, float val);
      float getSlope(const string& dim, Range start);
      float computeSlope(const string& dim, RMVertex*, RMVertex*);

      void registerValues() {
        insertArgument(&slopes);
      }

};

OValueStream& operator<<(NMM::OValueStream& ov, const map<string,map<Range    ,pair<float,uint> > >& slope_map);
IValueStream& operator>>(NMM::IValueStream& iv, map<string,map<Range    ,pair<float,uint> > >& slope_map);
OValueStream& operator<<(NMM::OValueStream& ov, const map<Range    ,pair<float,uint> >& range_map);
IValueStream& operator>>(NMM::IValueStream& iv, map<Range    ,pair<float,uint> >& range_map);
OValueStream& operator<<(NMM::OValueStream& ov, const pair<float,uint>& slope_pair);
IValueStream& operator>>(NMM::IValueStream& iv, pair<float,uint>& slope_pair);


}

#endif
