/* NMM - Network-Integrated Multimedia Middleware
 *
 * Copyright (C) 2002-2005
 *                    NMM work group,
 *                    Computer Graphics Lab,
 *                    Saarland University, Germany
 *                    http://www.networkmultimedia.org
 *
 * Maintainer:        Michael Repplinger <repplinger@cs.uni-sb.de>
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or (at your option) any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 * 
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307
 * USA
 */


#ifndef _NMM_STDOVALUESTREAM_HPP_
#define _NMM_STDOVALUESTREAM_HPP_


#include "ValueStream.hpp"

namespace NMM {

/** This OValueStream reads values, that are written with the StdOValueStream.
  */
class StdOValueStream : public OValueStream {
public:
/** @name Constructor & Destructor
  */
//@{
/** Constructor
  */
  StdOValueStream();

/**
  */
  StdOValueStream(std::ostream& );

/** Destructor
  */
  virtual ~StdOValueStream();

/** virtual copy constructor
  */
  virtual StdOValueStream* copy() const { return new StdOValueStream(); }
//@}

/** @name The operators
  */
//@{
/** This method writes a string i to the stream and returns a reference to the stream.
  */  
  virtual StdOValueStream& operator<<(const string& i)         ;

/** This method writes an u_int16_t i to the stream and returns a reference to the stream.
  */
  virtual StdOValueStream& operator<<(const u_int16_t& )      ;

/** This method writes an int16_t i to the stream and returns a reference to the stream.
  */
  virtual StdOValueStream& operator<<(const int16_t& i)      ;

/** This method writes an u_int32_t i to the stream and returns a reference to the stream.
  */
  virtual StdOValueStream& operator<<(const u_int32_t& i)    ;

/** This method writes an int32_t i to the stream and returns a reference to the stream.
  */
  virtual StdOValueStream& operator<<(const int32_t& i)        ;

/** This method writes an u_int64_t i to the stream and returns a reference to the stream.
  */
  virtual StdOValueStream& operator<<(const u_int64_t& i)     ;

/** This method writes an int64_t i to the stream and returns a reference to the stream.
  */
  virtual StdOValueStream& operator<<(const int64_t& i)       ;

/** This method writes an bool i to the stream and returns a reference to the stream.
  */
  virtual StdOValueStream& operator<<(const bool& i)          ;

/** This method writes an unsigned long i to the stream and returns a reference to the stream.
  */
  virtual StdOValueStream& operator<<(const unsigned long& i)  ;

/** This method writes an long i to the stream and returns a reference to the stream.
  */
  virtual StdOValueStream& operator<<(const long& i)          ;

/** This method writes an double i to the stream and returns a reference to the stream.
  */
  virtual StdOValueStream& operator<<(const double& i)        ;

/** This method writes an float i to the stream and returns a reference to the stream.
  */
  virtual StdOValueStream& operator<<(const float& i)         ;

/** This method writes an char i to the stream and returns a reference to the stream.
  */
  virtual StdOValueStream& operator<<(const char& i)          ;

/** This method writes an unsigend char i to the stream and returns a reference to the stream.
  */
  virtual StdOValueStream& operator<<(const unsigned char& i)  ;

/** This method must be called if no other values are written.
  */
  virtual void writeData() ;
//@}

/** @name Write char-array
  */
//@{
/** This method writes an char buffer of given size to the stream.
  * \param buffer Poitner to the char array.
  * \param size Size of the buffer.
  */
  virtual void writeBuffer(const char* buffer, unsigned int size);

/** This method writes an unsigned char buffer of given size to the stream.
  * \param buffer Poitner to the unsigned char array.
  * \param size Size of the buffer.
  */
  virtual void writeBuffer(const unsigned char* buffer, unsigned int size);
//@}

/** @name Write TypeInfo
  */
//@{
/** This method starts a new compound type and tries to map given information to a magic number.
  * If no such magic number exist, the type information are written.
  * \param type The type name of the struct
  * \param ns namespace of the struct
  */
  virtual void beginType(const string& type, const string& ns = "std");

/** This method set the end flag of a current type and must be called after all attributes of a
  * complex struct are written.
  */
  virtual void endType() ;

/** This method returns the type name of this ValueStream
  */
  virtual const string& getOValueStreamType() const { return __o_value_stream_type; }
//@}
private:
  void writeBlanks();

private:
  ostream*      __std_stream;              ///< StdStream used to wirte data;
  unsigned int  __tab_counter;             ///< counter for space added between each type;
public:
  static const string     __o_value_stream_type;           ///< static string with the class name
};



}

#endif // _NMM_ValueStream_HPP_


