/* NMM - Network-Integrated Multimedia Middleware
 *
 * Copyright (C) 2002-2005
 *                    NMM work group,
 *                    Computer Graphics Lab,
 *                    Saarland University, Germany
 *                    http://www.networkmultimedia.org
 *
 * Maintainer:        Michael Repplinger <repplinger@cs.uni-sb.de>
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or (at your option) any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307
 * USA
 */


#include "NetStream.hpp"
#include "nmm/utils/net/Sock.hpp"

#include <errno.h>       // for errnos
#include <sys/socket.h>  // for inet_ntoa
#include <sys/types.h>
#include <fstream>

namespace NMM {
  
  NetStream::NetStream(Sock& sock)
    : __stream(0),
      __sock(sock),
      __write_counter(0),
      __read_counter(0)
  { /*__stream = &cerr;*/ }

  NetStream::NetStream(const NetStream& other)
    : IStream(other),
      OStream(other),
      __stream(other.__stream),
      __sock(other.__sock),
      __write_counter(0),
      __read_counter(0)
  { /*__stream = &cerr;*/ }
  
  NetStream::~NetStream() {}
  
  OStream& NetStream::operator<<(const u_int16_t& value) {
    writeValue(&value, sizeof(value));
    return *this;
  }
  
  OStream& NetStream::operator<<(const u_int32_t& value) {
    writeValue(&value, sizeof(value));
    return *this;
  }
  
  OStream& NetStream::operator<<(const u_int64_t& value) {
    writeValue(&value, sizeof(value));
    return *this;
  }
  
  OStream& NetStream::operator<<(const int16_t&value) {
    writeValue(&value, sizeof(value));
    return *this;
  }
  
  OStream& NetStream::operator<<(const int32_t& value) {
    writeValue(&value, sizeof(value));
    return *this;
  }
  
  OStream& NetStream::operator<<(const int64_t& value) {
    writeValue(&value, sizeof(value));
    return *this;
  }
  
  OStream& NetStream::operator<<(const bool& value) {
    writeValue(&value, sizeof(value));
    return *this;
  }
  OStream& NetStream::operator<<(const unsigned long& value) {
    writeValue(&value, sizeof(value));
    return *this;
  }
  
  OStream& NetStream::operator<<(const long& value) {
    writeValue(&value, sizeof(value));
    return *this;
  }
  
  OStream& NetStream::operator<<(const double& value) {
    writeValue(&value, sizeof(value));
    return *this;
  }
  OStream& NetStream::operator<<(const float& value) {
    writeValue(&value, sizeof(value));
    return *this;
  };
  OStream& NetStream::operator<<(const char& value) {
    writeValue(&value, sizeof(value));
    return *this;
  }

  OStream& NetStream::operator<<(const unsigned char& value) {
    writeValue(&value, sizeof(value));
    return *this;
  }

  OStream& NetStream::operator<<(const string& value) {
    (*this) << value.size();
    
    if (value.size() == 0) {
      return *this;
    }
    
    writeValue(value.c_str(), value.size());
    return *this;
  }
  
  void NetStream::writeBuffer(const char* value, unsigned int size) {
    writeValue(value, size);
  }
  
  void NetStream::writeBuffer(const unsigned char* value, unsigned int size) {
    writeValue(value, size);
  }
  
  /*************************************/
  IStream& NetStream::operator>>(u_int16_t& value) {
    readValue(&value, sizeof(value));
    return *this;
  }
  IStream& NetStream::operator>>(u_int32_t& value) {
    readValue(&value, sizeof(value));
    return *this;
  }
  IStream& NetStream::operator>>(u_int64_t& value) {
    readValue(&value, sizeof(value));
    return *this;
  }
  IStream& NetStream::operator>>(int16_t& value ) {
    readValue(&value, sizeof(value));
    return *this;
  }
  IStream& NetStream::operator>>(int32_t& value) {
    readValue(&value, sizeof(value));
    return *this;
  }
  
  IStream& NetStream::operator>>(int64_t& value) {
    readValue(&value, sizeof(value));
    return *this;
  }
  IStream& NetStream::operator>>(bool&  value) {
    readValue(&value, sizeof(value));
    return *this;
  }
  IStream& NetStream::operator>>(unsigned long& value) {
    readValue(&value, sizeof(value));
    return *this;        
  }
  
  IStream& NetStream::operator>>(long&  value) {
    readValue(&value, sizeof(value));
    return *this;
  }
  
  IStream& NetStream::operator>>(double& value) {
    readValue(&value, sizeof(value));
    return *this;
  }
  IStream& NetStream::operator>>(float& value) {
    readValue(&value, sizeof(value));
    return *this;
  }
  IStream& NetStream::operator>>(char& value) {
    readValue(&value, sizeof(value));
    return *this;
  }
  IStream& NetStream::operator>>(unsigned char& value) {
    readValue(&value, sizeof(value));
    return *this;
  }
  IStream& NetStream::operator>>(string& value) {
    size_t size = 0;
    (*this) >> size;

    if (size == 0) {   //string is empty
      value.resize(0);
    } else {
      /* Added additional check before calling method resize, to avoid
       * needless memory allocation*/
      if (value.size() != size) {
        value.resize(size);
      }
      readValue(&value[0], size);
    }

    return *this;
  }

  void NetStream::readBuffer(char* value, unsigned int size ) {
    readValue(value, size);
  }

  void NetStream::readBuffer(unsigned char* value, unsigned int size ) {
    readValue(value, size);
  }

  bool NetStream::receivePacket() {
    return true;
  }

}
