/* NMM - Network-Integrated Multimedia Middleware
 *
 * Copyright (C) 2002-2005
 *                    NMM work group,
 *                    Computer Graphics Lab,
 *                    Saarland University, Germany
 *                    http://www.networkmultimedia.org
 *
 * Maintainer:        Marco Lohse <mlohse@cs.uni-sb.de>
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or (at your option) any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 * 
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307
 * USA
 */



#ifndef _NMM_CLOCK_HPP_
#define _NMM_CLOCK_HPP_

#include "Types.hpp"
#include <sys/time.h> // for gettimeofday()

namespace NMM {

  //! This is a first and simple implementation of a media stream clock.
  /*!
   * A clock object keeps on counting the nanoseconds since the time of 
   * its creation. In the constructor, its initial clock value is set to
   * zero. This clock uses the system-clock to compute the time value.
   * You can query the time of the creation of a clock object by calling
   * the getSystemStartTime - method.
   */
  class Clock {

  private:

    struct timeval systime;         //! The system time at the moment.
    struct timeval systemstarttime; //! The system time at the time
                                    //! the clock is generated.
    struct timeval diff;            //! Internally used to compute the 
                                    //! difference between both values.

  public:

    //! The main constructor.
    /*! 
     * Returns a clock object with a time value of zero.
     */
    Clock();

    //! The main destructor.
    ~Clock();

    //! Returns the clock's time value.
    Time getTime();

    /*!
     * Returns the starttime of the clock referring to the internal system
     * clock.
     */
    inline struct timeval getSystemStartTime() const {return systemstarttime;}
  };
}

#endif // _NMM_CLOCK_HPP_










