/* NMM - Network-Integrated Multimedia Middleware
 *
 * Copyright (C) 2002-2005
 *                    NMM work group,
 *                    Computer Graphics Lab,
 *                    Saarland University, Germany
 *                    http://www.networkmultimedia.org
 *
 * Maintainer:        Marco Lohse <mlohse@cs.uni-sb.de>
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or (at your option) any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 * 
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307
 * USA
 */



#ifndef _NMM_GENERIC_UNBUFFERED_SYNC_SINK_NODE_HPP_
#define _NMM_GENERIC_UNBUFFERED_SYNC_SINK_NODE_HPP_

#include "nmm/base/sync/GenericSyncSinkNode.hpp"

namespace NMM {

  //! This class is a superclass for synchronized sink nodes.
  /*!
   *  Synchronization can be switched on and off by the setSynchronized -
   *  method. By default the switch is turned off, so an application 
   *  that don't uses synchronization should not care about it.
   *  In synchronized mode, be careful that incoming buffers are 
   *  timestamped: The GenericUSyncSinkNode tries to present them according
   *  to their sync_time in the timestamps. 
   */
  class GenericUSyncSinkNode : public GenericSyncSinkNode {
  public:
    //! The main constructor.
    GenericUSyncSinkNode(const char* name,
			 const StreamQueue::Mode queue_mode 
			 = StreamQueue::DEFAULT_MODE,
			 const int size = StreamQueue::DEFAULT_SIZE);
    
    /** Check if sink is buffered (from ISynchronizedSink) */
    virtual bool isBuffered() const {
      return false;
    }
    
    //! The main destructor.
    virtual ~GenericUSyncSinkNode();
  };
}

#endif // _NMM_GENERIC_UNBUFFERED_SYNC_SINK_NODE_HPP_


