/* NMM - Network-Integrated Multimedia Middleware
 *
 * Copyright (C) 2002-2005
 *                    NMM work group,
 *                    Computer Graphics Lab,
 *                    Saarland University, Germany
 *                    http://www.networkmultimedia.org
 *
 * Maintainer:        Marco Lohse <mlohse@cs.uni-sb.de>
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or (at your option) any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 * 
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307
 * USA
 */



#ifndef _NMM_UNBUFFERED_SINK_CONTROLLER_HPP_
#define _NMM_UNBUFFERED_SINK_CONTROLLER_HPP_

#include "SinkController.hpp"

namespace NMM {

  class GenericUSyncSinkNode;
  class AudioVideoSynchronizer;

  //! This class represents a synchronization controller for sink nodes.
  /*!
   *  In addition to the isBufferValid - method from controller, a 
   *  sinkcontroller has a waitToPresent - method which allows it to
   *  tell the sink node how long it should wait before presenting the 
   *  next buffer.
   */
  class USinkController : public SinkController {
  private:
    GenericUSyncSinkNode* node;
    
  public:
    //! Constructor that only sets the mode.
    /*!
     *  The name is "Controller" as default.
     */
    USinkController(const IController::Mode mode);

    //! The main constructor.
    /*!
     *  Sets the name and the mode, default values are "Controller" and
     *  DEFAULT_MODE (OFFLINE).
     */
    USinkController(GenericUSyncSinkNode* parent = 0, 
		    const IController::Mode mode = Controller::DEFAULT_MODE);

    //! The main destructor.
    virtual ~USinkController();

    //! This method is by a Controller::isBufferValid - call. 
    /*!
     *  It checks if the buffer should be prepared for presenatation or discarded. 
     */
    virtual bool checkBufferValid(const Timestamp timestamp);

    //! Tells the node how long it should wait until presenting the next buffer.
    /*!
     *  This method is called by the GenericSyncSinkNode to get to know how long
     *  it has to wait until the optimal time for presentation of this buffer 
     *  will be reached.
     *  \param timestamp The timestamp of the buffer that should be presented.
     *  \return An interval which represents the time to wait.
     */
    Interval waitToPresent(const Timestamp timestamp); 
  };

}

#endif // _NMM_UNBUFFERED_SINK_CONTROLLER_HPP_
