/* NMM - Network-Integrated Multimedia Middleware
 *
 * Copyright (C) 2002-2005
 *                    NMM work group,
 *                    Computer Graphics Lab,
 *                    Saarland University, Germany
 *                    http://www.networkmultimedia.org
 *
 * Maintainer:        Marco Lohse <mlohse@cs.uni-sb.de>
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or (at your option) any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 * 
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307
 * USA
 */


#ifndef _NMM_USER_TIME_HPP_
#define _NMM_USER_TIME_HPP_

#include "Types.hpp"
#include <iosfwd>

namespace NMM {

  class OValueStream;
  class IValueStream;
  class Interval;
  class Time;
  
  //! Provides an easily readable time format. 
  /*!
   * Time and Interval are used for internal computations referring 
   * to time. UserTime can be used to display those values.
   */
  struct UserTime {
    int hour;       //!< The hours
    int min;        //!< The minutes
    int sec;        //!< The seconds
    int msec;       //!< The milliseconds

    //! The default constructor.
    UserTime() : hour(0), min(0), sec(0), msec(0) {};
    
    //! Creates a UserTime that has an equal value as a given Time.
    UserTime(const Time& time);
    
    //! Creates a UserTime that has an equal value as a given Interval.
    UserTime(const Interval& interval);
    
    //! Output UserTime to standard stream.
    /*!
     *  Outputs the formatted user time to a stream, optionally with seconds and
     *  microseconds.
     */
    void output(std::ostream& stream, bool with_seconds = false, bool with_microseconds = false) const;
  };

  //! The output operator for UserTime.
  /*!
   *  This is equivalent to calling output(stream, true, true).
   */
  std::ostream& operator<< (std::ostream& stream, const UserTime& time);

  /*!
   *  This method must be implemented if Events with a UserTime parameter
   *  should be transported over a net connection.
   */
  OValueStream& operator<<(OValueStream& ov, const UserTime&);

  /*!
   *  This method must be implemented if Events with a UserTime parameter
   *  should be transported over a net connection.
   */
  IValueStream& operator>>(IValueStream& s, UserTime&);

}

#endif // _NMM_USER_TIME_HPP_

