
#ifndef NMM_IMMBoxStateImpl_HPP
#define NMM_IMMBoxStateImpl_HPP

#include "nmm/NMMTypes.hpp"
#include "nmm/base/NMMObject.hpp"
#include "IMMBoxState.hpp"

#include "nmm/interfaces/base/INodeImpl.hpp"
#include "nmm/interfaces/video/osd/manager/ITextViewImpl.hpp"

namespace NMM
{

  class IMMBoxStateImpl  : public virtual NMMObject { 
  public:
    IMMBoxStateImpl() { addInterface(new IMMBoxState(*this)); registerEvents(); };

    virtual ~IMMBoxStateImpl() {};

  protected:

    virtual Result initialize(const string& message = "") = 0;

    virtual Result uninitialize() = 0;

    virtual Result create(const string& message = "") = 0;

    virtual Result kill() = 0;


  private:
    void registerEvents();
  };

  class IDistMMBoxStateImpl  : virtual public IMMBoxStateImpl { 
  public:
    IDistMMBoxStateImpl() { addInterface(new IDistMMBoxState(*this)); registerEvents(); };

    virtual ~IDistMMBoxStateImpl() {};

  protected:

    virtual INode* createNode(const NodeDescription& ndescr) = 0;

    virtual INode* getAudioNode() = 0;

    virtual INode* getOsdNode() = 0;

    virtual INode* getDisplayNode() = 0;

    virtual void setNeedVideo(const bool& b) = 0;

    virtual void setNeedAudio(const bool& b) = 0;


  private:
    void registerEvents();
  };

  class IDistMP3StateImpl  : virtual public IDistMMBoxStateImpl { 
  public:
    IDistMP3StateImpl() { addInterface(new IDistMP3State(*this)); registerEvents(); };

    virtual ~IDistMP3StateImpl() {};

  protected:

    virtual INode* getMPEGAudioDecodeNode() = 0;

    virtual IInputJack* getMPEGAudioInputJack() = 0;

    virtual INode* getPlaybackNode() = 0;

    virtual void addTextLine(const string& line) = 0;

    virtual void selectTextLine(const unsigned int& num) = 0;


  private:
    void registerEvents();
  };

}

#endif
