
#ifndef NMM_ILADSPA_HPP
#define NMM_ILADSPA_HPP

#include "nmm/NMMTypes.hpp"
#include "nmm/base/Interface.hpp"
#include "nmm/base/Event.hpp"
#include <memory>


namespace NMM
{

  class ILADSPA : virtual public Interface { 
  public:
    ILADSPA() {  setTypeInfo(TypeInfo(ILADSPA::ILADSPA_type, "NMM")); };

    ILADSPA(NMMObject& parent) : Interface(parent) {  setTypeInfo(TypeInfo(ILADSPA::ILADSPA_type, "NMM")); setEventReceiver(parent); };

    virtual ~ILADSPA() {};

    static const char* ILADSPA_type;

    virtual  string getInterfaceType() const { return ILADSPA_type; }

    Interface* copy() const { return new ILADSPA(*this); }
    static const char* addLADSPAPlugin_event;
    static const char* removeLADSPAPluginByName_event;
    static const char* removeLADSPAPluginByIndex_event;
    static const char* getLADSPAPluginCount_event;
    static const char* setConfigFilename_event;
    static const char* setConfigProfileByName_event;
    static const char* setConfigProfileByIndex_event;
    static const char* resetToBaseValuesByName_event;
    static const char* resetToBaseValuesByIndex_event;
    static const char* setControlInValueByName_event;
    static const char* setControlInValueByIndex_event;
    static const char* getControlInValueByName_event;
    static const char* getControlInValueByIndex_event;
    static const char* getControlInNameByName_event;
    static const char* getControlInNameByIndex_event;
    static const char* setControlInBaseByName_event;
    static const char* setControlInBaseByIndex_event;
    static const char* getControlInBaseByName_event;
    static const char* getControlInBaseByIndex_event;
    static const char* getLADSPAPluginIndex_event;
    static const char* getLADSPAPluginName_event;
    static const char* getControlInCountByName_event;
    static const char* getControlInCountByIndex_event;
    static const char* hasControlInUpperBoundByName_event;
    static const char* hasControlInUpperBoundByIndex_event;
    static const char* hasControlInLowerBoundByName_event;
    static const char* hasControlInLowerBoundByIndex_event;
    static const char* getControlInBoundsByName_event;
    static const char* getControlInBoundsByIndex_event;
    static const char* getLADSPAPluginRealNameByName_event;
    static const char* getLADSPAPluginRealNameByIndex_event;
    static const char* getLADSPAPluginLabelByName_event;
    static const char* getLADSPAPluginLabelByIndex_event;
    static const char* getLADSPAPluginLibraryNameByName_event;
    static const char* getLADSPAPluginLibraryNameByIndex_event;

    virtual bool addLADSPAPlugin(const string& value, string& name, unsigned int& execpos)
      throw(const Exception&);

    virtual void removeLADSPAPluginByName(const string& name)
      throw(const Exception&);

    virtual void removeLADSPAPluginByIndex(const unsigned int& index)
      throw(const Exception&);

    virtual unsigned int getLADSPAPluginCount()
      throw(const Exception&);

    virtual void setConfigFilename(const string& value)
      throw(const Exception&);

    virtual void setConfigProfileByName(const string& name, const string& profile)
      throw(const Exception&);

    virtual void setConfigProfileByIndex(const unsigned int& index, const string& profile)
      throw(const Exception&);

    virtual void resetToBaseValuesByName(const string& name)
      throw(const Exception&);

    virtual void resetToBaseValuesByIndex(const unsigned int& index)
      throw(const Exception&);

    virtual void setControlInValueByName(const string& name, const unsigned int& port, const float& value)
      throw(const Exception&);

    virtual void setControlInValueByIndex(const unsigned int& index, const unsigned int& port, const float& value)
      throw(const Exception&);

    virtual float getControlInValueByName(const string& name, const unsigned int& port)
      throw(const Exception&);

    virtual float getControlInValueByIndex(const unsigned int& index, const unsigned int& port)
      throw(const Exception&);

    virtual string getControlInNameByName(const string& name, const unsigned int& port)
      throw(const Exception&);

    virtual string getControlInNameByIndex(const unsigned int& index, const unsigned int& port)
      throw(const Exception&);

    virtual void setControlInBaseByName(const string& name, const unsigned int& port, const float& value)
      throw(const Exception&);

    virtual void setControlInBaseByIndex(const unsigned int& index, const unsigned int& port, const float& value)
      throw(const Exception&);

    virtual float getControlInBaseByName(const string& name, const unsigned int& port)
      throw(const Exception&);

    virtual float getControlInBaseByIndex(const unsigned int& index, const unsigned int& port)
      throw(const Exception&);

    virtual unsigned int getLADSPAPluginIndex(const string& name)
      throw(const Exception&);

    virtual string getLADSPAPluginName(const unsigned int& index)
      throw(const Exception&);

    virtual unsigned int getControlInCountByName(const string& name)
      throw(const Exception&);

    virtual unsigned int getControlInCountByIndex(const unsigned int& index)
      throw(const Exception&);

    virtual bool hasControlInUpperBoundByName(const string& name, const unsigned int& port)
      throw(const Exception&);

    virtual bool hasControlInUpperBoundByIndex(const unsigned int& index, const unsigned int& port)
      throw(const Exception&);

    virtual bool hasControlInLowerBoundByName(const string& name, const unsigned int& port)
      throw(const Exception&);

    virtual bool hasControlInLowerBoundByIndex(const unsigned int& index, const unsigned int& port)
      throw(const Exception&);

    virtual void getControlInBoundsByName(const string& name, const unsigned int& port, float& lower, float& upper)
      throw(const Exception&);

    virtual void getControlInBoundsByIndex(const unsigned int& index, const unsigned int& port, float& lower, float& upper)
      throw(const Exception&);

    virtual string getLADSPAPluginRealNameByName(const string& name)
      throw(const Exception&);

    virtual string getLADSPAPluginRealNameByIndex(const unsigned int& index)
      throw(const Exception&);

    virtual string getLADSPAPluginLabelByName(const string& name)
      throw(const Exception&);

    virtual string getLADSPAPluginLabelByIndex(const unsigned int& index)
      throw(const Exception&);

    virtual string getLADSPAPluginLibraryNameByName(const string& name)
      throw(const Exception&);

    virtual string getLADSPAPluginLibraryNameByIndex(const unsigned int& index)
      throw(const Exception&);


  };

  typedef auto_ptr<ILADSPA> ILADSPA_var;




}

#endif
