
#include "IStreamJump.hpp"
#include "nmm/base/InterfaceFactory.hpp"


namespace NMM
{
  
  
  Result IStreamJump::fastForward(const int& value) /* in-stream */ 
      throw(const Exception&)
  {
    TOutValue<Result > return_value;
    TInValue<int > value_value(value, false);
    Event event("IStreamJump::fastForward", &value_value);
    event.setReturnValue(&return_value);
    if(sendEvent(event) == FAILURE) {
      if (event.getException()) {
        throw Exception(*event.getException());
      }
    }
    return return_value.getRValue();
  }
  
  Result IStreamJump::streamJump(const int& value) /* in-stream */ 
      throw(const Exception&)
  {
    TOutValue<Result > return_value;
    TInValue<int > value_value(value, false);
    Event event("IStreamJump::streamJump", &value_value);
    event.setReturnValue(&return_value);
    if(sendEvent(event) == FAILURE) {
      if (event.getException()) {
        throw Exception(*event.getException());
      }
    }
    return return_value.getRValue();
  }
  
  Result IStreamJump::sendIFramesOnly(const int& n) /* in-stream */ 
      throw(const Exception&)
  {
    TOutValue<Result > return_value;
    TInValue<int > n_value(n, false);
    Event event("IStreamJump::sendIFramesOnly", &n_value);
    event.setReturnValue(&return_value);
    if(sendEvent(event) == FAILURE) {
      if (event.getException()) {
        throw Exception(*event.getException());
      }
    }
    return return_value.getRValue();
  }

  const char* IStreamJump::IStreamJump_type = "IStreamJump";
  TIFactoryObject<IStreamJump > __IStreamJump_prototype("IStreamJump");


  const char* IStreamJump::fastForward_event = "IStreamJump::fastForward";
  Event* IStreamJump::create_fastForward(const int& value)
  {
    TInValue<int >* value_value = new TInValue<int >();
    value_value->setValue(value);

    return new Event("IStreamJump::fastForward", value_value);
  }

  const char* IStreamJump::streamJump_event = "IStreamJump::streamJump";
  Event* IStreamJump::create_streamJump(const int& value)
  {
    TInValue<int >* value_value = new TInValue<int >();
    value_value->setValue(value);

    return new Event("IStreamJump::streamJump", value_value);
  }

  const char* IStreamJump::sendIFramesOnly_event = "IStreamJump::sendIFramesOnly";
  Event* IStreamJump::create_sendIFramesOnly(const int& n)
  {
    TInValue<int >* n_value = new TInValue<int >();
    n_value->setValue(n);

    return new Event("IStreamJump::sendIFramesOnly", n_value);
  }



}


