
#include "IInputJack.hpp"
#include "nmm/base/InterfaceFactory.hpp"

#include "IOutputJack.hpp"
#include "nmm/base/CommunicationChannel.hpp"


namespace NMM
{
  
  
  CommunicationChannel* IInputJack::connectTo(const IOutputJack& jack) 
      throw(const Exception&, const BindingException&)
  {
    TOutValue<CommunicationChannel* > return_value;
    TInValue<IOutputJack > jack_value(jack, false);
    Event event("IInputJack::connectTo", &jack_value);
    event.setReturnValue(&return_value);
    if(sendEvent(event) == FAILURE) {
      if (event.getException()) {
        if(BindingException* ex = dynamic_cast<BindingException*>(event.getException())) {
          throw BindingException(*ex);
        }
        throw Exception(*event.getException());
      }
    }
    return return_value.getRValue();
  }
  
  IOutputJack* IInputJack::getDataPartner()
      throw(const Exception&)
  {
    TOutValue<IOutputJack* > return_value;
    Event event("IInputJack::getDataPartner");
    event.setReturnValue(&return_value);
    if(sendEvent(event) == FAILURE) {
      if (event.getException()) {
        throw Exception(*event.getException());
      }
    }
    return return_value.getRValue();
  }
  
  ITransportStrategy* IInputJack::getTransportStrategy()
      throw(const Exception&)
  {
    TOutValue<ITransportStrategy* > return_value;
    Event event("IInputJack::getTransportStrategy");
    event.setReturnValue(&return_value);
    if(sendEvent(event) == FAILURE) {
      if (event.getException()) {
        throw Exception(*event.getException());
      }
    }
    return return_value.getRValue();
  }

  const char* IInputJack::IInputJack_type = "IInputJack";
  TIFactoryObject<IInputJack > __IInputJack_prototype("IInputJack");


  const char* IInputJack::connectTo_event = "IInputJack::connectTo";
  const char* IInputJack::getDataPartner_event = "IInputJack::getDataPartner";
  const char* IInputJack::getTransportStrategy_event = "IInputJack::getTransportStrategy";


}


