
#include "IJack.hpp"
#include "nmm/base/InterfaceFactory.hpp"

#include "INode.hpp"
#include "nmm/base/TransportStrategy.hpp"

namespace NMM
{
  
  
  INode* IJack::getIParent()
      throw(const Exception&)
  {
    TOutValue<INode* > return_value;
    Event event("IJack::getIParent");
    event.setReturnValue(&return_value);
    if(sendEvent(event) == FAILURE) {
      if (event.getException()) {
        throw Exception(*event.getException());
      }
    }
    return return_value.getRValue();
  }
  
  string IJack::getJackTag() const
      throw(const Exception&)
  {
    TOutValue<string > return_value;
    Event event("IJack::getJackTag");
    event.setReturnValue(&return_value);
    if(sendEvent(event) == FAILURE) {
      if (event.getException()) {
        throw Exception(*event.getException());
      }
    }
    return return_value.getRValue();
  }
  
  bool IJack::isConnected() const
      throw(const Exception&)
  {
    TOutValue<bool > return_value;
    Event event("IJack::isConnected");
    event.setReturnValue(&return_value);
    if(sendEvent(event) == FAILURE) {
      if (event.getException()) {
        throw Exception(*event.getException());
      }
    }
    return return_value.getRValue();
  }
  
  void IJack::setTransportStrategy(const TSAddress* address) 
      throw(const Exception&, const BindingException&)
  {
    TInValue<TSAddress* > address_value (const_cast<TSAddress* >(address), false);
    Event event("IJack::setTransportStrategy", &address_value);
    if(sendEvent(event) == FAILURE) {
      if (event.getException()) {
        if(BindingException* ex = dynamic_cast<BindingException*>(event.getException())) {
          throw BindingException(*ex);
        }
        throw Exception(*event.getException());
      }
    }
  }
  
  void IJack::disconnect()
      throw(const Exception&)
  {
    Event event("IJack::disconnect");
    if(sendEvent(event) == FAILURE) {
      if (event.getException()) {
        throw Exception(*event.getException());
      }
    }
  }
  
  void IJack::connectToJack() /* in-stream */ 
      throw(const Exception&)
  {
    Event event("IJack::connectToJack");
    if(sendEvent(event) == FAILURE) {
      if (event.getException()) {
        throw Exception(*event.getException());
      }
    }
  }

  const char* IJack::IJack_type = "IJack";
  TIFactoryObject<IJack > __IJack_prototype("IJack");


  const char* IJack::getIParent_event = "IJack::getIParent";
  const char* IJack::getJackTag_event = "IJack::getJackTag";
  const char* IJack::isConnected_event = "IJack::isConnected";
  const char* IJack::setTransportStrategy_event = "IJack::setTransportStrategy";
  const char* IJack::disconnect_event = "IJack::disconnect";
  const char* IJack::connectToJack_event = "IJack::connectToJack";
  Event* IJack::create_connectToJack()
  {

    return new Event("IJack::connectToJack");
  }



}


