
#ifndef NMM_IReferenceCount_HPP
#define NMM_IReferenceCount_HPP

#include "nmm/NMMTypes.hpp"
#include "nmm/base/Interface.hpp"
#include "nmm/base/Event.hpp"
#include <memory>


namespace NMM
{

  class IReferenceCount : virtual public Interface { 
  public:
    IReferenceCount() {  setTypeInfo(TypeInfo(IReferenceCount::IReferenceCount_type, "NMM")); };

    IReferenceCount(NMMObject& parent) : Interface(parent) {  setTypeInfo(TypeInfo(IReferenceCount::IReferenceCount_type, "NMM")); setEventReceiver(parent); };

    virtual ~IReferenceCount() {};

    static const char* IReferenceCount_type;

    virtual  string getInterfaceType() const { return IReferenceCount_type; }

    Interface* copy() const { return new IReferenceCount(*this); }
    static const char* increaseCount_event;
    static const char* decreaseCount_event;
    static const char* getCount_event;
    /**
     * Increases the ref count by i.
     */

    virtual void increaseCount(const unsigned int& i = 1)
      throw(const Exception&);
    /**
     * Decreases the ref count by i.
     */

    virtual void decreaseCount(const unsigned int& i = 1)
      throw(const Exception&);
    /**
     * Returns the current ref count.
     */

    virtual unsigned int getCount()
      throw(const Exception&);


  };

  typedef auto_ptr<IReferenceCount> IReferenceCount_var;




}

#endif
