
#ifndef NMM_IPluginManagerImpl_HPP
#define NMM_IPluginManagerImpl_HPP

#include "nmm/NMMTypes.hpp"
#include "nmm/base/NMMObject.hpp"
#include "IPluginManager.hpp"


namespace NMM
{

  class IPluginManagerImpl  : public virtual NMMObject { 
  public:
    IPluginManagerImpl() { addInterface(new IPluginManager(*this)); registerEvents(); };

    virtual ~IPluginManagerImpl() {};

  protected:
/** This method adds a plugin path.
  */

    virtual void addPluginPath(const string& path) = 0;
/** this wrapper method calls the three methods loadPlugin, activateRegistryPart, activateFactoryPart
  */

    virtual void loadLibrary(const string& path) = 0;
/** loads a specific plugin
  */

    virtual void loadPlugin(const string& plugin_name) 
      throw(const Exception&) = 0;
/** This method loads all plugins in a given directory. These plugins must be 'external' plugins which does 
    not provide NMM nodes. Thus, this method does not search and register NMM nodes, even if they exist in 
    a plugin */    

    virtual void loadExternalPlugins(const string& directory) 
      throw(const Exception&) = 0;
/** This method removes all plugins located in a given directory. These plugins must be 'external' plugins 
    which does not provide NMM nodes. Thus, this method does not unregister NMM nodes, even if they 
    exist */

    virtual void removeExternalPlugins(const string& directory) 
      throw(const Exception&) = 0;
/** activates the registry part of a plugin
  */

    virtual void activateRegistryPart(const string& plugin) 
      throw(const Exception&) = 0;
/** activates the factory part of a plugin
  */

    virtual void activateFactoryPart(const string& plugin) 
      throw(const Exception&) = 0;
/** deactivates the registry partof a a plugin
  */

    virtual void deactivateRegistryPart(const string& plugin) 
      throw(const Exception&) = 0;
/** deactivates the factoy part of a plugin
  */

    virtual void deactivateFactoryPart(const string& plugin) 
      throw(const Exception&) = 0;
/** removes an already loaded plugin with given name plugin_name
  */

    virtual void removeLibrary(const string& plugin_name) 
      throw(const Exception&) = 0;
/** returns the ObjectID's of a specific plugin
  */

    virtual list<ObjectID >  getPluginID(const string& plugin, const unsigned int& index) = 0;
/** returns the plugins available in the path list
  */

    virtual list<string >  getPlugins() const = 0;
/** returns the plugins available in the path list
  */

    virtual list<string >  getUsedPlugins() const = 0;
/** returns the plugins available in the path list
  */

    virtual list<string >  getPluginPath() const = 0;


  private:
    void registerEvents();
  };

}

#endif
