
#ifndef NMM_IServerRegistry_HPP
#define NMM_IServerRegistry_HPP

#include "nmm/NMMTypes.hpp"
#include "nmm/base/Interface.hpp"
#include "nmm/base/Event.hpp"
#include <memory>

#include "IRegistry.hpp"
#include "ISession.hpp"
#include "../resourcemgr/IResourceManager.hpp"

namespace NMM
{
/** \class IServerRegistry
  * Base class for all Registry Interfaces.
  */

  class IServerRegistry : virtual public IRegistry { 
  public:
    IServerRegistry() {  setTypeInfo(TypeInfo(IServerRegistry::IServerRegistry_type, "NMM")); };

    IServerRegistry(NMMObject& parent) : IRegistry(parent) {  setTypeInfo(TypeInfo(IServerRegistry::IServerRegistry_type, "NMM")); setEventReceiver(parent); };

    virtual ~IServerRegistry() {};

    static const char* IServerRegistry_type;

    virtual  string getInterfaceType() const { return IServerRegistry_type; }

    Interface* copy() const { return new IServerRegistry(*this); }
    static const char* getTransactionTimeout_event;
    static const char* setTransactionTimeout_event;
    static const char* getNodeLeasingTime_event;
    static const char* setNodeLeasingTime_event;
    static const char* refreshNodeLeasing_event;
    static const char* refreshTransaction_event;
    static const char* startResourceManager_event;
    static const char* getResourceManager_event;
    static const char* releaseResourceManager_event;
    static const char* startIPerfServer_event;
    static const char* stopIPerfServer_event;
    static const char* requestNMMObject_event;
    static const char* releaseNMMObject_event;
    static const char* requestSharedGraph_event;
    static const char* releaseSession_event;
    static const char* releaseSharedNode_event;
    static const char* initSession_event;
    static const char* requestGraph_event;
/** This method returns the timout for access control
  */

    virtual unsigned int getTransactionTimeout()
      throw(const Exception&);
/** This method sets the timout for access control.
  * \param time_out The new timeout for the access control
  */

    virtual void setTransactionTimeout(const unsigned int& time_out)
      throw(const Exception&);
/** This method returns the leasing time_out for nodes.
  */

    virtual unsigned int getNodeLeasingTime()
      throw(const Exception&);
/** This method sets the leasing time_out for nodes.
  * \param time_out The new timeout for nodes.
  */

    virtual void setNodeLeasingTime(const unsigned int& time_out)
      throw(const Exception&);
/** This method refreshes the reservation of a leased nodes.
  */

    virtual unsigned int refreshNodeLeasing(const RegistryID& reg_id, const ObjectID& node_id, const UserInfo& user_info)
      throw(const Exception&);
/** This method refreshes the enabled access control.
  */

    virtual unsigned int refreshTransaction(const UserInfo& user_info)
      throw(const Exception&);
/** This method starts the ResourceManager
  */

    virtual void startResourceManager()
      throw(const Exception&);
/** This method returns the interface to access the ResourceManager
  * which is started if necessary
  */

    virtual IResourceManager* getResourceManager()
      throw(const Exception&);
/** Release the ResourceManager
  */

    virtual void releaseResourceManager()
      throw(const Exception&);

    virtual void startIPerfServer()
      throw(const Exception&);

    virtual void stopIPerfServer()
      throw(const Exception&);

    virtual Interface* requestNMMObject(const ObjectDescription& od)
      throw(const Exception&);

    virtual void releaseNMMObject(const Interface& i_face)
      throw(const Exception&);

    virtual ISession* requestSharedGraph(GraphDescription& request, const UserInfo& u_info)
      throw(const Exception&, const RegistryException&);

    virtual void releaseSession(const ISession& session, const UserInfo& u_info)
      throw(const Exception&);

    virtual void releaseSharedNode(const INode& node)
      throw(const Exception&);
  /* initialize a new session */

    virtual ISession* initSession(GraphDescription& gd, const UserInfo& u_info, const ISession* given_session)
      throw(const Exception&);

    virtual void requestGraph(GraphDescription& gd, const UserInfo& user_info)
      throw(const Exception&);


  };

  typedef auto_ptr<IServerRegistry> IServerRegistry_var;




}

#endif
