
#ifndef NMM_IController_HPP
#define NMM_IController_HPP

#include "nmm/NMMTypes.hpp"
#include "nmm/base/Interface.hpp"
#include "nmm/base/Event.hpp"
#include <memory>


namespace NMM
{

  class IPing : virtual public Interface { 
  public:
    IPing() {  setTypeInfo(TypeInfo(IPing::IPing_type, "NMM")); };

    IPing(NMMObject& parent) : Interface(parent) {  setTypeInfo(TypeInfo(IPing::IPing_type, "NMM")); setEventReceiver(parent); };

    virtual ~IPing() {};

    static const char* IPing_type;

    virtual  string getInterfaceType() const { return IPing_type; }

    Interface* copy() const { return new IPing(*this); }
    static const char* ping_event;
    /**
     * Can be used to estimate the delay between method call and execution
     * @return the system time when the method call was executed
     */

    virtual Time ping()
      throw(const Exception&);


  };

  typedef auto_ptr<IPing> IPing_var;



  class IController : virtual public IPing { 
  public:    /**
     * Controller mode.
     */

  enum Mode { 
    ONLINE = 0 ,   /**<
			    * In this mode, the controller looks at the
			    * timestamps of the buffers, and the
			    * isBufferValid - method can evaluate to false.
			    * Then the node will discard this buffer.
			    */

    OFFLINE = 1   /**<
			    * This mode is used for offline tasks like
			    * cd grabbing. The controller declares all buffers
			    * as valid, so no buffer will be discarded by the
			    * nodes, and no buffer will be lost.
			    */
 };

    IController() {  setTypeInfo(TypeInfo(IController::IController_type, "NMM")); };

    IController(NMMObject& parent) : IPing(parent) {  setTypeInfo(TypeInfo(IController::IController_type, "NMM")); setEventReceiver(parent); };

    virtual ~IController() {};

    static const char* IController_type;

    virtual  string getInterfaceType() const { return IController_type; }

    Interface* copy() const { return new IController(*this); }
    static const char* setMode_event;
    static const char* permitDiscardBuffers_event;
    static const char* pause_event;
    static const char* wakeup_event;
    static const char* reset_event;
    static const char* setBuffersInvalid_event;
    static const char* setBuffersInvalidTimed_event;

    virtual Result setMode(const IController::Mode& value)
      throw(const Exception&);
    /**
     * Permit discarding buffers.
     * Set the flag that allows or disallows discarding buffers.
     * Set by default. In ONLINE mode, the controller may discard buffers if this flag is set.
     */

    virtual Result permitDiscardBuffers(const bool& value)
      throw(const Exception&);
    /**
     * The controller is told to pause the node via this method
     */

    virtual Result pause()
      throw(const Exception&);
    /**
     * The controller is told to wake up the node via this method
     */

    virtual Result wakeup()
      throw(const Exception&);
    /**
     * The controller is told to reset the node via this method
     */

    virtual Result reset()
      throw(const Exception&);
    /**
     * Sets whether buffers should always be regarded as invalid (and therefore not presented)
     * or if the usual checks should be performed to check whether a buffer is valid or not.
     * This allows to enable/disable a node.
     * @param status true for marking all buffers as invalid
     */

    virtual void setBuffersInvalid(const bool& status)
      throw(const Exception&);
    /**
     * Sets whether buffers should always be regarded as invalid (and therefore not presented)
     * or if the usual checks should be performed to check whether a buffer is valid or not.
     * This allows to enable/disable a node starting from a particular time.
     * @param status true for marking all buffers as invalid
     * @param status_time timestamp from which on the new status should be active
     */

    virtual void setBuffersInvalidTimed(const bool& status, const Timestamp& status_time)
      throw(const Exception&);


  };

  typedef auto_ptr<IController> IController_var;

  OValueStream& operator<<(OValueStream& ov, const IController::Mode& t);
  IValueStream& operator>>(IValueStream& iv, IController::Mode& t);



}

#endif
