
#include "ISinkSynchronizer.hpp"
#include "nmm/base/InterfaceFactory.hpp"


namespace NMM
{
  
  
  void ISinkSynchronizer::setAverageLatency(const Interval& value, const int& sink)
      throw(const Exception&)
  {
    TInValue<Interval > value_value(value, false);
    TInValue<int > sink_value(sink, false);
    Event event("ISinkSynchronizer::setAverageLatency", &value_value, &sink_value);
    if(sendEvent(event) == FAILURE) {
      if (event.getException()) {
        throw Exception(*event.getException());
      }
    }
  }
  
  Result ISinkSynchronizer::setFirstBufferInTimeAverageLatency(const Interval& value, const int& sink, const Timestamp& first_ts)
      throw(const Exception&)
  {
    TOutValue<Result > return_value;
    TInValue<Interval > value_value(value, false);
    TInValue<int > sink_value(sink, false);
    TInValue<Timestamp > first_ts_value(first_ts, false);
    Event event("ISinkSynchronizer::setFirstBufferInTimeAverageLatency", &value_value, &sink_value, &first_ts_value);
    event.setReturnValue(&return_value);
    if(sendEvent(event) == FAILURE) {
      if (event.getException()) {
        throw Exception(*event.getException());
      }
    }
    return return_value.getRValue();
  }
  
  Result ISinkSynchronizer::removeSinkController(const ISinkController* controller)
      throw(const Exception&)
  {
    TOutValue<Result > return_value;
    TInValue<ISinkController* > controller_value (const_cast<ISinkController* >(controller), false);
    Event event("ISinkSynchronizer::removeSinkController", &controller_value);
    event.setReturnValue(&return_value);
    if(sendEvent(event) == FAILURE) {
      if (event.getException()) {
        throw Exception(*event.getException());
      }
    }
    return return_value.getRValue();
  }

  const char* ISinkSynchronizer::ISinkSynchronizer_type = "ISinkSynchronizer";
  TIFactoryObject<ISinkSynchronizer > __ISinkSynchronizer_prototype("ISinkSynchronizer");


  const char* ISinkSynchronizer::setAverageLatency_event = "ISinkSynchronizer::setAverageLatency";
  const char* ISinkSynchronizer::setFirstBufferInTimeAverageLatency_event = "ISinkSynchronizer::setFirstBufferInTimeAverageLatency";
  const char* ISinkSynchronizer::removeSinkController_event = "ISinkSynchronizer::removeSinkController";


}


