
#ifndef NMM_ISinkSynchronizerImpl_HPP
#define NMM_ISinkSynchronizerImpl_HPP

#include "nmm/NMMTypes.hpp"
#include "nmm/base/NMMObject.hpp"
#include "ISinkSynchronizer.hpp"

#include "ISynchronizerImpl.hpp"
#include "ISinkControllerImpl.hpp"

namespace NMM
{

  class ISinkSynchronizerImpl  : virtual public ISynchronizerImpl { 
  public:
    ISinkSynchronizerImpl() { addInterface(new ISinkSynchronizer(*this)); registerEvents(); };

    virtual ~ISinkSynchronizerImpl() {};

  protected:
    /**
     * Set average latency.
     * This method is used by the sink controllers and should not be called 
     * directly.
     * With this method, the sink controllers tell the
     * synchronizer the average latency of their incoming buffers.
     * @param average_latency The average_latency of incoming buffers computed
     *                        by the Controller.
     * @param sink sink ID
     */

    virtual void setAverageLatency(const Interval& value, const int& sink) = 0;
    /**
     * Set average latency for first buffer in time after syncReset.
     * This method is used by the sink controllers and should not be called 
     * directly.
     * With this method, the sink controllers tell the
     * synchronizer the average latency of their first buffer in time after syncReset. 
     * @param average_latency The average_latency of incoming buffers computed
     *                        by the Controller.
     * @param sink sink ID
     * @param first_ts the timestamp of the first buffer in time
     * @return SUCCESS if value is good enough compared to theo_latency; FAILURE otherwise
     */

    virtual Result setFirstBufferInTimeAverageLatency(const Interval& value, const int& sink, const Timestamp& first_ts) = 0;
    /**
     * Remove a sink.
     * Deletes the sink with the specified interface.
     * @param controller sink controller
     */

    virtual Result removeSinkController(const ISinkController* controller) = 0;


  private:
    void registerEvents();
  };

}

#endif
