
#include "ICDDA.hpp"
#include "nmm/base/InterfaceFactory.hpp"


namespace NMM
{
  
  
  Result ICDDA::setTrack(const int& track)
      throw(const Exception&)
  {
    TOutValue<Result > return_value;
    TInValue<int > track_value(track, false);
    Event event("ICDDA::setTrack", &track_value);
    event.setReturnValue(&return_value);
    if(sendEvent(event) == FAILURE) {
      if (event.getException()) {
        throw Exception(*event.getException());
      }
    }
    return return_value.getRValue();
  }
  
  int ICDDA::getTrack()
      throw(const Exception&)
  {
    TOutValue<int > return_value;
    Event event("ICDDA::getTrack");
    event.setReturnValue(&return_value);
    if(sendEvent(event) == FAILURE) {
      if (event.getException()) {
        throw Exception(*event.getException());
      }
    }
    return return_value.getRValue();
  }
  
  void ICDDA::setRepeat(const bool& repeat)
      throw(const Exception&)
  {
    TInValue<bool > repeat_value(repeat, false);
    Event event("ICDDA::setRepeat", &repeat_value);
    if(sendEvent(event) == FAILURE) {
      if (event.getException()) {
        throw Exception(*event.getException());
      }
    }
  }
  
  bool ICDDA::getRepeat()
      throw(const Exception&)
  {
    TOutValue<bool > return_value;
    Event event("ICDDA::getRepeat");
    event.setReturnValue(&return_value);
    if(sendEvent(event) == FAILURE) {
      if (event.getException()) {
        throw Exception(*event.getException());
      }
    }
    return return_value.getRValue();
  }
  
  int ICDDA::getTotalTracks()
      throw(const Exception&)
  {
    TOutValue<int > return_value;
    Event event("ICDDA::getTotalTracks");
    event.setReturnValue(&return_value);
    if(sendEvent(event) == FAILURE) {
      if (event.getException()) {
        throw Exception(*event.getException());
      }
    }
    return return_value.getRValue();
  }
  
  void ICDDA::numberTracks(const bool& number)
      throw(const Exception&)
  {
    TInValue<bool > number_value(number, false);
    Event event("ICDDA::numberTracks", &number_value);
    if(sendEvent(event) == FAILURE) {
      if (event.getException()) {
        throw Exception(*event.getException());
      }
    }
  }
  
  bool ICDDA::isFinished()
      throw(const Exception&)
  {
    TOutValue<bool > return_value;
    Event event("ICDDA::isFinished");
    event.setReturnValue(&return_value);
    if(sendEvent(event) == FAILURE) {
      if (event.getException()) {
        throw Exception(*event.getException());
      }
    }
    return return_value.getRValue();
  }
  
  void ICDDA::setOneTrackOnly(const bool& enable)
      throw(const Exception&)
  {
    TInValue<bool > enable_value(enable, false);
    Event event("ICDDA::setOneTrackOnly", &enable_value);
    if(sendEvent(event) == FAILURE) {
      if (event.getException()) {
        throw Exception(*event.getException());
      }
    }
  }
  
  bool ICDDA::getOneTrackOnly()
      throw(const Exception&)
  {
    TOutValue<bool > return_value;
    Event event("ICDDA::getOneTrackOnly");
    event.setReturnValue(&return_value);
    if(sendEvent(event) == FAILURE) {
      if (event.getException()) {
        throw Exception(*event.getException());
      }
    }
    return return_value.getRValue();
  }

  const char* ICDDA::ICDDA_type = "ICDDA";
  TIFactoryObject<ICDDA > __ICDDA_prototype("ICDDA");


  const char* ICDDA::setTrack_event = "ICDDA::setTrack";
  const char* ICDDA::getTrack_event = "ICDDA::getTrack";
  const char* ICDDA::setRepeat_event = "ICDDA::setRepeat";
  const char* ICDDA::getRepeat_event = "ICDDA::getRepeat";
  const char* ICDDA::getTotalTracks_event = "ICDDA::getTotalTracks";
  const char* ICDDA::numberTracks_event = "ICDDA::numberTracks";
  const char* ICDDA::isFinished_event = "ICDDA::isFinished";
  const char* ICDDA::setOneTrackOnly_event = "ICDDA::setOneTrackOnly";
  const char* ICDDA::getOneTrackOnly_event = "ICDDA::getOneTrackOnly";


}


