
#ifndef NMM_IBufferDropImpl_HPP
#define NMM_IBufferDropImpl_HPP

#include "nmm/NMMTypes.hpp"
#include "nmm/base/NMMObject.hpp"
#include "IBufferDrop.hpp"

#include "nmm/interfaces/base/INodeImpl.hpp"

namespace NMM
{

  class IBufferDropImpl  : public virtual NMMObject { 
  public:
    IBufferDropImpl() { addInterface(new IBufferDrop(*this)); registerEvents(); };

    virtual ~IBufferDropImpl() {};

  protected:
    /**
     * Sets which buffers should be dropped (e.g. a '2' meaning every second buffer
     * should be dropped; a '3' meaning every third buffer)
     */

    virtual void setBufferToDrop(const unsigned int& buffer_num) = 0;
    /**
     * Use this method to specify a drop ratio, e.g. dropping of 5 out
     * of 25 frames.
     * \param numerator The numerator of the ratio.
     * \param denominator The denominator of the ratio.
     */

    virtual void setDropRatio(const unsigned int& numerator, const unsigned int& denominator) = 0;


  private:
    void registerEvents();
  };

  class ITimedBufferDropImpl  : public virtual NMMObject { 
  public:
    ITimedBufferDropImpl() { addInterface(new ITimedBufferDrop(*this)); registerEvents(); };

    virtual ~ITimedBufferDropImpl() {};

  protected:
    /**
     * Adds an in and out point for removing timestamped buffers froms stream
     */

    virtual void addInOutPoint(const unsigned int& in_secs, const unsigned int& in_millisecs, const unsigned int& out_secs, const unsigned int& out_millisecs) /* limited(INode::CONSTRUCTED) */ = 0;
    /**
     * If true, all inout-points have to be specified relative to first timestamp in stream
     */

    virtual void useRelativeInOutPoints(const bool& relative) /* limited(INode::CONSTRUCTED) */ = 0;
    /**
     * Gets the status of relative or absolute inout-points
     */

    virtual bool isRelativeInOutPoints() = 0;


  private:
    void registerEvents();
  };

}

#endif
