
#ifndef NMM_ITrackDuration_HPP
#define NMM_ITrackDuration_HPP

#include "nmm/NMMTypes.hpp"
#include "nmm/base/Interface.hpp"
#include "nmm/base/Event.hpp"
#include <memory>


namespace NMM
{
  /**
   *  @brief TrackDuration events
   *
   *  The trackDuration event is sent by read nodes, decoders or demuxers. It contains the duration of the
   *  audio/video track that is being processed. Applications that need to know the duration of the track
   *  can register a listener for this event.
   */

  class ITrackDuration : virtual public Interface { 
  public:
    ITrackDuration() {  setTypeInfo(TypeInfo(ITrackDuration::ITrackDuration_type, "NMM")); };

    ITrackDuration(NMMObject& parent) : Interface(parent) {  setTypeInfo(TypeInfo(ITrackDuration::ITrackDuration_type, "NMM")); setEventReceiver(parent); };

    virtual ~ITrackDuration() {};

    static const char* ITrackDuration_type;

    virtual  string getInterfaceType() const { return ITrackDuration_type; }

    Interface* copy() const { return new ITrackDuration(*this); }
    static const char* trackDuration_event;
    static Event* create_trackDuration(const Interval& i);
  /* instream (protected) and non-instream (public) */        /**
     *  @brief Duration of the current track
     *  @param duration of the track (in seconds and nanoseconds)
     *
     *  Instream event containing the duration of the current track.
     */

    virtual Result trackDuration(const Interval& i) /* in-stream */ 
      throw(const Exception&);


  };

  typedef auto_ptr<ITrackDuration> ITrackDuration_var;




}

#endif
