
#ifndef NMM_ITrackSize_HPP
#define NMM_ITrackSize_HPP

#include "nmm/NMMTypes.hpp"
#include "nmm/base/Interface.hpp"
#include "nmm/base/Event.hpp"
#include <memory>


namespace NMM
{
  /**
   *  @brief Used to receive trackSize events
   *
   *  This interface is used to receive the size (in bytes) of a track (video/audio).
   *  Decoders for example need the size to compute the duration of a track.
   */

  class ITrackSize : virtual public Interface { 
  public:
    ITrackSize() {  setTypeInfo(TypeInfo(ITrackSize::ITrackSize_type, "NMM")); };

    ITrackSize(NMMObject& parent) : Interface(parent) {  setTypeInfo(TypeInfo(ITrackSize::ITrackSize_type, "NMM")); setEventReceiver(parent); };

    virtual ~ITrackSize() {};

    static const char* ITrackSize_type;

    virtual  string getInterfaceType() const { return ITrackSize_type; }

    Interface* copy() const { return new ITrackSize(*this); }
    static const char* trackSize_event;
    static Event* create_trackSize(const u_int64_t& bytes);
  /* instream (protected) and non-instream (public) */        /**
     *  @brief Used to receive trackSize events
     *  @param the size of the track in bytes
     *
     *  This method is used to receive the size of a track.
     */

    virtual Result trackSize(const u_int64_t& bytes) /* in-stream */ 
      throw(const Exception&);


  };

  typedef auto_ptr<ITrackSize> ITrackSize_var;




}

#endif
