
#ifndef NMM_ITrackSizeImpl_HPP
#define NMM_ITrackSizeImpl_HPP

#include "nmm/NMMTypes.hpp"
#include "nmm/base/NMMObject.hpp"
#include "ITrackSize.hpp"


namespace NMM
{
  /**
   *  @brief Used to receive trackSize events
   *
   *  This interface is used to receive the size (in bytes) of a track (video/audio).
   *  Decoders for example need the size to compute the duration of a track.
   */

  class ITrackSizeImpl  : public virtual NMMObject { 
  public:
    ITrackSizeImpl() { addInterface(new ITrackSize(*this)); registerEvents(); };

    virtual ~ITrackSizeImpl() {};

  protected:
    /**
     *  @brief Used to receive trackSize events
     *  @param the size of the track in bytes
     *
     *  This method is used to receive the size of a track.
     */

    virtual Result trackSize(const u_int64_t& bytes) /* in-stream */  = 0;


  private:
    void registerEvents();
  };

}

#endif
