
#ifndef NMM_IProgressImpl_HPP
#define NMM_IProgressImpl_HPP

#include "nmm/NMMTypes.hpp"
#include "nmm/base/NMMObject.hpp"
#include "IProgress.hpp"


namespace NMM
{
  /**
   *  @brief Progress sender interface
   *
   *  A progress sender is a node which, if enabled, sends progress events downstream.
   *  An IProgressListener interface can be used to catch these events.
   */

  class IProgressImpl  : public virtual NMMObject { 
  public:
    IProgressImpl() { addInterface(new IProgress(*this)); registerEvents(); };

    virtual ~IProgressImpl() {};

  protected:
    /**
     *  @brief Enable or disable sending progress information
     *  @param enabled True to enable sending progress information
     *
     *  This method enables or disables sending of progress information. If enables, this node will send progress
     *  events (IProgressListener::setProgress) at regular intervals, defined by the node.
     */

    virtual void sendProgressInformation(const bool& value) = 0;
    /**
     *  @brief Sets the interval for sending setProgress events
     *  @param The number of buffers after which a setProgress event should be sent
     *
     *  This method changes the interval in which setProgress events are sent.
     */

    virtual void setProgressInterval(const int& buffers) = 0;


  private:
    void registerEvents();
  };

}

#endif
