
#ifndef NMM_IProgressListener_HPP
#define NMM_IProgressListener_HPP

#include "nmm/NMMTypes.hpp"
#include "nmm/base/Interface.hpp"
#include "nmm/base/Event.hpp"
#include <memory>


namespace NMM
{
  /**
   *  @brief Progress events
   *
   *  Instream events for sending progress information.
   *  Nodes that implement the IProgress interface send events from this interface.
   *  Applications that need progress information typically register listeners for these events.
   */

  class IProgressListener : virtual public Interface { 
  public:
    IProgressListener() {  setTypeInfo(TypeInfo(IProgressListener::IProgressListener_type, "NMM")); };

    IProgressListener(NMMObject& parent) : Interface(parent) {  setTypeInfo(TypeInfo(IProgressListener::IProgressListener_type, "NMM")); setEventReceiver(parent); };

    virtual ~IProgressListener() {};

    static const char* IProgressListener_type;

    virtual  string getInterfaceType() const { return IProgressListener_type; }

    Interface* copy() const { return new IProgressListener(*this); }
    static const char* setProgress_event;
    static Event* create_setProgress(const u_int64_t& numerator, const u_int64_t& denominator);
  /* instream (protected) and non-instream (public) */        /**
     *  @brief Progress information.
     *  @param numerator the current progress
     *  @param denominator the maximum progress
     *
     *  Instream event containing the current state of progress.
     */

    virtual void setProgress(const u_int64_t& numerator, const u_int64_t& denominator) /* in-stream */ 
      throw(const Exception&);


  };

  typedef auto_ptr<IProgressListener> IProgressListener_var;




}

#endif
