
#ifndef NMM_ITimeProgressImpl_HPP
#define NMM_ITimeProgressImpl_HPP

#include "nmm/NMMTypes.hpp"
#include "nmm/base/NMMObject.hpp"
#include "ITimeProgress.hpp"


namespace NMM
{
  /**
   *  @brief Progress sender interface
   *
   *  A node implementing ITimeProgress can send time progress information,
   *  e.g. to inform the application about the current playback time
   *  of an audio or video file.
   */

  class ITimeProgressImpl  : public virtual NMMObject { 
  public:
    ITimeProgressImpl() { addInterface(new ITimeProgress(*this)); registerEvents(); };

    virtual ~ITimeProgressImpl() {};

  protected:
    /**
     *  @brief Sets the time interval after which progress events are sent (default 1s).
     *
     *  @param The time interval after which a time progress event is sent.
     */

    virtual void setProgressUpdateInterval(const Interval& update_interval) = 0;
    /**
     *  @brief Enable or disable sending progress information
     *  @param True to enable sending progress information
     *
     *  This method enables or disables sending of progress information. If enables, this node will send progress
     *  events at regular intervals.
     */

    virtual void setProgressUpdateEnabled(const bool& enable_update) = 0;


  private:
    void registerEvents();
  };

}

#endif
