
#ifndef NMM_INxtvepgPlugin_HPP
#define NMM_INxtvepgPlugin_HPP

#include "nmm/NMMTypes.hpp"
#include "nmm/base/Interface.hpp"
#include "nmm/base/Event.hpp"
#include <memory>


namespace NMM
{

  class INxtvepgPlugin : virtual public Interface { 
  public:
    INxtvepgPlugin() {  setTypeInfo(TypeInfo(INxtvepgPlugin::INxtvepgPlugin_type, "NMM")); };

    INxtvepgPlugin(NMMObject& parent) : Interface(parent) {  setTypeInfo(TypeInfo(INxtvepgPlugin::INxtvepgPlugin_type, "NMM")); setEventReceiver(parent); };

    virtual ~INxtvepgPlugin() {};

    static const char* INxtvepgPlugin_type;

    virtual  string getInterfaceType() const { return INxtvepgPlugin_type; }

    Interface* copy() const { return new INxtvepgPlugin(*this); }
    static const char* setLocation_event;
    static const char* setPort_event;
    static const char* addProvider_event;
    static const char* getProvider_event;
    static const char* setConfigFile_event;
    static const char* getConfigFile_event;
    static const char* setDBDir_event;
    static const char* getDBDir_event;
    static const char* setLocale_event;
    static const char* getLocale_event;
    static const char* setPeriod_event;
    static const char* getPeriod_event;
    static const char* setUseTv_split_event;
    static const char* getUseTv_split_event;
    /**
     * sets the remote host where the nxtvepg-daemon if different from localhost
     *@param loc location of nxtvepg-daemon
     */

    virtual void setLocation(const string& loc)
      throw(const Exception&);
    /**
    * sets the remote port where the nxtvepg-daemon if different from localhost
    *@param int port
    */

    virtual void setPort(const int& p)
      throw(const Exception&);
    /**
     * adding a provider-string for XML-Dump
     */

    virtual void addProvider(const string& p)
      throw(const Exception&);
    /**
     * returns the providers for XMLDump, that are selected
     */

    virtual list<string >  getProvider()
      throw(const Exception&);
    /**
     * sets the path and name of the plugin-Configuration-File
     */

    virtual void setConfigFile(const string& file)
      throw(const Exception&);
    /**
     *returns the path and name of Configuraaiton-File used
     */

    virtual string getConfigFile()
      throw(const Exception&);
    /**
     * seting database-directory for nxtvepg-tool
     */

    virtual void setDBDir(const string& db)
      throw(const Exception&);
    /**
     * get database-directory
     */

    virtual string getDBDir()
      throw(const Exception&);
    /**
     * if the file contains data for differnet countris, specify, wich data should be evaluated
     *@param abreviation for locale, eg de for germany
     */

    virtual void setLocale(const string& l)
      throw(const Exception&);
    /**
     * get the locale-string set
     *@return localstring
     */

    virtual string getLocale()
      throw(const Exception&);
    /**
     * set the interval when a new grabb-command should be issued. The syntax of the string must be XY, where X=d|m|s Y=0-...
     * X=d means daily, the number specifies, after what amount of days the command should be re-launched
     *@param period
     */

    virtual void setPeriod(const string& p)
      throw(const Exception&);
    /**
     * get the period set
     *@return period
     */

    virtual string getPeriod()
      throw(const Exception&);
    /**
     * enbale/disable the use of tv_split, wich is contains in xmltv-tools-distribution
     */

    virtual void setUseTv_split(const bool& b)
      throw(const Exception&);

    virtual bool getUseTv_split()
      throw(const Exception&);


  };

  typedef auto_ptr<INxtvepgPlugin> INxtvepgPlugin_var;




}

#endif
