
#ifndef NMM_ITVGuidePeerObject_HPP
#define NMM_ITVGuidePeerObject_HPP

#include "nmm/NMMTypes.hpp"
#include "nmm/base/Interface.hpp"
#include "nmm/base/Event.hpp"
#include <memory>

#include "nmm/interfaces/misc/tvguide/ProgramDescription.hpp"
#include "nmm/interfaces/misc/tvguide/ProgramFilter.hpp"

namespace NMM
{
  /**
   *Interface to get notification from remote TVGuides of available information
   */

  class ITVGuidePeerObject : virtual public Interface { 
  public:
    ITVGuidePeerObject() {  setTypeInfo(TypeInfo(ITVGuidePeerObject::ITVGuidePeerObject_type, "NMM")); };

    ITVGuidePeerObject(NMMObject& parent) : Interface(parent) {  setTypeInfo(TypeInfo(ITVGuidePeerObject::ITVGuidePeerObject_type, "NMM")); setEventReceiver(parent); };

    virtual ~ITVGuidePeerObject() {};

    static const char* ITVGuidePeerObject_type;

    virtual  string getInterfaceType() const { return ITVGuidePeerObject_type; }

    Interface* copy() const { return new ITVGuidePeerObject(*this); }
    static const char* peerResult_event;
    static const char* peerChannelResult_event;
    /**
     *This method is calles from the remote TVGuide to notify the local one of available information. 
     *@param pdlist list of ProgramDescriptions the local TVGuide has rewusted
     *@param id if the search. This is for the application wich started the request to match the resault to a request
     *@param history This list stores the ITVGuidePeerObect-interfaces of TVGuide wich should be notified about the result
     */

    virtual Result peerResult(const ProgramDescriptionList& pdlist, const int& id, const list<Interface* > & history)
      throw(const Exception&);

    virtual Result peerChannelResult(const ChannelDescriptionList& cdlist, const list<Interface* > & history)
      throw(const Exception&);


  };

  typedef auto_ptr<ITVGuidePeerObject> ITVGuidePeerObject_var;


  /**
   *Interface to search remote TVGuides for information 
   */

  class ITVGuidePeerSearch : virtual public Interface { 
  public:
    ITVGuidePeerSearch() {  setTypeInfo(TypeInfo(ITVGuidePeerSearch::ITVGuidePeerSearch_type, "NMM")); };

    ITVGuidePeerSearch(NMMObject& parent) : Interface(parent) {  setTypeInfo(TypeInfo(ITVGuidePeerSearch::ITVGuidePeerSearch_type, "NMM")); setEventReceiver(parent); };

    virtual ~ITVGuidePeerSearch() {};

    static const char* ITVGuidePeerSearch_type;

    virtual  string getInterfaceType() const { return ITVGuidePeerSearch_type; }

    Interface* copy() const { return new ITVGuidePeerSearch(*this); }
    static const char* peerSearch_event;
    static const char* peerChannelSearch_event;
    /**
     *method to search a remote TVGuide for information. 
     *@param filter A ProgramFilter containing the request
     *@param history This list stores the ITVGuidePeerObect-interfaces of TVGuide wich should be notified about the result
     */

    virtual Result peerSearch(const ProgramFilter& filter, const list<Interface* > & history)
      throw(const Exception&);

    virtual Result peerChannelSearch(const list<Interface* > & history)
      throw(const Exception&);


  };

  typedef auto_ptr<ITVGuidePeerSearch> ITVGuidePeerSearch_var;




}

#endif
