
#ifndef NMM_ITVGuidePlugin_HPP
#define NMM_ITVGuidePlugin_HPP

#include "nmm/NMMTypes.hpp"
#include "nmm/base/Interface.hpp"
#include "nmm/base/Event.hpp"
#include <memory>


namespace NMM
{

  class ITVGuidePlugin : virtual public Interface { 
  public:
  enum State { 
    CONSTRUCTED = 1 , 
    INITIALIZED = 2 , 
    STARTED = 16  };

    ITVGuidePlugin() {  setTypeInfo(TypeInfo(ITVGuidePlugin::ITVGuidePlugin_type, "NMM")); };

    ITVGuidePlugin(NMMObject& parent) : Interface(parent) {  setTypeInfo(TypeInfo(ITVGuidePlugin::ITVGuidePlugin_type, "NMM")); setEventReceiver(parent); };

    virtual ~ITVGuidePlugin() {};

    static const char* ITVGuidePlugin_type;

    virtual  string getInterfaceType() const { return ITVGuidePlugin_type; }

    Interface* copy() const { return new ITVGuidePlugin(*this); }
    static const char* getName_event;
    static const char* start_event;
    static const char* stop_event;
    static const char* enable_event;
    static const char* disable_event;
    static const char* init_event;
    static const char* deinit_event;
    static const char* getEnabled_event;
    static const char* getState_event;
    /**
     * returns the name of the plugin
     */

    virtual string getName()
      throw(const Exception&);
    /**
     * Starts an update
     */

    virtual Result start()
      throw(const Exception&);
    /**
     * stops the update
     */

    virtual Result stop()
      throw(const Exception&);
    /**
     * enables the Plugin
     */

    virtual void enable()
      throw(const Exception&);
    /**
     * disables the plugin
     */

    virtual void disable()
      throw(const Exception&);
    /**
     * initializes the plugin
     */

    virtual Result init()
      throw(const Exception&);
    /**
     * deinitializes the plugin
     */

    virtual Result deinit()
      throw(const Exception&);
    /**
     * get the status of enabled-flag
     *@return status of enabled-flag
     */

    virtual bool getEnabled()
      throw(const Exception&);
    /**
     * get the current State of the Plugin
     *@return plugin-state
     */

    virtual ITVGuidePlugin::State getState()
      throw(const Exception&);


  };

  typedef auto_ptr<ITVGuidePlugin> ITVGuidePlugin_var;

  OValueStream& operator<<(OValueStream& ov, const ITVGuidePlugin::State& t);
  IValueStream& operator>>(IValueStream& iv, ITVGuidePlugin::State& t);



}

#endif
