
#ifndef NMM_IXMLTVPlugin_HPP
#define NMM_IXMLTVPlugin_HPP

#include "nmm/NMMTypes.hpp"
#include "nmm/base/Interface.hpp"
#include "nmm/base/Event.hpp"
#include <memory>


namespace NMM
{

  class IXMLTVPlugin : virtual public Interface { 
  public:
    IXMLTVPlugin() {  setTypeInfo(TypeInfo(IXMLTVPlugin::IXMLTVPlugin_type, "NMM")); };

    IXMLTVPlugin(NMMObject& parent) : Interface(parent) {  setTypeInfo(TypeInfo(IXMLTVPlugin::IXMLTVPlugin_type, "NMM")); setEventReceiver(parent); };

    virtual ~IXMLTVPlugin() {};

    static const char* IXMLTVPlugin_type;

    virtual  string getInterfaceType() const { return IXMLTVPlugin_type; }

    Interface* copy() const { return new IXMLTVPlugin(*this); }
    static const char* setFilename_event;
    static const char* getFilename_event;
    static const char* setConfigFile_event;
    static const char* getConfigFile_event;
    static const char* setLocale_event;
    static const char* getLocale_event;
    static const char* setPeriod_event;
    static const char* getPeriod_event;
    static const char* setFillHolesInDB_event;
    static const char* getFillHolesInDB_event;
    /**
     * Set the path and file where XMLTV stores its downloaded data
     *@param file filename of datafile
     */

    virtual void setFilename(const string& file)
      throw(const Exception&);
    /**
     * get the filename of the temp datafile
     *@return path + datafile
     */

    virtual string getFilename()
      throw(const Exception&);
    /**
     * Set the patch and name of the temporary config-file, where the plugin stores its current position in update
     *@param filepath and filename of configfile
     */

    virtual void setConfigFile(const string& file)
      throw(const Exception&);
    /**
     * get the path and name of the confgifile
     *@return path and name of the confgifile
     */

    virtual string getConfigFile()
      throw(const Exception&);
    /**
     * Select the XMLTV-Tool by setting the country-information
     *@param loc abreviation or complete tool-name of XMLTV-Tools
     */

    virtual Result setLocale(const string& loc)
      throw(const Exception&);
    /**
     * get the currently used Locale-information
     *@return locale
     */

    virtual string getLocale()
      throw(const Exception&);
    /**
     * set the period when a new update will start
     *@param period in form d|h|m <number>
     */

    virtual void setPeriod(const string& p)
      throw(const Exception&);
    /**
     * get the configured period
     *@return period
     */

    virtual string getPeriod()
      throw(const Exception&);
    /**
     * if the argument is set to true, the grabber will try to get data from the past.
     * if it is possible or not depends on the grabber used
     *@param bool if true, the plugins tries to grab data from the past
     */

    virtual void setFillHolesInDB(const bool& b)
      throw(const Exception&);
    /**
     * 
     *@return true try to grab data from past
     *@return false only grab data from 'now' on
     */

    virtual bool getFillHolesInDB()
      throw(const Exception&);


  };

  typedef auto_ptr<IXMLTVPlugin> IXMLTVPlugin_var;




}

#endif
