/* NMM - Network-Integrated Multimedia Middleware
 *
 * Copyright (C) 2002-2005
 *                    NMM work group,
 *                    Computer Graphics Lab,
 *                    Saarland University, Germany
 *                    http://www.networkmultimedia.org
 *
 * Maintainer:        chwellner <chwellner@graphics.cs.uni-sb.de>
 *                     
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version 2
 * of the License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307
 * USA
 */

#ifndef PROGRAMFILTER_HPP
#define PROGRAMFILTER_HPP PROGRAMFILTER_HPP

#include <iostream>
#include <list>
#include <string>

#include "nmm/base/serialize/ComplexValue.hpp"
#include "nmm/base/TSAddress.hpp"

namespace NMM{

  /** \class ProgramFilter
   * This class represents a Filter holding search-criterias for search programs in TVGuide
   */

  class ProgramFilter : public SerializedValue{

  private:
    NMM::TValue<string> starttime;
    NMM::TValue<string> endtime;
    NMM::TValue<string> phrase;

    NMM::TValue<list<string> > categories;
    NMM::TValue<list<string> > channels;

    NMM::TValue<bool> starttimeEdited; //Only needed for EPGSearchState in MMBox yet

    //Per-To_peer-variables
    NMM::TValue<unsigned int> id;
    NMM::TValue<list<string> > visitedPeerHosts;
    //add TTL
  public:
    ProgramFilter();
    ProgramFilter(const ProgramFilter&);
    ~ProgramFilter();

    /**
     *Set the start-time at wich point the search should begin
     *@param string starttime
     */
    void setStartTime(string st);
    /**
     *Get the start-time set
     *@return start-time
     */
    string getStartTime() const;

    /**
     *Set the end-time at wich point the search should stop
     *@param string endtime
     */
    void setEndTime(string et);
    /**
     *Get the end-time set
     *@return string end-time
     */
    string getEndTime() const;

    /**
     *Set a string wich the program-informations should contain
     *@param string-pattern
     */
    void setSearchPhrase(string p);
    /**
     *Get the SearchPhrase set
     *@return string-pattern
     */
    string getSearchPhrase() const;

    /**
     *Add a category in wich the resulting programs should fit in
     *@param category
     */
    void addCategory(string c);
    /**
     *Get the categories set
     *@return list<string> categories
     */
    list<string> getCategories() const;
    
    /**
     *Add a channel to search inly programms on this channel. This must be an available channel-ID
     *@param channel-ID
     */
    void addChannel(string c);
    /**
     *Get the channel-IDs set
     *@return list<string> channels
     */
    list<string> getChannels() const;

    void setStarttimeEdited(bool b){starttimeEdited = b;}
    bool getStarttimeEdited(){ return starttimeEdited.getRValue();}

    /**
     * adds a PeerHost that is vidited by this ProgramFilter. 
     * This funktion is used to create a history of already visited hosts when searching in the network for information
     */
    void addPeerHost(string name);
    /**
     * test if this ProgramFilter has already visited the given host
     */
    bool hasPeerHost(string name);
    /**
     * Clears the list of already visited hosts
     */
    void clearPeerHosts();

    void setID(unsigned int i){id = i;}
    unsigned int getID() const {return id.getRValue();}

    void localizeTimes();

    //Interface SerialzedValue
    ProgramFilter* copy();

    bool operator==(const ProgramFilter &pf) const;

  private:
    void registerValues();
    string transformTime(string s);

  };

  OValueStream& operator<<(OValueStream& os, const list<ProgramFilter>& pf);
  IValueStream& operator>>(IValueStream& is, list<ProgramFilter>& pf);

}

#endif
