
#ifndef NMM_ITVTimer_HPP
#define NMM_ITVTimer_HPP

#include "nmm/NMMTypes.hpp"
#include "nmm/base/Interface.hpp"
#include "nmm/base/Event.hpp"
#include <memory>

#include "nmm/misc/tvtimer/TVTimerEntry.hpp"
#include "nmm/misc/tvtimer/TVTimerExceptions.hpp"
#include "nmm/interfaces/video/tv/TVChannelList.hpp"
#include "nmm/interfaces/misc/tvguide/ProgramDescription.hpp"
#include "nmm/base/NMMApplication.hpp"
#include "nmm/base/registry/NodeDescription.hpp"
#include "nmm/interfaces/misc/tvguide/ITVGuide.hpp"
#include "nmm/interfaces/base/registry/IServerRegistry.hpp"

namespace NMM
{

  class ITVTimer : virtual public Interface { 
  public:
    ITVTimer() {  setTypeInfo(TypeInfo(ITVTimer::ITVTimer_type, "NMM")); };

    ITVTimer(NMMObject& parent) : Interface(parent) {  setTypeInfo(TypeInfo(ITVTimer::ITVTimer_type, "NMM")); setEventReceiver(parent); };

    virtual ~ITVTimer() {};

    static const char* ITVTimer_type;

    virtual  string getInterfaceType() const { return ITVTimer_type; }

    Interface* copy() const { return new ITVTimer(*this); }
    static const char* programTimer_event;
    static const char* changeTimer_event;
    static const char* deleteTimer_event;
    static const char* getTimers_event;
    static const char* getTimer_event;
    static const char* getSleepingTimers_event;
    static const char* getRecordingTimers_event;
    static const char* getRecordedTimers_event;
    static const char* getConflictTimers_event;
    static const char* getConflictsWithTimer_event;
    static const char* loadTimer_event;
    static const char* saveTimer_event;
    static const char* setTVGuide_event;
    static const char* getTVGuide_event;
    static const char* setDirectory_event;
    static const char* getDirectory_event;
    static const char* getChannelConfig_event;
    /**
       Program Timer using TvTimerEntry 
    */

    virtual Result programTimer(const TVTimerEntry& entry)
      throw(const Exception&, const TimerAlreadyProgrammedException&);
    /**
       ChangeTimer using TvTimerEntry 
    */

    virtual Result changeTimer(const TVTimerEntry& newEntry)
      throw(const Exception&, const ActionNotAllowedException&);
    /**
       Delete Timer with ID i 
    */

    virtual Result deleteTimer(const int& i, const string& user = "")
      throw(const Exception&, const ActionNotAllowedException&);

    virtual list<TVTimerEntry >  getTimers(const string& day = "", const string& channel = "")
      throw(const Exception&);
    /**
       Get the timer with the specified ID
    */

    virtual TVTimerEntry getTimer(const unsigned int& id)
      throw(const Exception&);
    /**
       Get all timers with status SLEEPING
    */

    virtual list<TVTimerEntry >  getSleepingTimers()
      throw(const Exception&);
    /**
       Get all timers with status RECORDING
    */

    virtual list<TVTimerEntry >  getRecordingTimers()
      throw(const Exception&);
    /**
       Get all timers with status RECORDED or RECORDED_WITH_ERRORS
    */

    virtual list<TVTimerEntry >  getRecordedTimers()
      throw(const Exception&);
    /**
       Get all timers witch Conflicts
     */

    virtual list<TVTimerEntry >  getConflictTimers()
      throw(const Exception&);
    /**
       Get all timers witch Conflicts with the given timer
     */

    virtual list<TVTimerEntry >  getConflictsWithTimer(const TVTimerEntry& entry)
      throw(const Exception&);
    /**
       Load timers from XML-File
    */

    virtual Result loadTimer(const string& filename)
      throw(const Exception&);
    /**
       Save timers to XMLFile
    */

    virtual Result saveTimer(const string& filename)
      throw(const Exception&);
    /**
       Set the pointer to the TVGuide
    */

    virtual void setTVGuide(const ITVGuide* itvg)
      throw(const Exception&);
    /**
       Get the pointer to the TVGuide
    */

    virtual ITVGuide* getTVGuide()
      throw(const Exception&);
    /**
       Set the directory where the recording should be stored
    */

    virtual void setDirectory(const string& dir)
      throw(const Exception&);
    /**
       Get the directory where the recordings are stored
    */

    virtual string getDirectory()
      throw(const Exception&);

    virtual TVChannelList getChannelConfig()
      throw(const Exception&);


  };

  typedef auto_ptr<ITVTimer> ITVTimer_var;




}

#endif
