
#include "IObjectAcceptor.hpp"
#include "nmm/base/InterfaceFactory.hpp"


namespace NMM
{
  
  
  void IObjectAcceptor::connectToObject(const NMMReference& reference) 
      throw(const Exception&, const BindingException&)
  {
    TInValue<NMMReference > reference_value(reference, false);
    Event event("IObjectAcceptor::connectToObject", &reference_value);
    if(sendEvent(event) == FAILURE) {
      if (event.getException()) {
        if(BindingException* ex = dynamic_cast<BindingException*>(event.getException())) {
          throw BindingException(*ex);
        }
        throw Exception(*event.getException());
      }
    }
  }
  
  void IObjectAcceptor::connectToService(const string& service) 
      throw(const Exception&, const BindingException&)
  {
    TInValue<string > service_value(service, false);
    Event event("IObjectAcceptor::connectToService", &service_value);
    if(sendEvent(event) == FAILURE) {
      if (event.getException()) {
        if(BindingException* ex = dynamic_cast<BindingException*>(event.getException())) {
          throw BindingException(*ex);
        }
        throw Exception(*event.getException());
      }
    }
  }
  
  list<string >  IObjectAcceptor::getServiceList() const
      throw(const Exception&)
  {
    TOutValue<list<string >  > return_value;
    Event event("IObjectAcceptor::getServiceList");
    event.setReturnValue(&return_value);
    if(sendEvent(event) == FAILURE) {
      if (event.getException()) {
        throw Exception(*event.getException());
      }
    }
    return return_value.getRValue();
  }

  const char* IObjectAcceptor::IObjectAcceptor_type = "IObjectAcceptor";
  TIFactoryObject<IObjectAcceptor > __IObjectAcceptor_prototype("IObjectAcceptor");


  const char* IObjectAcceptor::connectToObject_event = "IObjectAcceptor::connectToObject";
  const char* IObjectAcceptor::connectToService_event = "IObjectAcceptor::connectToService";
  const char* IObjectAcceptor::getServiceList_event = "IObjectAcceptor::getServiceList";


}


