
#include "IRTPControl.hpp"
#include "nmm/base/InterfaceFactory.hpp"


namespace NMM
{
  
  
  void IRTPControl::setCNAME(const string& cname)
      throw(const Exception&)
  {
    TInValue<string > cname_value(cname, false);
    Event event("IRTPControl::setCNAME", &cname_value);
    if(sendEvent(event) == FAILURE) {
      if (event.getException()) {
        throw Exception(*event.getException());
      }
    }
  }
  
  string IRTPControl::getCNAME() const
      throw(const Exception&)
  {
    TOutValue<string > return_value;
    Event event("IRTPControl::getCNAME");
    event.setReturnValue(&return_value);
    if(sendEvent(event) == FAILURE) {
      if (event.getException()) {
        throw Exception(*event.getException());
      }
    }
    return return_value.getRValue();
  }
  
  void IRTPControl::setTTL(const unsigned int& ttl)
      throw(const Exception&)
  {
    TInValue<unsigned int > ttl_value(ttl, false);
    Event event("IRTPControl::setTTL", &ttl_value);
    if(sendEvent(event) == FAILURE) {
      if (event.getException()) {
        throw Exception(*event.getException());
      }
    }
  }
  
  unsigned int IRTPControl::getTTL() const
      throw(const Exception&)
  {
    TOutValue<unsigned int > return_value;
    Event event("IRTPControl::getTTL");
    event.setReturnValue(&return_value);
    if(sendEvent(event) == FAILURE) {
      if (event.getException()) {
        throw Exception(*event.getException());
      }
    }
    return return_value.getRValue();
  }
  
  void IRTPControl::setBufferSize(const size_t& buffer_size)
      throw(const Exception&)
  {
    TInValue<size_t > buffer_size_value(buffer_size, false);
    Event event("IRTPControl::setBufferSize", &buffer_size_value);
    if(sendEvent(event) == FAILURE) {
      if (event.getException()) {
        throw Exception(*event.getException());
      }
    }
  }
  
  size_t IRTPControl::getBufferSize() const
      throw(const Exception&)
  {
    TOutValue<size_t > return_value;
    Event event("IRTPControl::getBufferSize");
    event.setReturnValue(&return_value);
    if(sendEvent(event) == FAILURE) {
      if (event.getException()) {
        throw Exception(*event.getException());
      }
    }
    return return_value.getRValue();
  }
  
  void IRTPControl::setReceiveBufferSize(const size_t& recv_buffer_size)
      throw(const Exception&)
  {
    TInValue<size_t > recv_buffer_size_value(recv_buffer_size, false);
    Event event("IRTPControl::setReceiveBufferSize", &recv_buffer_size_value);
    if(sendEvent(event) == FAILURE) {
      if (event.getException()) {
        throw Exception(*event.getException());
      }
    }
  }
  
  size_t IRTPControl::getReceiveBufferSize() const
      throw(const Exception&)
  {
    TOutValue<size_t > return_value;
    Event event("IRTPControl::getReceiveBufferSize");
    event.setReturnValue(&return_value);
    if(sendEvent(event) == FAILURE) {
      if (event.getException()) {
        throw Exception(*event.getException());
      }
    }
    return return_value.getRValue();
  }
  
  void IRTPControl::setMaxFrameSize(const size_t& max_frame_size)
      throw(const Exception&)
  {
    TInValue<size_t > max_frame_size_value(max_frame_size, false);
    Event event("IRTPControl::setMaxFrameSize", &max_frame_size_value);
    if(sendEvent(event) == FAILURE) {
      if (event.getException()) {
        throw Exception(*event.getException());
      }
    }
  }
  
  size_t IRTPControl::getMaxFrameSize() const
      throw(const Exception&)
  {
    TOutValue<size_t > return_value;
    Event event("IRTPControl::getMaxFrameSize");
    event.setReturnValue(&return_value);
    if(sendEvent(event) == FAILURE) {
      if (event.getException()) {
        throw Exception(*event.getException());
      }
    }
    return return_value.getRValue();
  }
  
  void IRTPControl::setSessionBandwidth(const unsigned int& buffer_size)
      throw(const Exception&)
  {
    TInValue<unsigned int > buffer_size_value(buffer_size, false);
    Event event("IRTPControl::setSessionBandwidth", &buffer_size_value);
    if(sendEvent(event) == FAILURE) {
      if (event.getException()) {
        throw Exception(*event.getException());
      }
    }
  }
  
  unsigned int IRTPControl::getSessionBandwidth() const
      throw(const Exception&)
  {
    TOutValue<unsigned int > return_value;
    Event event("IRTPControl::getSessionBandwidth");
    event.setReturnValue(&return_value);
    if(sendEvent(event) == FAILURE) {
      if (event.getException()) {
        throw Exception(*event.getException());
      }
    }
    return return_value.getRValue();
  }

  const char* IRTPControl::IRTPControl_type = "IRTPControl";
  TIFactoryObject<IRTPControl > __IRTPControl_prototype("IRTPControl");


  const char* IRTPControl::setCNAME_event = "IRTPControl::setCNAME";
  const char* IRTPControl::getCNAME_event = "IRTPControl::getCNAME";
  const char* IRTPControl::setTTL_event = "IRTPControl::setTTL";
  const char* IRTPControl::getTTL_event = "IRTPControl::getTTL";
  const char* IRTPControl::setBufferSize_event = "IRTPControl::setBufferSize";
  const char* IRTPControl::getBufferSize_event = "IRTPControl::getBufferSize";
  const char* IRTPControl::setReceiveBufferSize_event = "IRTPControl::setReceiveBufferSize";
  const char* IRTPControl::getReceiveBufferSize_event = "IRTPControl::getReceiveBufferSize";
  const char* IRTPControl::setMaxFrameSize_event = "IRTPControl::setMaxFrameSize";
  const char* IRTPControl::getMaxFrameSize_event = "IRTPControl::getMaxFrameSize";
  const char* IRTPControl::setSessionBandwidth_event = "IRTPControl::setSessionBandwidth";
  const char* IRTPControl::getSessionBandwidth_event = "IRTPControl::getSessionBandwidth";


}


