
#include "IUDPControl.hpp"
#include "nmm/base/InterfaceFactory.hpp"


namespace NMM
{
  
  
  void IUDPControl::setUsePacketStream(const bool& use_packet_stream)
      throw(const Exception&)
  {
    TInValue<bool > use_packet_stream_value(use_packet_stream, false);
    Event event("IUDPControl::setUsePacketStream", &use_packet_stream_value);
    if(sendEvent(event) == FAILURE) {
      if (event.getException()) {
        throw Exception(*event.getException());
      }
    }
  }
  
  bool IUDPControl::getUsePacketStream() const
      throw(const Exception&)
  {
    TOutValue<bool > return_value;
    Event event("IUDPControl::getUsePacketStream");
    event.setReturnValue(&return_value);
    if(sendEvent(event) == FAILURE) {
      if (event.getException()) {
        throw Exception(*event.getException());
      }
    }
    return return_value.getRValue();
  }
  
  void IUDPControl::setPacketSize(const ssize_t& size)
      throw(const Exception&)
  {
    TInValue<ssize_t > size_value(size, false);
    Event event("IUDPControl::setPacketSize", &size_value);
    if(sendEvent(event) == FAILURE) {
      if (event.getException()) {
        throw Exception(*event.getException());
      }
    }
  }
  
  ssize_t IUDPControl::getPacketSize() const
      throw(const Exception&)
  {
    TOutValue<ssize_t > return_value;
    Event event("IUDPControl::getPacketSize");
    event.setReturnValue(&return_value);
    if(sendEvent(event) == FAILURE) {
      if (event.getException()) {
        throw Exception(*event.getException());
      }
    }
    return return_value.getRValue();
  }
  
  void IUDPControl::setMaxPartialMessages(const unsigned int& max_partial)
      throw(const Exception&)
  {
    TInValue<unsigned int > max_partial_value(max_partial, false);
    Event event("IUDPControl::setMaxPartialMessages", &max_partial_value);
    if(sendEvent(event) == FAILURE) {
      if (event.getException()) {
        throw Exception(*event.getException());
      }
    }
  }
  
  unsigned int IUDPControl::getMaxPartialMessages() const
      throw(const Exception&)
  {
    TOutValue<unsigned int > return_value;
    Event event("IUDPControl::getMaxPartialMessages");
    event.setReturnValue(&return_value);
    if(sendEvent(event) == FAILURE) {
      if (event.getException()) {
        throw Exception(*event.getException());
      }
    }
    return return_value.getRValue();
  }
  
  void IUDPControl::resetStatistics()
      throw(const Exception&)
  {
    Event event("IUDPControl::resetStatistics");
    if(sendEvent(event) == FAILURE) {
      if (event.getException()) {
        throw Exception(*event.getException());
      }
    }
  }
  
  ssize_t IUDPControl::getTotalSentBytes() const
      throw(const Exception&)
  {
    TOutValue<ssize_t > return_value;
    Event event("IUDPControl::getTotalSentBytes");
    event.setReturnValue(&return_value);
    if(sendEvent(event) == FAILURE) {
      if (event.getException()) {
        throw Exception(*event.getException());
      }
    }
    return return_value.getRValue();
  }
  
  ssize_t IUDPControl::getTotalReceivedBytes() const
      throw(const Exception&)
  {
    TOutValue<ssize_t > return_value;
    Event event("IUDPControl::getTotalReceivedBytes");
    event.setReturnValue(&return_value);
    if(sendEvent(event) == FAILURE) {
      if (event.getException()) {
        throw Exception(*event.getException());
      }
    }
    return return_value.getRValue();
  }
  
  unsigned int IUDPControl::getTotalSentPackets() const
      throw(const Exception&)
  {
    TOutValue<unsigned int > return_value;
    Event event("IUDPControl::getTotalSentPackets");
    event.setReturnValue(&return_value);
    if(sendEvent(event) == FAILURE) {
      if (event.getException()) {
        throw Exception(*event.getException());
      }
    }
    return return_value.getRValue();
  }
  
  unsigned int IUDPControl::getTotalReceivedPackets() const
      throw(const Exception&)
  {
    TOutValue<unsigned int > return_value;
    Event event("IUDPControl::getTotalReceivedPackets");
    event.setReturnValue(&return_value);
    if(sendEvent(event) == FAILURE) {
      if (event.getException()) {
        throw Exception(*event.getException());
      }
    }
    return return_value.getRValue();
  }
  
  unsigned int IUDPControl::getTotalDiscardedPackets() const
      throw(const Exception&)
  {
    TOutValue<unsigned int > return_value;
    Event event("IUDPControl::getTotalDiscardedPackets");
    event.setReturnValue(&return_value);
    if(sendEvent(event) == FAILURE) {
      if (event.getException()) {
        throw Exception(*event.getException());
      }
    }
    return return_value.getRValue();
  }
  
  unsigned int IUDPControl::getTotalLostPackets() const
      throw(const Exception&)
  {
    TOutValue<unsigned int > return_value;
    Event event("IUDPControl::getTotalLostPackets");
    event.setReturnValue(&return_value);
    if(sendEvent(event) == FAILURE) {
      if (event.getException()) {
        throw Exception(*event.getException());
      }
    }
    return return_value.getRValue();
  }

  const char* IUDPControl::IUDPControl_type = "IUDPControl";
  TIFactoryObject<IUDPControl > __IUDPControl_prototype("IUDPControl");


  const char* IUDPControl::setUsePacketStream_event = "IUDPControl::setUsePacketStream";
  const char* IUDPControl::getUsePacketStream_event = "IUDPControl::getUsePacketStream";
  const char* IUDPControl::setPacketSize_event = "IUDPControl::setPacketSize";
  const char* IUDPControl::getPacketSize_event = "IUDPControl::getPacketSize";
  const char* IUDPControl::setMaxPartialMessages_event = "IUDPControl::setMaxPartialMessages";
  const char* IUDPControl::getMaxPartialMessages_event = "IUDPControl::getMaxPartialMessages";
  const char* IUDPControl::resetStatistics_event = "IUDPControl::resetStatistics";
  const char* IUDPControl::getTotalSentBytes_event = "IUDPControl::getTotalSentBytes";
  const char* IUDPControl::getTotalReceivedBytes_event = "IUDPControl::getTotalReceivedBytes";
  const char* IUDPControl::getTotalSentPackets_event = "IUDPControl::getTotalSentPackets";
  const char* IUDPControl::getTotalReceivedPackets_event = "IUDPControl::getTotalReceivedPackets";
  const char* IUDPControl::getTotalDiscardedPackets_event = "IUDPControl::getTotalDiscardedPackets";
  const char* IUDPControl::getTotalLostPackets_event = "IUDPControl::getTotalLostPackets";


}


