
#ifndef NMM_IUDPControlImpl_HPP
#define NMM_IUDPControlImpl_HPP

#include "nmm/NMMTypes.hpp"
#include "nmm/base/NMMObject.hpp"
#include "IUDPControl.hpp"

#include "IIPControlImpl.hpp"

namespace NMM
{
  /** UDP control interface.
   * This interface is provided by the UDPStrategy for setting options and reading
   * statistics for the UDP transport protocol.
   */

  class IUDPControlImpl  : virtual public IIPControlImpl { 
  public:
    IUDPControlImpl() { addInterface(new IUDPControl(*this)); registerEvents(); };

    virtual ~IUDPControlImpl() {};

  protected:
    /** @name Packet Stream Options
     */
    /** This method sets whether the UDPStrategy should use the PacketStream.
     * The packet stream guarantees consistent, yet unreliable, delivery of NMM messages.
     */

    virtual void setUsePacketStream(const bool& use_packet_stream) = 0;
    /** This method gets whether the UDPStrategy should use the PacketStream.
     * The packet stream guarantees consistent, yet unreliable, delivery of NMM messages.
     */

    virtual bool getUsePacketStream() const = 0;
    /** This method sets the maximum size of a UDP packet.
     */

    virtual void setPacketSize(const ssize_t& size) = 0;
    /** This method returns the maximum size of a UDP packet.
     */

    virtual ssize_t getPacketSize() const = 0;
    /** This method sets the maximum number of partial messages in the receive queue.
     * Packets in the receive queue with a message ID less than the value set by this method are
     * thrown away.
     */

    virtual void setMaxPartialMessages(const unsigned int& max_partial) = 0;
    /** This method returns the maximum number of partial messages in the receive queue.
     * Packets in the receive queue with a message ID less than the value returned by this method are
     * thrown away.
     */

    virtual unsigned int getMaxPartialMessages() const = 0;
    /** @name Protocol Statistics
     */
    /** This method resets all statistics in the UDPStrategy
     */

    virtual void resetStatistics() = 0;
    /** This method returns the total amount of bytes sent.
     */

    virtual ssize_t getTotalSentBytes() const = 0;
    /** This method returns the total amount of bytes received.
     */

    virtual ssize_t getTotalReceivedBytes() const = 0;
    /** This method returns the total number of packets sent
     */

    virtual unsigned int getTotalSentPackets() const = 0;
    /** This method returns the total number of packets received
     */

    virtual unsigned int getTotalReceivedPackets() const = 0;
    /** This method returns the total number of discarded incoming packets
     */

    virtual unsigned int getTotalDiscardedPackets() const = 0;
    /** This method returns the total number of lost incoming packets
     */

    virtual unsigned int getTotalLostPackets() const = 0;


  private:
    void registerEvents();
  };

}

#endif
