
#ifndef NMM_IFramerateConverterImpl_HPP
#define NMM_IFramerateConverterImpl_HPP

#include "nmm/NMMTypes.hpp"
#include "nmm/base/NMMObject.hpp"
#include "IFramerateConverter.hpp"


namespace NMM
{
  /**
   * Framerate converter interface.
   */

  class IFramerateConverterImpl  : public virtual NMMObject { 
  public:
    IFramerateConverterImpl() { addInterface(new IFramerateConverter(*this)); registerEvents(); };

    virtual ~IFramerateConverterImpl() {};

  protected:
    /**
     * Set time offset.
     * Set the time offset for the output stream. The output stream normally begins at the
     * time of the first incoming buffer. This method can be used to shift the timestamp of
     * the first output buffer relative to the time of the first incoming buffer.
     *
     * If this method is called in STARTED, it will take effect after the next syncReset event.
     */

    virtual void setTimeOffset(const Interval& start) = 0;
    /**
     * Set time offset.
     * Set the time offset for the output stream. The output stream normally begins at the
     * time of the first incoming buffer. This method can be used to shift the timestamp of
     * the first output buffer relative to the time of the first incoming buffer.
     *
     * If this method is called in STARTED, it will take effect after the next syncReset event.
     */

    virtual void setTimeOffsetMillisecs(const int& millisecs) = 0;
    /**
     * Set frame offset for interpolation.
     * This method controls where in the output video stream an input video frame exactly starts.
     * The value should be between 0 and 1. A value of zero means that an input frame starts at
     * exactly its own input timestamp. A value of 0.5 (which is the default) means that the
     * frame starts at the average between its timestamp and its predecessor's timestamp.
     */

    virtual void setFrameOffset(const float& alpha) = 0;
    /**
     * Get statistics since last sync reset.
     * Get total number of input buffers.
     */

    virtual int getTotalInBuffers() const = 0;
    /**
     * Get statistics since last sync reset.
     * Get total number of output buffers.
     */

    virtual int getTotalOutBuffers() const = 0;
    /**
     * Get statistics since last sync reset.
     * Get total number of buffers dropped.
     */

    virtual int getTotalDroppedBuffers() const = 0;
    /**
     * Get statistics since last sync reset.
     * Get total number of copied created.
     */

    virtual int getTotalCopiedBuffers() const = 0;


  private:
    void registerEvents();
  };

}

#endif
