
#ifndef NMM_ICamera_HPP
#define NMM_ICamera_HPP

#include "nmm/NMMTypes.hpp"
#include "nmm/base/Interface.hpp"
#include "nmm/base/Event.hpp"
#include <memory>


namespace NMM
{

  class ICamera : virtual public Interface { 
  public:
    ICamera() {  setTypeInfo(TypeInfo(ICamera::ICamera_type, "NMM")); };

    ICamera(NMMObject& parent) : Interface(parent) {  setTypeInfo(TypeInfo(ICamera::ICamera_type, "NMM")); setEventReceiver(parent); };

    virtual ~ICamera() {};

    static const char* ICamera_type;

    virtual  string getInterfaceType() const { return ICamera_type; }

    Interface* copy() const { return new ICamera(*this); }
    static const char* setBrightness_event;
    static const char* setAutoExposure_event;
    static const char* setSharpness_event;
    static const char* setWhiteBalance_event;
    static const char* setBlueBalance_event;
    static const char* setRedBalance_event;
    static const char* setHue_event;
    static const char* setSaturation_event;
    static const char* setGamma_event;
    static const char* setShutter_event;
    static const char* setGain_event;
    static const char* setIris_event;
    static const char* setFocus_event;
    static const char* setZoom_event;
    static const char* onePush_event;
    static const char* getBrightness_event;
    static const char* getAutoExposure_event;
    static const char* getSharpness_event;
    static const char* getWhiteBalance_event;
    static const char* getBlueBalance_event;
    static const char* getRedBalance_event;
    static const char* getHue_event;
    static const char* getSaturation_event;
    static const char* getGamma_event;
    static const char* getShutter_event;
    static const char* getGain_event;
    static const char* getIris_event;
    static const char* getFocus_event;
    static const char* getZoom_event;
    static const char* getBrightnessRange_event;
    static const char* getAutoExposureRange_event;
    static const char* getSharpnessRange_event;
    static const char* getWhiteBalanceRange_event;
    static const char* getHueRange_event;
    static const char* getSaturationRange_event;
    static const char* getGammaRange_event;
    static const char* getShutterRange_event;
    static const char* getGainRange_event;
    static const char* getIrisRange_event;
    static const char* getFocusRange_event;
    static const char* getZoomRange_event;
    static const char* hasBrightness_event;
    static const char* hasAutoExposure_event;
    static const char* hasSharpness_event;
    static const char* hasWhiteBalance_event;
    static const char* hasHue_event;
    static const char* hasSaturation_event;
    static const char* hasGamma_event;
    static const char* hasShutter_event;
    static const char* hasGain_event;
    static const char* hasIris_event;
    static const char* hasFocus_event;
    static const char* hasZoom_event;

    virtual void setBrightness(const int& value)
      throw(const Exception&);

    virtual void setAutoExposure(const int& value)
      throw(const Exception&);

    virtual void setSharpness(const int& value)
      throw(const Exception&);

    virtual void setWhiteBalance(const int& value)
      throw(const Exception&);

    virtual void setBlueBalance(const int& value)
      throw(const Exception&);

    virtual void setRedBalance(const int& value)
      throw(const Exception&);

    virtual void setHue(const int& value)
      throw(const Exception&);

    virtual void setSaturation(const int& value)
      throw(const Exception&);

    virtual void setGamma(const int& value)
      throw(const Exception&);

    virtual void setShutter(const int& value)
      throw(const Exception&);

    virtual void setGain(const int& value)
      throw(const Exception&);

    virtual void setIris(const int& value)
      throw(const Exception&);

    virtual void setFocus(const int& value)
      throw(const Exception&);

    virtual void setZoom(const int& value)
      throw(const Exception&);

    virtual void onePush()
      throw(const Exception&);

    virtual int getBrightness() const
      throw(const Exception&);

    virtual int getAutoExposure() const
      throw(const Exception&);

    virtual int getSharpness() const
      throw(const Exception&);

    virtual int getWhiteBalance() const
      throw(const Exception&);

    virtual int getBlueBalance() const
      throw(const Exception&);

    virtual int getRedBalance() const
      throw(const Exception&);

    virtual int getHue() const
      throw(const Exception&);

    virtual int getSaturation() const
      throw(const Exception&);

    virtual int getGamma() const
      throw(const Exception&);

    virtual int getShutter() const
      throw(const Exception&);

    virtual int getGain() const
      throw(const Exception&);

    virtual int getIris() const
      throw(const Exception&);

    virtual int getFocus() const
      throw(const Exception&);

    virtual int getZoom() const
      throw(const Exception&);

    virtual pair<int, int >  getBrightnessRange() const
      throw(const Exception&);

    virtual pair<int, int >  getAutoExposureRange() const
      throw(const Exception&);

    virtual pair<int, int >  getSharpnessRange() const
      throw(const Exception&);

    virtual pair<int, int >  getWhiteBalanceRange() const
      throw(const Exception&);

    virtual pair<int, int >  getHueRange() const
      throw(const Exception&);

    virtual pair<int, int >  getSaturationRange() const
      throw(const Exception&);

    virtual pair<int, int >  getGammaRange() const
      throw(const Exception&);

    virtual pair<int, int >  getShutterRange() const
      throw(const Exception&);

    virtual pair<int, int >  getGainRange() const
      throw(const Exception&);

    virtual pair<int, int >  getIrisRange() const
      throw(const Exception&);

    virtual pair<int, int >  getFocusRange() const
      throw(const Exception&);

    virtual pair<int, int >  getZoomRange() const
      throw(const Exception&);

    virtual bool hasBrightness() const
      throw(const Exception&);

    virtual bool hasAutoExposure() const
      throw(const Exception&);

    virtual bool hasSharpness() const
      throw(const Exception&);

    virtual bool hasWhiteBalance() const
      throw(const Exception&);

    virtual bool hasHue() const
      throw(const Exception&);

    virtual bool hasSaturation() const
      throw(const Exception&);

    virtual bool hasGamma() const
      throw(const Exception&);

    virtual bool hasShutter() const
      throw(const Exception&);

    virtual bool hasGain() const
      throw(const Exception&);

    virtual bool hasIris() const
      throw(const Exception&);

    virtual bool hasFocus() const
      throw(const Exception&);

    virtual bool hasZoom() const
      throw(const Exception&);


  };

  typedef auto_ptr<ICamera> ICamera_var;




}

#endif
