
#include "IVideoDisplay.hpp"
#include "nmm/base/InterfaceFactory.hpp"


namespace NMM
{
  
  
  void IVideoDisplay::setFullscreen(const bool& value)
      throw(const Exception&)
  {
    TInValue<bool > value_value(value, false);
    Event event("IVideoDisplay::setFullscreen", &value_value);
    if(sendEvent(event) == FAILURE) {
      if (event.getException()) {
        throw Exception(*event.getException());
      }
    }
  }
  
  bool IVideoDisplay::getFullscreen() const
      throw(const Exception&)
  {
    TOutValue<bool > return_value;
    Event event("IVideoDisplay::getFullscreen");
    event.setReturnValue(&return_value);
    if(sendEvent(event) == FAILURE) {
      if (event.getException()) {
        throw Exception(*event.getException());
      }
    }
    return return_value.getRValue();
  }
  
  bool IVideoDisplay::hasFullscreen() const
      throw(const Exception&)
  {
    TOutValue<bool > return_value;
    Event event("IVideoDisplay::hasFullscreen");
    event.setReturnValue(&return_value);
    if(sendEvent(event) == FAILURE) {
      if (event.getException()) {
        throw Exception(*event.getException());
      }
    }
    return return_value.getRValue();
  }
  
  bool IVideoDisplay::isFullscreenOnly() const
      throw(const Exception&)
  {
    TOutValue<bool > return_value;
    Event event("IVideoDisplay::isFullscreenOnly");
    event.setReturnValue(&return_value);
    if(sendEvent(event) == FAILURE) {
      if (event.getException()) {
        throw Exception(*event.getException());
      }
    }
    return return_value.getRValue();
  }
  
  bool IVideoDisplay::makeScreenshot(const string& filename) const
      throw(const Exception&)
  {
    TOutValue<bool > return_value;
    TInValue<string > filename_value(filename, false);
    Event event("IVideoDisplay::makeScreenshot", &filename_value);
    event.setReturnValue(&return_value);
    if(sendEvent(event) == FAILURE) {
      if (event.getException()) {
        throw Exception(*event.getException());
      }
    }
    return return_value.getRValue();
  }
  
  void IVideoDisplay::setLeftWindowAlignment() /* limited(INode::CONSTRUCTED) */ /* limited(INode::INITIALIZED) */
      throw(const Exception&, const IllegalStateException&)
  {
    Event event("IVideoDisplay::setLeftWindowAlignment");
    if(sendEvent(event) == FAILURE) {
      if (event.getException()) {
        if(IllegalStateException* ex = dynamic_cast<IllegalStateException*>(event.getException())) {
          throw IllegalStateException(*ex);
        }
        throw Exception(*event.getException());
      }
    }
  }
  
  void IVideoDisplay::setRightWindowAlignment() /* limited(INode::CONSTRUCTED) */ /* limited(INode::INITIALIZED) */
      throw(const Exception&, const IllegalStateException&)
  {
    Event event("IVideoDisplay::setRightWindowAlignment");
    if(sendEvent(event) == FAILURE) {
      if (event.getException()) {
        if(IllegalStateException* ex = dynamic_cast<IllegalStateException*>(event.getException())) {
          throw IllegalStateException(*ex);
        }
        throw Exception(*event.getException());
      }
    }
  }
  
  void IVideoDisplay::setCenterWindowAlignment() /* limited(INode::CONSTRUCTED) */ /* limited(INode::INITIALIZED) */
      throw(const Exception&, const IllegalStateException&)
  {
    Event event("IVideoDisplay::setCenterWindowAlignment");
    if(sendEvent(event) == FAILURE) {
      if (event.getException()) {
        if(IllegalStateException* ex = dynamic_cast<IllegalStateException*>(event.getException())) {
          throw IllegalStateException(*ex);
        }
        throw Exception(*event.getException());
      }
    }
  }

  const char* IVideoDisplay::IVideoDisplay_type = "IVideoDisplay";
  TIFactoryObject<IVideoDisplay > __IVideoDisplay_prototype("IVideoDisplay");


  const char* IVideoDisplay::setFullscreen_event = "IVideoDisplay::setFullscreen";
  const char* IVideoDisplay::getFullscreen_event = "IVideoDisplay::getFullscreen";
  const char* IVideoDisplay::hasFullscreen_event = "IVideoDisplay::hasFullscreen";
  const char* IVideoDisplay::isFullscreenOnly_event = "IVideoDisplay::isFullscreenOnly";
  const char* IVideoDisplay::makeScreenshot_event = "IVideoDisplay::makeScreenshot";
  const char* IVideoDisplay::setLeftWindowAlignment_event = "IVideoDisplay::setLeftWindowAlignment";
  const char* IVideoDisplay::setRightWindowAlignment_event = "IVideoDisplay::setRightWindowAlignment";
  const char* IVideoDisplay::setCenterWindowAlignment_event = "IVideoDisplay::setCenterWindowAlignment";


}


