
#ifndef NMM_IProgressBar_HPP
#define NMM_IProgressBar_HPP

#include "nmm/NMMTypes.hpp"
#include "nmm/base/Interface.hpp"
#include "nmm/base/Event.hpp"
#include <memory>

#include "IWidget.hpp"

namespace NMM
{
  /**
   * Progress bar widget interface.
   */

  class IProgressBar : virtual public IWidget { 
  public:
    IProgressBar() {  setTypeInfo(TypeInfo(IProgressBar::IProgressBar_type, "NMM")); };

    IProgressBar(NMMObject& parent) : IWidget(parent) {  setTypeInfo(TypeInfo(IProgressBar::IProgressBar_type, "NMM")); setEventReceiver(parent); };

    virtual ~IProgressBar() {};

    static const char* IProgressBar_type;

    virtual  string getInterfaceType() const { return IProgressBar_type; }

    Interface* copy() const { return new IProgressBar(*this); }
    static const char* setMaxProgress_event;
    static const char* setProgress_event;
    static const char* setBarColor_event;
    static const char* setBarSpaces_event;
    /** Set maximum value of progress bar. */

    virtual void setMaxProgress(const u_int64_t& value)
      throw(const Exception&);
    /** Set current progress value. */

    virtual void setProgress(const u_int64_t& value)
      throw(const Exception&);
    /**
     * Set bar color.
     * @param value bar color (rgba format)
     */

    virtual void setBarColor(const u_int32_t& value)
      throw(const Exception&);
    /**
     * Definition of bar spaces.
     * @param space space between progress bars
     * @param space_step gap between spaces
     */

    virtual void setBarSpaces(const unsigned int& space, const unsigned int& space_step)
      throw(const Exception&);


  };

  typedef auto_ptr<IProgressBar> IProgressBar_var;




}

#endif
