
#ifndef NMM_IWidgetFactoryImpl_HPP
#define NMM_IWidgetFactoryImpl_HPP

#include "nmm/NMMTypes.hpp"
#include "nmm/base/NMMObject.hpp"
#include "IWidgetFactory.hpp"

#include "IWidgetImpl.hpp"
#include "IWidgetCompositeImpl.hpp"
#include "IProgressBarImpl.hpp"
#include "ITextViewImpl.hpp"
#include "ITimeViewImpl.hpp"
#include "IImageViewImpl.hpp"
#include "IRadioButtonsImpl.hpp"
#include "IButtonsImpl.hpp"
#include "IBorderDecoratorImpl.hpp"
#include "IHeaderDecoratorImpl.hpp"
#include "IPlaneDecoratorImpl.hpp"
#include "IScrollDecoratorImpl.hpp"

namespace NMM
{
  /**
   * Widget factory.
   * This interface, which is implemented by the OSD manager, is used
   * for creating widget objects. I wrote a seperate interface for this for the
   * following reasons:
   * - It has a lot of methods only for creating widgets that would inflate IOSDManager
   * - It depends on all the widget interfaces. I don't want IOSDManager to include them all
   */

  class IWidgetFactoryImpl  : public virtual NMMObject { 
  public:
    IWidgetFactoryImpl() { addInterface(new IWidgetFactory(*this)); registerEvents(); };

    virtual ~IWidgetFactoryImpl() {};

  protected:
    /**
     * Create a widget composite.
     * Creates a new widget and returns an interface for it.
     * @return an interface for this widget
     */

    virtual IWidgetComposite* createWidgetComposite() = 0;
    /**
     * Create a progress bar.
     * Creates a new widget and returns an interface for it.
     * @return an interface for this widget
     */

    virtual IProgressBar* createProgressBar() = 0;
    /**
     * Create a text view.
     * Creates a new widget and returns an interface for it.
     * @return an interface for this widget
     */

    virtual ITextView* createTextView() = 0;
    /**
     * Create a time view.
     * Creates a new widget and returns an interface for it.
     * @return an interface for this widget
     */

    virtual ITimeView* createTimeView() = 0;
    /**
     * Create an image view.
     * Creates a new widget and returns an interface for it.
     * @return an interface for this widget
     */

    virtual IImageView* createImageView() = 0;
    /**
     * Create radio buttons.
     * Creates a new widget and returns an interface for it.
     * @return an interface for this widget
     */

    virtual IRadioButtons* createRadioButtons() = 0;
    /**
     * Create buttons.
     * Creates a new widget and returns an interface for it.
     * @return an interface for this widget
     */

    virtual IButtons* createButtons() = 0;
    /**
     * Create border decorator.
     * Creates a new widget and returns an interface for it.
     * @param internal the internal widget of the decorator
     * @return an interface for this widget
     */

    virtual IBorderDecorator* createBorderDecorator(const IWidget& internal) = 0;
    /**
     * Create border decorator.
     * Creates a new widget and returns an interface for it.
     * @param internal the internal widget of the decorator
     * @return an interface for this widget
     */

    virtual IHeaderDecorator* createHeaderDecorator(const IWidget& internal) = 0;
    /**
     * Create plane decorator.
     * Creates a new widget and returns an interface for it.
     * @param internal the internal widget of the decorator
     * @return an interface for this widget
     */

    virtual IPlaneDecorator* createPlaneDecorator(const IWidget& internal) = 0;
    /**
     * Create scroll decorator.
     * Creates a new widget and returns an interface for it.
     * @param internal the internal widget of the decorator
     * @return an interface for this widget
     */

    virtual IScrollDecorator* createScrollDecorator(const IWidget& internal) = 0;


  private:
    void registerEvents();
  };

}

#endif
