
#ifndef NMM_IWindowImpl_HPP
#define NMM_IWindowImpl_HPP

#include "nmm/NMMTypes.hpp"
#include "nmm/base/NMMObject.hpp"
#include "IWindow.hpp"

#include "IWidgetImpl.hpp"

namespace NMM
{
  /**
   * Window interface.
   * A window is a container for a widget in an OSDManager.
   * This interface can be retrieved through an IWidget or IOSDManager interface.
   */

  class IWindowImpl  : public virtual NMMObject { 
  public:
    IWindowImpl() { addInterface(new IWindow(*this)); registerEvents(); };

    virtual ~IWindowImpl() {};

  protected:
    /** Get window ID. */

    virtual int getWindowID() const = 0;
    /**
     * Set position of the window.
     * @param x x position of the upper left corner
     * @param y y position of the upper left corner
     */

    virtual void setPosition(const unsigned int& x, const unsigned int& y) = 0;
    /**
     * Get x and y position of the window.
     * @param x x position of the upper left corner
     * @param y y position of the upper left corner
     */

    virtual void getPosition(unsigned int& x, unsigned int& y) const = 0;
    /** Set width and height of the window. */

    virtual void setDimension(const unsigned int& w, const unsigned int& h) = 0;
    /** Get width and height of the window. */

    virtual void getDimension(unsigned int& w, unsigned int& h) const = 0;
    /**
     * Redraw the window with help of the widget.
     * Sets updateflag to true.
     */

    virtual void update() = 0;
    /**
     * Check if window is visible
     * @return true if window is visible
     */

    virtual bool getVisible() const = 0;
    /**
     * Set window visibility
     * @param show true to make window visible
     */

    virtual void setVisible(const bool& value) = 0;
    /**
     * Set window scale factor.
     */

    virtual void setScaleFactor(const double& x, const double& y) = 0;
    /**
     * Set content.
     * Set the widget that this window contains.
     */

    virtual void setContent(const IWidget& value) = 0;
    /**
     * Get content.
     * Get the widget that this window contains.
     */

    virtual IWidget* getContent() const = 0;


  private:
    void registerEvents();
  };

}

#endif
