
#include "ISPUDecoder.hpp"
#include "nmm/base/InterfaceFactory.hpp"


namespace NMM
{
  
  
  Result ISPUDecoder::setPaletteItem(const int& index, const int& color_y, const int& color_u, const int& color_v, const int& alpha) /* in-stream */ 
      throw(const Exception&)
  {
    TOutValue<Result > return_value;
    TInValue<int > index_value(index, false);
    TInValue<int > color_y_value(color_y, false);
    TInValue<int > color_u_value(color_u, false);
    TInValue<int > color_v_value(color_v, false);
    TInValue<int > alpha_value(alpha, false);
    Event event("ISPUDecoder::setPaletteItem", &index_value, &color_y_value, &color_u_value, &color_v_value, &alpha_value);
    event.setReturnValue(&return_value);
    if(sendEvent(event) == FAILURE) {
      if (event.getException()) {
        throw Exception(*event.getException());
      }
    }
    return return_value.getRValue();
  }
  
  Result ISPUDecoder::initColorScheme(const int& scheme_nr, const int& selection, const int& activation) /* in-stream */ 
      throw(const Exception&)
  {
    TOutValue<Result > return_value;
    TInValue<int > scheme_nr_value(scheme_nr, false);
    TInValue<int > selection_value(selection, false);
    TInValue<int > activation_value(activation, false);
    Event event("ISPUDecoder::initColorScheme", &scheme_nr_value, &selection_value, &activation_value);
    event.setReturnValue(&return_value);
    if(sendEvent(event) == FAILURE) {
      if (event.getException()) {
        throw Exception(*event.getException());
      }
    }
    return return_value.getRValue();
  }
  
  Result ISPUDecoder::setDVDSelectedButton(const int& button) /* in-stream */ 
      throw(const Exception&)
  {
    TOutValue<Result > return_value;
    TInValue<int > button_value(button, false);
    Event event("ISPUDecoder::setDVDSelectedButton", &button_value);
    event.setReturnValue(&return_value);
    if(sendEvent(event) == FAILURE) {
      if (event.getException()) {
        throw Exception(*event.getException());
      }
    }
    return return_value.getRValue();
  }
  
  Result ISPUDecoder::setDVDButtonCount(const int& buttons) /* in-stream */ 
      throw(const Exception&)
  {
    TOutValue<Result > return_value;
    TInValue<int > buttons_value(buttons, false);
    Event event("ISPUDecoder::setDVDButtonCount", &buttons_value);
    event.setReturnValue(&return_value);
    if(sendEvent(event) == FAILURE) {
      if (event.getException()) {
        throw Exception(*event.getException());
      }
    }
    return return_value.getRValue();
  }
  
  Result ISPUDecoder::setDVDButtonCoords(const int& start_x, const int& start_y, const int& end_x, const int& end_y) /* in-stream */ 
      throw(const Exception&)
  {
    TOutValue<Result > return_value;
    TInValue<int > start_x_value(start_x, false);
    TInValue<int > start_y_value(start_y, false);
    TInValue<int > end_x_value(end_x, false);
    TInValue<int > end_y_value(end_y, false);
    Event event("ISPUDecoder::setDVDButtonCoords", &start_x_value, &start_y_value, &end_x_value, &end_y_value);
    event.setReturnValue(&return_value);
    if(sendEvent(event) == FAILURE) {
      if (event.getException()) {
        throw Exception(*event.getException());
      }
    }
    return return_value.getRValue();
  }
  
  Result ISPUDecoder::setDVDButtonScheme(const int& scheme) /* in-stream */ 
      throw(const Exception&)
  {
    TOutValue<Result > return_value;
    TInValue<int > scheme_value(scheme, false);
    Event event("ISPUDecoder::setDVDButtonScheme", &scheme_value);
    event.setReturnValue(&return_value);
    if(sendEvent(event) == FAILURE) {
      if (event.getException()) {
        throw Exception(*event.getException());
      }
    }
    return return_value.getRValue();
  }

  const char* ISPUDecoder::ISPUDecoder_type = "ISPUDecoder";
  TIFactoryObject<ISPUDecoder > __ISPUDecoder_prototype("ISPUDecoder");


  const char* ISPUDecoder::setPaletteItem_event = "ISPUDecoder::setPaletteItem";
  Event* ISPUDecoder::create_setPaletteItem(const int& index, const int& color_y, const int& color_u, const int& color_v, const int& alpha)
  {
    TInValue<int >* index_value = new TInValue<int >();
    index_value->setValue(index);
    TInValue<int >* color_y_value = new TInValue<int >();
    color_y_value->setValue(color_y);
    TInValue<int >* color_u_value = new TInValue<int >();
    color_u_value->setValue(color_u);
    TInValue<int >* color_v_value = new TInValue<int >();
    color_v_value->setValue(color_v);
    TInValue<int >* alpha_value = new TInValue<int >();
    alpha_value->setValue(alpha);

    return new Event("ISPUDecoder::setPaletteItem", index_value, color_y_value, color_u_value, color_v_value, alpha_value);
  }

  const char* ISPUDecoder::initColorScheme_event = "ISPUDecoder::initColorScheme";
  Event* ISPUDecoder::create_initColorScheme(const int& scheme_nr, const int& selection, const int& activation)
  {
    TInValue<int >* scheme_nr_value = new TInValue<int >();
    scheme_nr_value->setValue(scheme_nr);
    TInValue<int >* selection_value = new TInValue<int >();
    selection_value->setValue(selection);
    TInValue<int >* activation_value = new TInValue<int >();
    activation_value->setValue(activation);

    return new Event("ISPUDecoder::initColorScheme", scheme_nr_value, selection_value, activation_value);
  }

  const char* ISPUDecoder::setDVDSelectedButton_event = "ISPUDecoder::setDVDSelectedButton";
  Event* ISPUDecoder::create_setDVDSelectedButton(const int& button)
  {
    TInValue<int >* button_value = new TInValue<int >();
    button_value->setValue(button);

    return new Event("ISPUDecoder::setDVDSelectedButton", button_value);
  }

  const char* ISPUDecoder::setDVDButtonCount_event = "ISPUDecoder::setDVDButtonCount";
  Event* ISPUDecoder::create_setDVDButtonCount(const int& buttons)
  {
    TInValue<int >* buttons_value = new TInValue<int >();
    buttons_value->setValue(buttons);

    return new Event("ISPUDecoder::setDVDButtonCount", buttons_value);
  }

  const char* ISPUDecoder::setDVDButtonCoords_event = "ISPUDecoder::setDVDButtonCoords";
  Event* ISPUDecoder::create_setDVDButtonCoords(const int& start_x, const int& start_y, const int& end_x, const int& end_y)
  {
    TInValue<int >* start_x_value = new TInValue<int >();
    start_x_value->setValue(start_x);
    TInValue<int >* start_y_value = new TInValue<int >();
    start_y_value->setValue(start_y);
    TInValue<int >* end_x_value = new TInValue<int >();
    end_x_value->setValue(end_x);
    TInValue<int >* end_y_value = new TInValue<int >();
    end_y_value->setValue(end_y);

    return new Event("ISPUDecoder::setDVDButtonCoords", start_x_value, start_y_value, end_x_value, end_y_value);
  }

  const char* ISPUDecoder::setDVDButtonScheme_event = "ISPUDecoder::setDVDButtonScheme";
  Event* ISPUDecoder::create_setDVDButtonScheme(const int& scheme)
  {
    TInValue<int >* scheme_value = new TInValue<int >();
    scheme_value->setValue(scheme);

    return new Event("ISPUDecoder::setDVDButtonScheme", scheme_value);
  }



}


