
#ifndef NMM_IIVTV_HPP
#define NMM_IIVTV_HPP

#include "nmm/NMMTypes.hpp"
#include "nmm/base/Interface.hpp"
#include "nmm/base/Event.hpp"
#include <memory>

#include "nmm/interfaces/video/tv/ExtIVTVStructs.hpp"

#include "ITV.hpp"

namespace NMM
{
  /** Interface for IVTV driver.
   */

  class IIVTV : virtual public ITV { 
  public:
    IIVTV() {  setTypeInfo(TypeInfo(IIVTV::IIVTV_type, "NMM")); };

    IIVTV(NMMObject& parent) : ITV(parent) {  setTypeInfo(TypeInfo(IIVTV::IIVTV_type, "NMM")); setEventReceiver(parent); };

    virtual ~IIVTV() {};

    static const char* IIVTV_type;

    virtual  string getInterfaceType() const { return IIVTV_type; }

    Interface* copy() const { return new IIVTV(*this); }
    static const char* getCodecParams_event;
    static const char* setCodecParams_event;
    static const char* setGOPSize_event;
    static const char* setBFrames_event;
    static const char* setGOPClosure_event;
    static const char* setPulldown_event;
    static const char* setBitrate_event;
    static const char* setPeakBitrate_event;
    static const char* setStreamType_event;
    static const char* setBitrateMode_event;
    static const char* setDNRMode_event;
    static const char* setDNRType_event;
    static const char* setDNRSpatial_event;
    static const char* setDNRTemporal_event;
    static const char* setAspect_event;
    /** @name Codec.
     */
//@{
    /** This method returns the codec parameters.
     */

    virtual IVTVCodec getCodecParams() const /* limited(INode::INITIALIZED) */ /* limited(INode::ACTIVATED) */ /* limited(INode::STARTED) */ /* limited(INode::OUTPUT_INITIALIZED) */
      throw(const Exception&, const IllegalStateException&);
    /** This method sets the codec parameters.
     */

    virtual void setCodecParams(const IVTVCodec& codec) /* limited(INode::INITIALIZED) */ /* limited(INode::OUTPUT_INITIALIZED) */
      throw(const Exception&, const IllegalStateException&);
    /** This method sets the GOP size
     */

    virtual void setGOPSize(const unsigned int& value) /* limited(INode::INITIALIZED) */ /* limited(INode::OUTPUT_INITIALIZED) */
      throw(const Exception&, const IllegalStateException&);
    /** This method sets the number of B-Frames
     */

    virtual void setBFrames(const unsigned int& value) /* limited(INode::INITIALIZED) */ /* limited(INode::OUTPUT_INITIALIZED) */
      throw(const Exception&, const IllegalStateException&);
    /** This method sets the GOP closure mode of the codec (1=close GOP, 0=open GOP)
     */

    virtual void setGOPClosure(const unsigned int& value) /* limited(INode::INITIALIZED) */ /* limited(INode::OUTPUT_INITIALIZED) */
      throw(const Exception&, const IllegalStateException&);
    /** This method sets inverse telecine (1=ON, 0=OFF)
     */

    virtual void setPulldown(const unsigned int& value) /* limited(INode::INITIALIZED) */ /* limited(INode::OUTPUT_INITIALIZED) */
      throw(const Exception&, const IllegalStateException&);
    /** This method sets bitrate
     */

    virtual void setBitrate(const unsigned int& value) /* limited(INode::INITIALIZED) */ /* limited(INode::OUTPUT_INITIALIZED) */
      throw(const Exception&, const IllegalStateException&);
    /** This method sets peak bitrate (must be greater than bitrate)
     */

    virtual void setPeakBitrate(const unsigned int& value) /* limited(INode::INITIALIZED) */ /* limited(INode::OUTPUT_INITIALIZED) */
      throw(const Exception&, const IllegalStateException&);
    /** This method sets the Streamtype
     *   IVTV_STREAM_PS     = 0
     *   IVTV_STREAM_TS     = 1
     *   IVTV_STREAM_MPEG1  = 2
     *   IVTV_STREAM_PES_AV = 3
     *   IVTV_STREAM_PES_V  = 5
     *   IVTV_STREAM_PES_A  = 7
     *   IVTV_STREAM_DVD    = 10
    */

    virtual void setStreamType(const unsigned int& value) /* limited(INode::INITIALIZED) */ /* limited(INode::OUTPUT_INITIALIZED) */
      throw(const Exception&, const IllegalStateException&);
    /** This method sets bitrate mode
     */

    virtual void setBitrateMode(const unsigned int& value) /* limited(INode::INITIALIZED) */ /* limited(INode::OUTPUT_INITIALIZED) */
      throw(const Exception&, const IllegalStateException&);
    /** This method sets DNRMode
     */

    virtual void setDNRMode(const unsigned int& value) /* limited(INode::INITIALIZED) */ /* limited(INode::OUTPUT_INITIALIZED) */
      throw(const Exception&, const IllegalStateException&);
    /** This method sets DNRType
     */

    virtual void setDNRType(const unsigned int& value) /* limited(INode::INITIALIZED) */ /* limited(INode::OUTPUT_INITIALIZED) */
      throw(const Exception&, const IllegalStateException&);
    /** This method sets DNRSpatial
     */

    virtual void setDNRSpatial(const unsigned int& value) /* limited(INode::INITIALIZED) */ /* limited(INode::OUTPUT_INITIALIZED) */
      throw(const Exception&, const IllegalStateException&);
    /** This method sets DNRTemporal
     */

    virtual void setDNRTemporal(const unsigned int& value) /* limited(INode::INITIALIZED) */ /* limited(INode::OUTPUT_INITIALIZED) */
      throw(const Exception&, const IllegalStateException&);
    /** This method sets aspect
     */

    virtual void setAspect(const unsigned int& value) /* limited(INode::INITIALIZED) */ /* limited(INode::OUTPUT_INITIALIZED) */
      throw(const Exception&, const IllegalStateException&);

//@}

  };

  typedef auto_ptr<IIVTV> IIVTV_var;




}

#endif
