
#ifndef NMM_ITVImpl_HPP
#define NMM_ITVImpl_HPP

#include "nmm/NMMTypes.hpp"
#include "nmm/base/NMMObject.hpp"
#include "ITV.hpp"

#include "nmm/interfaces/base/INodeImpl.hpp"

namespace NMM
{
  /** Interface for a Video4Linux based TVCard.
   */

  class ITVImpl  : public virtual NMMObject { 
  public:
    ITVImpl() { addInterface(new ITV(*this)); registerEvents(); };

    virtual ~ITVImpl() {};

  protected:
    /** @name Audio Inputs
     */
//@{
    /** This method returns the number of available audio inputs.
     */

    virtual unsigned int getAudioCount() const /* limited(INode::INITIALIZED) */ /* limited(INode::ACTIVATED) */ /* limited(INode::STARTED) */ /* limited(INode::OUTPUT_INITIALIZED) */ = 0;
    /** This method returns the properties of the specified audio input.
     * @param index Index of audio input, ranges from 0 to getAudioCount()-1
     */

    virtual TVAudio getAudio(const unsigned int& index) const /* limited(INode::INITIALIZED) */ /* limited(INode::ACTIVATED) */ /* limited(INode::STARTED) */ /* limited(INode::OUTPUT_INITIALIZED) */ = 0;
    /** This method returns the list of available audio inputs.
     */

    virtual list<TVAudio >  getAudioList() const /* limited(INode::INITIALIZED) */ /* limited(INode::ACTIVATED) */ /* limited(INode::STARTED) */ /* limited(INode::OUTPUT_INITIALIZED) */ = 0;
    /** This method returns the index of the current audio input.
     */

    virtual unsigned int getCurrentAudio() const /* limited(INode::INITIALIZED) */ /* limited(INode::ACTIVATED) */ /* limited(INode::STARTED) */ /* limited(INode::OUTPUT_INITIALIZED) */ = 0;
    /** This method sets the index of the current audio input.
     * @param index Index of audio input, ranges from 0 to getAudioCount()-1
     */

    virtual void setCurrentAudio(const unsigned int& index) /* limited(INode::INITIALIZED) */ /* limited(INode::ACTIVATED) */ /* limited(INode::STARTED) */ /* limited(INode::OUTPUT_INITIALIZED) */ = 0;
//@}
    /** @name Inputs
     */
//@{
    /** This method returns the number of available inputs.
     */

    virtual unsigned int getInputCount() const /* limited(INode::INITIALIZED) */ /* limited(INode::ACTIVATED) */ /* limited(INode::STARTED) */ /* limited(INode::OUTPUT_INITIALIZED) */ = 0;
    /** This method returns the properties of the specified input.
     * @param num Input number, ranges from 0 to getInputCount()-1
     */

    virtual TVInput getInput(const unsigned int& num) const /* limited(INode::INITIALIZED) */ /* limited(INode::ACTIVATED) */ /* limited(INode::STARTED) */ /* limited(INode::OUTPUT_INITIALIZED) */ = 0;
    /** This method returns the list of available inputs.
     */

    virtual list<TVInput >  getInputList() const /* limited(INode::INITIALIZED) */ /* limited(INode::ACTIVATED) */ /* limited(INode::STARTED) */ /* limited(INode::OUTPUT_INITIALIZED) */ = 0;
    /** This method returns the current input.
     */

    virtual unsigned int getCurrentInput() const /* limited(INode::INITIALIZED) */ /* limited(INode::ACTIVATED) */ /* limited(INode::STARTED) */ /* limited(INode::OUTPUT_INITIALIZED) */ = 0;
    /** This method sets the current input.
     * @param num Input number, ranges from 0 to getInputCount()-1
     */

    virtual void setCurrentInput(const unsigned int& num) /* limited(INode::INITIALIZED) */ /* limited(INode::ACTIVATED) */ /* limited(INode::STARTED) */ /* limited(INode::OUTPUT_INITIALIZED) */ = 0;
//@}
    /** @name Standards
     */
//@{
    /** This method returns the identifier of the standard which matches the given name.
     * If no such standard is found, zero is returned.
     */

    virtual unsigned int getStandardByName(const string& name) const /* limited(INode::INITIALIZED) */ /* limited(INode::ACTIVATED) */ /* limited(INode::STARTED) */ /* limited(INode::OUTPUT_INITIALIZED) */ = 0;
    /** This method returns the properties of the specified standard.
     * @param num Standard number
     */

    virtual TVStandard getStandard(const unsigned int& num) const /* limited(INode::INITIALIZED) */ /* limited(INode::ACTIVATED) */ /* limited(INode::STARTED) */ /* limited(INode::OUTPUT_INITIALIZED) */ = 0;
    /** This method returns the list of supported standards
     */

    virtual list<TVStandard >  getStandardList() const /* limited(INode::INITIALIZED) */ /* limited(INode::ACTIVATED) */ /* limited(INode::STARTED) */ /* limited(INode::OUTPUT_INITIALIZED) */ = 0;
    /** This method returns the standard set for the current input.
     */

    virtual unsigned int getCurrentStandard() const /* limited(INode::INITIALIZED) */ /* limited(INode::ACTIVATED) */ /* limited(INode::STARTED) */ /* limited(INode::OUTPUT_INITIALIZED) */ = 0;
    /** This method sets the standard for the current input
     * @param num Standard number
     */

    virtual void setCurrentStandard(const unsigned int& standard) /* limited(INode::INITIALIZED) */ /* limited(INode::ACTIVATED) */ /* limited(INode::STARTED) */ /* limited(INode::OUTPUT_INITIALIZED) */ = 0;
    /** This method sets the standard for the current input by name.
     */

    virtual void setStandardByName(const string& name) /* limited(INode::INITIALIZED) */ /* limited(INode::ACTIVATED) */ /* limited(INode::STARTED) */ /* limited(INode::OUTPUT_INITIALIZED) */ = 0;
//@}
    /** @name Tuners.
     */
//@{
    /** This method returns the number of tuners.
     */

    virtual unsigned int getTunerCount() const /* limited(INode::INITIALIZED) */ /* limited(INode::ACTIVATED) */ /* limited(INode::STARTED) */ /* limited(INode::OUTPUT_INITIALIZED) */ = 0;
    /** This method returns information about a tuner.
     * @param index Index of tuner, ranges from 0 to getTunerCount()-1
     */

    virtual TVTuner getTuner(const unsigned int& index) const /* limited(INode::INITIALIZED) */ /* limited(INode::ACTIVATED) */ /* limited(INode::STARTED) */ /* limited(INode::OUTPUT_INITIALIZED) */ = 0;
    /** This method returns the list of tuners.
     */

    virtual list<TVTuner >  getTunerList() const /* limited(INode::INITIALIZED) */ /* limited(INode::ACTIVATED) */ /* limited(INode::STARTED) */ /* limited(INode::OUTPUT_INITIALIZED) */ = 0;
    /** This method returns the current frequency of a tuner.
     * @param tuner Number of the tuner
     * @return The current frequency in MHz
     */

    virtual float getFrequency(const unsigned int& tuner) const /* limited(INode::INITIALIZED) */ /* limited(INode::ACTIVATED) */ /* limited(INode::STARTED) */ /* limited(INode::OUTPUT_INITIALIZED) */ = 0;
    /** This method sets the frequency of a tuner.
     * @param tuner Number of the tuner
     * @param frequency The frequency to set in MHz
     */

    virtual void setFrequency(const unsigned int& tuner, const float& frequency) /* limited(INode::INITIALIZED) */ /* limited(INode::ACTIVATED) */ /* limited(INode::STARTED) */ /* limited(INode::OUTPUT_INITIALIZED) */ = 0;
//@}
    /** @name Controls.
     */
//@{
    /** This method gets the current brightness.
     */

    virtual unsigned int getBrightness() const /* limited(INode::INITIALIZED) */ /* limited(INode::ACTIVATED) */ /* limited(INode::STARTED) */ /* limited(INode::OUTPUT_INITIALIZED) */ = 0;
    /** This method gets the current contrast.
     */

    virtual unsigned int getContrast() const /* limited(INode::INITIALIZED) */ /* limited(INode::ACTIVATED) */ /* limited(INode::STARTED) */ /* limited(INode::OUTPUT_INITIALIZED) */ = 0;
    /** This method gets the current saturation.
     */

    virtual unsigned int getSaturation() const /* limited(INode::INITIALIZED) */ /* limited(INode::ACTIVATED) */ /* limited(INode::STARTED) */ /* limited(INode::OUTPUT_INITIALIZED) */ = 0;
    /** This method gets the current hue.
     */    

    virtual int getHue() const /* limited(INode::INITIALIZED) */ /* limited(INode::ACTIVATED) */ /* limited(INode::STARTED) */ /* limited(INode::OUTPUT_INITIALIZED) */ = 0;
    /** This method gets the current volume.
     */

    virtual unsigned int getVolume() const /* limited(INode::INITIALIZED) */ /* limited(INode::ACTIVATED) */ /* limited(INode::STARTED) */ /* limited(INode::OUTPUT_INITIALIZED) */ = 0;
    /** This method gets the mute status.
     */

    virtual bool getMute() const /* limited(INode::INITIALIZED) */ /* limited(INode::ACTIVATED) */ /* limited(INode::STARTED) */ /* limited(INode::OUTPUT_INITIALIZED) */ = 0;
    /** This method sets the brightness.
     * @param brightness Brightness (0 to 255)
     */

    virtual void setBrightness(const unsigned int& brightness) /* limited(INode::INITIALIZED) */ /* limited(INode::ACTIVATED) */ /* limited(INode::STARTED) */ /* limited(INode::OUTPUT_INITIALIZED) */ = 0;
    /** This method sets the contrast.
     * @param contrast Contrast (0 to 127)
     */

    virtual void setContrast(const unsigned int& contrast) /* limited(INode::INITIALIZED) */ /* limited(INode::ACTIVATED) */ /* limited(INode::STARTED) */ /* limited(INode::OUTPUT_INITIALIZED) */ = 0;
    /** This method sets the saturation.
     * @param saturation Saturation (0 to 127)
     */

    virtual void setSaturation(const unsigned int& saturation) /* limited(INode::INITIALIZED) */ /* limited(INode::ACTIVATED) */ /* limited(INode::STARTED) */ /* limited(INode::OUTPUT_INITIALIZED) */ = 0;
    /** This method sets the hue.
     * @param hue Hue offset (-128 to 127)
     */

    virtual void setHue(const int& hue) /* limited(INode::INITIALIZED) */ /* limited(INode::ACTIVATED) */ /* limited(INode::STARTED) */ /* limited(INode::OUTPUT_INITIALIZED) */ = 0;
    /** This method sets the volume.
     * @param volume Volume (0 to 65535)
     */

    virtual void setVolume(const unsigned int& volume) /* limited(INode::INITIALIZED) */ /* limited(INode::ACTIVATED) */ /* limited(INode::STARTED) */ /* limited(INode::OUTPUT_INITIALIZED) */ = 0;
    /** This method sets the mute state.
     */

    virtual void setMute(const bool& mute) /* limited(INode::INITIALIZED) */ /* limited(INode::ACTIVATED) */ /* limited(INode::STARTED) */ /* limited(INode::OUTPUT_INITIALIZED) */ = 0;
//@}
    /** @name Frequency tables
     */
//@{
    /** This method returns the available frequency tables.
     */

    virtual list<string >  getFrequencyTableList() const /* limited(INode::INITIALIZED) */ /* limited(INode::ACTIVATED) */ /* limited(INode::STARTED) */ /* limited(INode::OUTPUT_INITIALIZED) */ = 0;
    /** This method returns the current frequency table.
     */

    virtual string getFrequencyTable() const /* limited(INode::INITIALIZED) */ /* limited(INode::ACTIVATED) */ /* limited(INode::STARTED) */ /* limited(INode::OUTPUT_INITIALIZED) */ = 0;
    /** This method sets the current frequency table.
     * @param name Name of the frequency table
     */

    virtual void setFrequencyTable(const string& table) /* limited(INode::INITIALIZED) */ /* limited(INode::ACTIVATED) */ /* limited(INode::STARTED) */ /* limited(INode::OUTPUT_INITIALIZED) */ = 0;
    /** This method set input-/output-properties in the MSP-matrix
     * @param input input-value
     * @param output output-value
    */

    virtual void setInputOutput(const int& input, const int& output) /* limited(INode::ACTIVATED) */ /* limited(INode::STARTED) */ /* limited(INode::OUTPUT_INITIALIZED) */ = 0;

//@}

  private:
    void registerEvents();
  };

}

#endif
