/* NMM - Network-Integrated Multimedia Middleware
 *
 * Copyright (C) 2002-2005
 *                    NMM work group,
 *                    Computer Graphics Lab,
 *                    Saarland University, Germany
 *                    http://www.networkmultimedia.org
 *
 * Maintainer:         
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or (at your option) any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 * 
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307
 * USA
 */

#include <unistd.h>
#include <fcntl.h>
#include <iostream>
#include "IRControl.hpp"


IRControl::IRControl(string name /*= "nmmirc"*/) {
  prog_name = name;
  initialized = false;
}; // IRControl(string name = "nmmirc")

IRControl::~IRControl() {
  if (initialized) {
    lirc_freeconfig(lirc_config);
    lirc_deinit();
  }
}; // ~IRControl()


bool IRControl::init(string cfg_file /*= ""*/) {
  if (!initialized) {
    lirc_fd = lirc_init(const_cast<char*>(prog_name.c_str()),1);
    if (-1 == lirc_fd) {
      return false;
    }

    fcntl(lirc_fd,F_SETOWN,getpid());
    int flags=fcntl(lirc_fd,F_GETFL,0);
    if(flags!=-1) {
      fcntl(lirc_fd,F_SETFL,flags|O_NONBLOCK);
    }
  }
    
  if ("" != cfg_file) {
    config_file = cfg_file;
  } else {
    config_file = 
      string(getenv("HOME"))
      + string("/.nmm/")
      + string("nmmirc.lirc");
  }
  if (lirc_readconfig( const_cast<char*>(config_file.c_str()),&lirc_config,NULL) != 0) {
    return false;
  }
  
  initialized = true;
  return true;
} // init(string config = "")



IRControl::IRKey IRControl::getKey(int msec) {
  if (!initialized) {
    // Exception werfen
    cout << "Not initialized!" << endl;
    return NOKEY;
  }

  fd_set read_fds;
  struct timeval time;

  /* Watch lirc's file descriptor (lirc_fd) to see when it has input. */
  FD_ZERO(&read_fds);
  FD_SET(lirc_fd, &read_fds);
  
  /* Wait up to 'msec' microseconds. */
  time.tv_sec = msec / 1000;
  time.tv_usec = (msec % 1000) * 1000;
//    if(!select(1, &read_fds, NULL, NULL, &time)) {
//      cout << "Nothing changed!" << endl;
//      return NOKEY;
//    }

  /* Data is available now. */
  char* code;
  if (lirc_nextcode(&code)!=0) {
    // exception werfen
    cout << "Could not read next code!" << endl;
    return NOKEY;
  }

  if (NULL == code) {
    return NOKEY;
  }

  char* c;
  if (0 != lirc_code2char(lirc_config,code,&c)) {
    // exception werfen
    cout << "Could not translate code to char!" << endl;
    return NOKEY;
  }
  
  IRKey pressed_key = NOKEY;
  if (NULL != c) {
    // switch (c) ...
    if (!strcmp(c,"KEY0"))     pressed_key=KEY0;
    if (!strcmp(c,"KEY1"))     pressed_key=KEY1;
    if (!strcmp(c,"KEY2"))     pressed_key=KEY2;
    if (!strcmp(c,"KEY3"))     pressed_key=KEY3;
    if (!strcmp(c,"KEY4"))     pressed_key=KEY4;
    if (!strcmp(c,"KEY5"))     pressed_key=KEY5;
    if (!strcmp(c,"KEY6"))     pressed_key=KEY6;
    if (!strcmp(c,"KEY7"))     pressed_key=KEY7;
    if (!strcmp(c,"KEY8"))     pressed_key=KEY8;
    if (!strcmp(c,"KEY9"))     pressed_key=KEY9;
    if (!strcmp(c,"START"))    pressed_key=START;
    if (!strcmp(c,"STOP"))     pressed_key=STOP;
    if (!strcmp(c,"PAUSE"))    pressed_key=PAUSE;
    if (!strcmp(c,"NEXT"))     pressed_key=NEXT;
    if (!strcmp(c,"PREVIOUS")) pressed_key=PREVIOUS;
    if (!strcmp(c,"FORWARD"))  pressed_key=FORWARD;
    if (!strcmp(c,"REWIND"))   pressed_key=REWIND;
    if (!strcmp(c,"POWER"))    pressed_key=POWER;
    if (NOKEY == pressed_key) {
      // unconfigured key
    }
  } else {
    return NOKEY;
  }

  free(code);

  return pressed_key;
} // getKey(int msec = 1000)
