/* NMM - Network-Integrated Multimedia Middleware
 *
 * Copyright (C) 2002-2005
 *                    NMM work group,
 *                    Computer Graphics Lab,
 *                    Saarland University, Germany
 *                    http://www.networkmultimedia.org
 *
 * Maintainer:        Christoph Wellner <chwellner@graphics.cs.uni-sb.de>
 *                     
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version 2
 * of the License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307
 * USA
 */

#include "TVGuidePlugin.hpp"

#include "nmm/misc/tvguide/TVGuidePluginManager.hpp"

namespace NMM{

  TVGuidePlugin::TVGuidePlugin(const string& name) :
    enabled(true),
    tvgobject(NULL)
  {
    pluginName = name;
    state = ITVGuidePlugin::CONSTRUCTED;
    STVGuidePluginManager::getInstance().registerPlugin(this);
  }

  TVGuidePlugin::~TVGuidePlugin(){
    STVGuidePluginManager::getInstance().unregisterPlugin(this);

  }

  Result TVGuidePlugin::init() throw(const StateTransitionFailedException& ){
    if(state != ITVGuidePlugin::CONSTRUCTED){
      throw StateTransitionFailedException();
    }else{
      if(doInit() == SUCCESS){
        state = ITVGuidePlugin::INITIALIZED;
        return SUCCESS;
      }else{
        throw StateTransitionFailedException();
      }
    }
  }

  Result TVGuidePlugin::deinit() throw(const StateTransitionFailedException& ){
    if(state != ITVGuidePlugin::INITIALIZED){
      throw StateTransitionFailedException();
    }else{
      doDeinit();
      state = ITVGuidePlugin::CONSTRUCTED;
      return SUCCESS;
    }
  }

  Result TVGuidePlugin::start() throw(const StateTransitionFailedException& ){
    if(state != ITVGuidePlugin::INITIALIZED){
      throw StateTransitionFailedException();
    }else{
      state = ITVGuidePlugin::STARTED;
      if(enabled){
        activateThread();
        
        return SUCCESS;
      }else{
        throw StateTransitionFailedException();
      }
    }
  }

  Result TVGuidePlugin::stop() throw(const StateTransitionFailedException& ){
    if(state != ITVGuidePlugin::STARTED){
      throw StateTransitionFailedException();
    }else{
      doStop();
      deactivateThread();
      state = ITVGuidePlugin::INITIALIZED;
      return SUCCESS;
    }
  }
  void TVGuidePlugin::disable(){
    enabled = false;
  }

  void TVGuidePlugin::enable(){
    enabled = true;
  }

  void TVGuidePlugin::run(){
    doStart();
  }

}
