/* NMM - Network-Integrated Multimedia Middleware
 *
 * Copyright (C) 2002-2005
 *                    NMM work group,
 *                    Computer Graphics Lab,
 *                    Saarland University, Germany
 *                    http://www.networkmultimedia.org
 *
 * Maintainer:        Christoph Wellner <chwellner@graphics.cs.uni-sb.de>
 *                     
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version 2
 * of the License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307
 * USA
 */

#ifndef NMM_TVGUIDEPLUGIN_HPP
#define NMM_TVGUIDEPLUGIN_HPP NMM_TVGUIDEPLUGIN_HPP

#include "nmm/interfaces/misc/tvguide/ITVGuidePluginImpl.hpp"
#include "nmm/interfaces/misc/tvguide/ITVGuideProducerImpl.hpp"

#include "nmm/utils/thread/ThreadedObject.hpp"

namespace NMM{

  class TVGuidePlugin : public ITVGuidePluginImpl,
                        public ITVGuideProducerImpl,
                        public ThreadedObject
  {

  private:
    string pluginName;
    bool enabled;

    ITVGuidePlugin::State state;
  public:
    /**
     * Pointer to TVGuide-Object to store the data collected
     */
    ITVGuide_var tvgobject;

    /** Constructor/Destructor **/
    TVGuidePlugin(const string& name);
    ~TVGuidePlugin();

    /**
     * Init-Method to initialize the Plugin
     */
    Result init() throw(const StateTransitionFailedException& );
    /**
     * start-Method to start the Plugin
     */
    Result start() throw(const StateTransitionFailedException& );
    /**
     * stop-Method to stop the Plugin
     */
    Result stop() throw(const StateTransitionFailedException& );
    /**
     * deinit-Method to deinitialize the Plugin
     */
    Result deinit() throw(const StateTransitionFailedException& );

    /**
     * Sets the TVGuide-Object to store the collected data
     *@param itvg ITVGuide-Interface of the TVGuide-Object
     */
    void registerTVGuideServer(const ITVGuide* itvg){tvgobject.reset(const_cast<ITVGuide *>(itvg));}

    /*
     * disbles the Plugin
     */
    void disable();    
    /**
     *enables the Plugin. Plugins are enabled by default
     */
    void enable();
    /**
     * tells if the Plugin is enabled or not
     *@param bool enabled-status of plugin
     */
    bool getEnabled(){return enabled;}

    /**
     * returns the name of the Plugin
     *@return string name of plugin
     */
    string getName(){return pluginName;}

    /**
     * Implementation of run() from threadedObject
     */
    void run();

    /**
     * queries the State of the plugin
     *@return state state of Plugin
     */
    ITVGuidePlugin::State getState(){return state; }

    virtual Result doStart() =0;
    virtual Result doInit() =0;
    virtual Result doStop() =0;
    virtual Result doDeinit() =0;



  };

}


#endif
