/* NMM - Network-Integrated Multimedia Middleware
 *
 * Copyright (C) 2002-2005
 *                    NMM work group,
 *                    Computer Graphics Lab,
 *                    Saarland University, Germany
 *                    http://www.networkmultimedia.org
 *
 * Maintainer:        chwellner <chwellner@graphics.cs.uni-sb.de>
 *                     
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version 2
 * of the License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307
 * USA
 */

#ifndef NMM_DVBEPGPLUGIN_HPP
#define NMM_DVBEPGPLUGIN_HPP NMM_DVBEPGPLUGIN_HPP

#include "nmm/interfaces/misc/tvguide/IDVBEPGPluginImpl.hpp"
#include "nmm/misc/tvguide/TVGuidePlugin.hpp"

#include "nmm/misc/tvguide/TVGuide.hpp"
#include "nmm/utils/thread/ThreadedObject.hpp"
#include "nmm/misc/timer/TimerEvents.hpp"

#include "nmm/interfaces/video/tv/ITVCard.hpp"

#include "nmm/base/ProxyApplication.hpp"

#include "nmm/interfaces/base/IUsageNotifyImpl.hpp"

#include "nmm/interfaces/misc/tvmanager/ITVManager.hpp"

namespace NMM{
  
  class DVBEPGPlugin: public TVGuidePlugin,
                      public IDVBEPGPluginImpl,
                      public IUsageNotifyImpl
  {
    
  private:
    
    ITVManager *tvmanager;

    bool termUpdate;
    int switchPeriod;
    string configFile;

    TimerProducer* timer;

    TEDObject0<DVBEPGPlugin> switch_timer;

  public:
    DVBEPGPlugin();
    ~DVBEPGPlugin();

  protected:

    /** Start aquiring EPG-Data from DVB */
    Result doStart();
    
    /** Stop aquiring EPG-Data from DVB */
    Result doStop();

    Result doInit();
    Result doDeinit();

    void setConfigFile(const string &f){configFile = f;}
    string getConfigFile(){return configFile;}

    Result switchChannel();

    void setTVManager(const ITVManager *tvm){tvmanager = dynamic_cast<ITVManager*>(tvm->copy());}

    //implement IUsageNofifyImpl
    Result becomeMaster(){return SUCCESS;}
    Result becomeSlave(){return SUCCESS;}
  };

}

#endif
