/* NMM - Network-Integrated Multimedia Middleware
 *
 * Copyright (C) 2002-2005
 *                    NMM work group,
 *                    Computer Graphics Lab,
 *                    Saarland University, Germany
 *                    http://www.networkmultimedia.org
 *
 * Maintainer:         
 *                     
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version 2
 * of the License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307,
 * USA.
 */

/*
 * scale and output functions are taken from minimad.c example of libmad package that is
 * copyrighted 2000-2001 by Robert Leslie
 */


#ifndef NMM_MPEGAUDIODECODENODE_HPP
#define NMM_MPEGAUDIODECODENODE_HPP NMM_MPEGAUDIODECODENODE_HPP

#include "nmm/base/GenericProcessorNode.hpp"
#include "nmm/interfaces/base/sync/ISyncResetImpl.hpp"
#include "nmm/interfaces/file/ITrackImpl.hpp"
#include "nmm/interfaces/file/ISeekReceiverImpl.hpp"
#include "nmm/interfaces/general/ITrackDurationImpl.hpp"

extern "C" {
  #include <mad.h>
}


namespace NMM {
  class MPEGAudioDecodeNode :
    public GenericProcessorNode,
    public ISyncResetImpl,
    public ITrackImpl,
    public ITrackDurationImpl,
    public ISeekRecieverImpl
  {
    
  private:
    Timestamp timestamp;  //! The timestamp for the next output buffer.
    long int my_delay;    //! The duration of the next output buffer at audio sink.
    Timestamp in_buf_ts;  //! The timestamp of the last in_buffer with valid timestamp.

    unsigned int sample_rate;
    int channels;
    unsigned long bitrate;
    Time seek_time;

    ThreadMutex mad_mutex;
    
    bool first_timestamp;

    void resetMP3(); //reset the library-Functions

    void setWOPInternal();
    
  protected:
    /**
    * >begin: Structures for MAD mpeg audio decoder
    */
    struct mad_stream stream;
    struct mad_frame  frame;
    struct mad_synth  synth;
    /* > end */

    /* size of our extra buffer space */
    int             fragment_size;

    /* bytes that could not decoded because data was needed (stream end) */
    int             rest_length;

    /* current length of incoming buffer */
    int             cur_buffer_length;

    /* pointer to end of audio stream data */
    unsigned char*  cur_buffer_end;

    /* pointer to begin of audio stream data */
    unsigned char*  tmpbuffer;

    /* size of tmpbuffer */
    int             inbuffer_size;

    /* track duration in seconds */
    u_int32_t track_duration;

    /* indicates when a trackDuration event should be sent */
    bool send_track_duration;

    virtual Result doInit();
    virtual Result doDeinit();
    virtual Result doInitOutput();
    virtual Message *processBuffer( Buffer *in_buffer);

    /**
     * reset workingflag and some variables
     */
    virtual Result doFlush();

    /**
     * Resets the parameters of timestamp computation.
     * The next outgoing buffer will have a stream_counter of 0.
     * If no input timestamp is present, it will also have a 
     * sync_time of 0.
     * MPEGAudioDecodeNode puts a timestamp in every outgoing buffer,
     * so the is_valid-flag in every timestamp is initialized to true.
     */
    virtual Result syncReset();
    
    virtual Result startTrack(const string &file_name);
    virtual Result endTrack();
    virtual Result finished();

    //ISeekReviecerInterface
    virtual Result seeked(const u_int64_t& filepos , const int64_t& offset);
    
    virtual Result doSetFormat(const Format& format);
    
    //! new seek method
    Result eventSeek(Interval& i, Time& t, long& num_bytes);

    //from ITrackSizeImpl
    Result trackDuration(const Interval& i);
    
   public:
    MPEGAudioDecodeNode( const char * = "MPEGAudioDecodeNode", StreamQueue::Mode = StreamQueue::MODE_SUSPEND, int = 20);
    virtual ~MPEGAudioDecodeNode();

  };

}

#endif //NMM_MPEGAUDIODECODENODE_HPP

