/* NMM - Network-Integrated Multimedia Middleware
 *
 * Copyright (C) 2002-2005
 *                    NMM work group,
 *                    Computer Graphics Lab,
 *                    Saarland University, Germany
 *                    http://www.networkmultimedia.org
 *
 * Maintainer:        David Maass <dama@graphics.cs.uni-sb.de>
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or (at your option) any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 * 
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307
 * USA
 */

#ifndef NMM_VORBISENCODENODE_HPP
#define NMM_VORBISENCODENODE_HPP

#include "nmm/base/GenericProcessorNode.hpp"
#include "nmm/interfaces/file/ITrackImpl.hpp"
#include "nmm/interfaces/audio/ITagInfoImpl.hpp"

/**
* including Vorbisheader
*/
extern "C" {
  #include <vorbis/vorbisenc.h>
}

namespace NMM {
  
  class VorbisEncodeNode : public GenericProcessorNode,
			   public ITrackImpl,
                           public ITagInfoImpl {

  public:

    VorbisEncodeNode(const char* name = "VorbisEncodeNode",
			  const StreamQueue::Mode queue_mode = StreamQueue::MODE_SUSPEND,
			  const int queue_size = StreamQueue::DEFAULT_SIZE);

    virtual ~VorbisEncodeNode();

  protected:
    enum EVENTSTATE { START, DATA, END, FINISHED, TEST };
    EVENTSTATE event;
    virtual Message* processBuffer(Buffer *in_buffer);
    virtual Result doInit();
    virtual Result doInitOutput();
    virtual Result doActivate();
    virtual Result doDeactivate();

    virtual Result startTrack(const string& filename);
    virtual Result endTrack();
    virtual Result finished();

    virtual Result TagInfo(const string& title, const string& album, const string& artist, const string& year, const string& genre);

  private:
    Result setup();
    int sample_rate,bitpersample,channels;	
    float quality;
    bool setupdone;
    Format* myInputFormat;
    Format* myOutputFormat;
    bool HeaderWritten;
    Buffer* rescue_buffer;
    /* variables needed by the vorbis encoder */
    ogg_stream_state OggStreamState;
    ogg_page OggPage;
    ogg_packet OggPacket;
    vorbis_info VorbisInfo;
    vorbis_comment VorbisComment;
    vorbis_dsp_state VorbisDspState;
    vorbis_block VorbisBlock;
    string trackname;
    string album;
    string year;
    string artist;
    string genre;
    string title;
    
  }; /* class VorbisEncodeNode */
}; /* namespace NMM */
#endif
