/* NMM - Network-Integrated Multimedia Middleware
 *
 * Copyright (C) 2002-2005
 *                    NMM work group,
 *                    Computer Graphics Lab,
 *                    Saarland University, Germany
 *                    http://www.networkmultimedia.org
 *
 * Maintainer:        Christian Gerstner <gerstner@graphics.cs.uni-sb.de>
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or (at your option) any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307
 * USA
 */

#include "g711.h"
#include "PCMDecodeNode.hpp"
#include <iostream>

#include "nmm/base/format/Property.hpp"
#include "nmm/base/registry/Plugin.hpp"

NMM_REGISTER_PLUGIN(NMM::PCMDecodeNode, "PCMDecodeNode")

namespace NMM {

  PCMDecodeNode::PCMDecodeNode( const char * name, StreamQueue::Mode qmode, int qsize)
    : GenericProcessorNode(name, qmode, qsize)
  {    
    codec = PCMU;
    
  };
  

  PCMDecodeNode::~PCMDecodeNode()
  {
    
  }
  
  Result PCMDecodeNode::doInit() {
    // input format -> pcmu samples
    Format *in_format = getOwnInputProperty()->addNewFormat(Format::audio_raw_type);
    in_format->addIntParamValue(Format::sample_rate_param, 8000);
    in_format->addIntParamValue(Format::channels_param, 1);
    in_format->addIntParamValue(Format::bitpersample_param,  8);
    in_format->addStringParamValue(Format::endian_param, "little");
    in_format->addStringParamValue(Format::encoding_param, "mu_law");
    in_format->addStringParamValue(Format::sign_param, "signed");
    getOwnInputProperty()->setDefaultFormat(in_format);

    Format *in_format2 = getOwnInputProperty()->addNewFormat(Format::audio_raw_type);
    in_format2->addIntParamValue(Format::sample_rate_param, 44100);
    in_format2->addIntParamValue(Format::channels_param, 2);
    in_format2->addIntParamValue(Format::bitpersample_param,  8);
    in_format2->addStringParamValue(Format::endian_param, "little");
    in_format2->addStringParamValue(Format::encoding_param, "mu_law");
    in_format2->addStringParamValue(Format::sign_param, "signed");

    Format *in_format3 = getOwnInputProperty()->addNewFormat(Format::audio_raw_type);
    in_format3->addIntParamValue(Format::sample_rate_param, 48000);
    in_format3->addIntParamValue(Format::channels_param, 2);
    in_format3->addIntParamValue(Format::bitpersample_param,  8);
    in_format3->addStringParamValue(Format::endian_param, "little");
    in_format3->addStringParamValue(Format::encoding_param, "mu_law");
    in_format3->addStringParamValue(Format::sign_param, "signed");

    // input format -> pcma samples
    Format *in_format4 = getOwnInputProperty()->addNewFormat(Format::audio_raw_type);
    in_format4->addIntParamValue(Format::sample_rate_param, 8000);
    in_format4->addIntParamValue(Format::channels_param, 1);
    in_format4->addIntParamValue(Format::bitpersample_param,  8);
    in_format4->addStringParamValue(Format::endian_param, "little");
    in_format4->addStringParamValue(Format::encoding_param, "a_law");
    in_format4->addStringParamValue(Format::sign_param, "signed");
    
    Format *in_format5 = getOwnInputProperty()->addNewFormat(Format::audio_raw_type);
    in_format5->addIntParamValue(Format::sample_rate_param, 44100);
    in_format5->addIntParamValue(Format::channels_param, 2);
    in_format5->addIntParamValue(Format::bitpersample_param,  8);
    in_format5->addStringParamValue(Format::endian_param, "little");
    in_format5->addStringParamValue(Format::encoding_param, "a_law");
    in_format5->addStringParamValue(Format::sign_param, "signed");

    Format *in_format6 = getOwnInputProperty()->addNewFormat(Format::audio_raw_type);
    in_format6->addIntParamValue(Format::sample_rate_param, 48000);
    in_format6->addIntParamValue(Format::channels_param, 2);
    in_format6->addIntParamValue(Format::bitpersample_param,  8);
    in_format6->addStringParamValue(Format::endian_param, "little");
    in_format6->addStringParamValue(Format::encoding_param, "a_law");
    in_format6->addStringParamValue(Format::sign_param, "signed");

    // output format -> audio raw type
    Format *out_format = getOwnOutputProperty()->addNewFormat(Format::audio_raw_type);
    out_format->addIntParamValue(Format::sample_rate_param,44100);
    out_format->addIntParamValue(Format::channels_param, 2);
    out_format->addIntParamValue(Format::bitpersample_param, 16);
    out_format->addStringParamValue(Format::endian_param, "little");
    out_format->addStringParamValue(Format::encoding_param, "linear");
    out_format->addStringParamValue(Format::sign_param, "signed");

    Format *out_format2 = getOwnOutputProperty()->addNewFormat(Format::audio_raw_type);
    out_format2->addIntParamValue(Format::sample_rate_param,8000);
    out_format2->addIntParamValue(Format::channels_param, 1);
    out_format2->addIntParamValue(Format::bitpersample_param, 16);
    out_format2->addStringParamValue(Format::endian_param, "little");
    out_format2->addStringParamValue(Format::encoding_param, "linear");
    out_format2->addStringParamValue(Format::sign_param, "signed");
    getOwnOutputProperty()->setDefaultFormat(out_format2);
    
    Format *out_format3 = getOwnOutputProperty()->addNewFormat(Format::audio_raw_type);
    out_format3->addIntParamValue(Format::sample_rate_param,48000);
    out_format3->addIntParamValue(Format::channels_param, 2);
    out_format3->addIntParamValue(Format::bitpersample_param, 16);
    out_format3->addStringParamValue(Format::endian_param, "little");
    out_format3->addStringParamValue(Format::encoding_param, "linear");
    out_format3->addStringParamValue(Format::sign_param, "signed");
    

return SUCCESS;
  }
  
  Result PCMDecodeNode::doDeinit() {
    
    return SUCCESS;
  }
  
  Result PCMDecodeNode::doInitOutput() {  
    // get input format.
    // verify that it is not NULL. the node's input must be connected now, or it is a failure.
    const Format* in_format = getInputFormat();
    if (in_format == NULL) {
      ERROR_STREAM("No input format" << endl);
      return FAILURE;
    }

    // init working input property
    Format* i_format = new Format( *in_format );
    getOwnWorkingInputProperty()->addNewFormat(i_format);
    getOwnWorkingInputProperty()->setDefaultFormat(i_format);

    int channels = i_format->getIntValue(Format::channels_param);
    int sample_rate = i_format->getIntValue(Format::sample_rate_param);
    string codec_string = i_format->getStringValue(Format::encoding_param);
      
      if(codec_string == "mu_law"){
        cout << "Using PCMUUUUUUU Decoding!" << endl;
        codec = PCMU;
      }else if(codec_string == "a_law"){
        cout << "Using PCMAAAAAAA Decoding!" << endl;
        codec = PCMA;
      }else{
        ERROR_STREAM("Unknown encoding_param " << codec_string << " in input format!" << endl);
      }

    // output format -> audio raw type
    Format *out_format = getOwnWorkingOutputProperty()->addNewFormat(Format::audio_raw_type);
    out_format->setFormatType("audio/raw");
    out_format->addIntParamValue(Format::sample_rate_param, sample_rate);
    out_format->addIntParamValue(Format::channels_param, channels);
    out_format->addIntParamValue(Format::bitpersample_param, 16);
    out_format->addStringParamValue(Format::endian_param, "little");
    out_format->addStringParamValue(Format::encoding_param, "linear");
    out_format->addStringParamValue(Format::sign_param, "signed");
    getOwnWorkingOutputProperty()->setDefaultFormat(out_format);
    
    int bit_per_sample = out_format->getIntValue(Format::bitpersample_param);

    DEBUG_STREAM(endl << endl <<
                 "PCMDecodeNode WorkingOutputProperty: " << endl <<
                 "sample_rate = " << sample_rate << endl <<
                 "channels = " << channels << endl <<
                 "bit per sample = " << bit_per_sample << endl <<
                 endl << endl);
    
    i_format->setIOPartner(out_format);
    out_format->setIOPartner(i_format);


    return SUCCESS;
  }
  

  Message *PCMDecodeNode::processBuffer( Buffer *in_buffer ) {
    
    Buffer* out;
    int in_buffer_size = in_buffer->getUsed();
    unsigned char* samples = NULL;
    short* dst = NULL;
    
    out = getNewBuffer(2*in_buffer_size, in_buffer);
    samples = reinterpret_cast<unsigned char*>(in_buffer->getData());
    dst = reinterpret_cast<short*>(out->getData());
    
    switch(codec){
    case PCMU:
      for(int i=0;i<in_buffer_size;){
        unsigned char value = *samples;
        short sample = ulaw2linear(value);
        dst[0]=sample;
        dst++;
        samples++;
        i++;
      }
      break;
    case PCMA:
      for(int i=0;i<in_buffer_size;){
        unsigned char value = *samples;
        short sample = alaw2linear(value);
        dst[0]=sample;
        dst++;
        samples++;
        i++;
      }
      break;
    default:
      ERROR_STREAM("Unknown used codec!");
    }

    in_buffer->release();
    out->setUsed(in_buffer_size*2);
    
    return out;
  }

}
