/* NMM - Network-Integrated Multimedia Middleware
 *
 * Copyright (C) 2002-2005
 *                    NMM work group,
 *                    Computer Graphics Lab,
 *                    Saarland University, Germany
 *                    http://www.networkmultimedia.org
 *
 * Maintainer:        
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or (at your option) any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 * 
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307
 * USA
 */


#ifndef _NMM_AVCOLLECTOR_HPP_
#define _NMM_AVCOLLECTOR_HPP_ _NMM_AVCOLLECTOR_HPP_

#include "nmm/interfaces/av/IAVMuxImpl.hpp"
#include "nmm/interfaces/file/ITrackImpl.hpp"
#include "nmm/base/GenericMultiplexerNode.hpp"

namespace NMM {

  /**
   * Audio/Video multiplexer.
   * This nodes puts events and buffers from two streams called "audio" and "video" together to
   * a single stream. Events and buffers receive labels, so that successor nodes can tell from
   * which input stream they came.
   */
  class AVMuxNode :
    public GenericMultiplexerNode,
    public ITrackImpl,
    public IAVMuxImpl
  {
  private:
    ThreadMutex mutex;

    int buf_count;
    int track_start_count;
    bool send_event;
    Buffer* last_buffer;
    
    bool audio_end_track;
    bool video_end_track;

    /** various members for discard mode */
    bool first_audio_buffer;
    bool first_video_buffer;
    bool audio_too_late;
    bool set_audio_too_late;
    Time first_audio_buffer_sync_time;
    Time first_video_buffer_sync_time;
    bool discard_mode;
    bool discard_mode_running;

    /** synchronized mode. synchronize audio and video */
    bool sync_mode;
    
    /** send all buffers remembered for synchronization */
    bool sync_flush;
    
    /** last video frame */
    Buffer* last_audio;
    
    /** last audio frame */
    Buffer* last_video;
    
    /** send an endTrack_event after sending remembered buffers */
    bool sync_end_track;
    
  protected:
    virtual Result doInit();
    virtual Result doInitOutput();
    
    virtual Result startTrack(const string& filename);
    virtual Result endTrack();
    virtual Result finished();
    
    /* IAVMux interface */
    virtual void setSyncMode(const bool& value);
    
    Message* processBuffer(Buffer*);
    
    Result processCEvent(CEvent*);
    
  public:

    AVMuxNode(const char *name = "AVMuxNode", StreamQueue::Mode = StreamQueue::MODE_SUSPEND, int = 20);
    ~AVMuxNode();
  };

}

#endif

