/* NMM - Network-Integrated Multimedia Middleware
 *
 * Copyright (C) 2002-2005
 *                    NMM work group,
 *                    Computer Graphics Lab,
 *                    Saarland University, Germany
 *                    http://www.networkmultimedia.org
 *
 * Maintainer:        NMM work group,
 *                    http://www.networkmultimedia.org
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or (at your option) any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 * 
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307
 * USA
 */


#ifndef __NMM_GENERICWRITENODE_HPP__
#define __NMM_GENERICWRITENODE_HPP__ __NMM_GENERICWRITENODE_HPP__

#include <fstream>
#include <string>

#include "nmm/utils/thread/Thread.hpp"
#include "nmm/base/GenericSinkNode.hpp"
#include "nmm/interfaces/file/ISinkFileHandlerImpl.hpp"
#include "nmm/interfaces/file/ITrackImpl.hpp"

namespace NMM {
  
  class GenericWriteNode :
    public GenericSinkNode,
    public ISinkFileHandlerImpl,
    public ITrackImpl
  {
  public: 
    GenericWriteNode(const char *name = "GenericWriteNode", 
                     const StreamQueue::Mode queue_mode = StreamQueue::MODE_SUSPEND, 
                     const int queue_size = 1);
    ~GenericWriteNode();
    
  protected:
    // GenericNode methods
    virtual Result doInit();
    virtual Result doInitOutput();
    virtual Result doActivate();
    virtual Result doDeactivate();
    virtual Message* processBuffer(Buffer *in_buffer);
    
    // ISinkFileHandler interface
    virtual void setWorkingDirectory(const string& directory);
    virtual string getWorkingDirectory();
    virtual void setFileExtension(const string& extension);
    virtual string getFileExtension();
    
    // IFileHandler interface
    virtual void setFilename(const string& filename);
    
    // ITrack interface
    virtual Result startTrack(const string& filename);
    virtual Result endTrack();
    virtual Result finished();

  private:
    /** This method tries to open the file if a filename has been set.
     */
    Result openFile();
    
    /** This method tries to close the file
     */
    Result closeFile();
    
    void makeDefaultFilename(const string& filename);
    
  private:
    enum SubState {
      CLOSED   = 1, ///< File is not opened. This is the initial state.
      STARTED  = 2, ///< File is not opened and a startTrack event has already been received.
      OPENED   = 4, ///< File is opened and data can be written to it.
    };
    
    string   __file_extension;               ///< Default output file extension
    string   __working_directory;            ///< Default output file directory
    bool     __override_default_filename;    ///< Override default filename
    SubState __sub_state;                    ///< Current sub state of node
    string   __filename;                     ///< Filename
    ofstream __out_fstream;                  ///< Stream to write to
    
    long int __bytes_written;                ///< Bytes written to stream
    int      __buffnum;                      ///< Buffers written to stream

    ThreadMutex __substate_mutex;            ///< Sub state mutex
  };
  
}

#endif  // __NMM_GENERICWRITENODE_HPP__
