/* NMM - Network-Integrated Multimedia Middleware
 *
 * Copyright (C) 2002-2005
 *                    NMM work group,
 *                    Computer Graphics Lab,
 *                    Saarland University, Germany
 *                    http://www.networkmultimedia.org
 *
 * Maintainer:        
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or (at your option) any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 * 
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307
 * USA
 */


#ifndef NMM_AVIWRITENODE_HPP
#define NMM_AVIWRITENODE_HPP NMM_AVIWRITENODE_HPP

#include <stdio.h>

#include "nmm/base/GenericSinkNode.hpp"
#include "nmm/interfaces/file/ISinkFileHandlerImpl.hpp"
#include "nmm/interfaces/av/ISetBufferLabelImpl.hpp"
#include "nmm/interfaces/video/IKeyframeImpl.hpp"
#include "nmm/interfaces/video/IFrameCountImpl.hpp"

// do forward declaration of needed structs from libmpdemux/muxer.h
struct muxer_t;
struct muxer_stream_t;

namespace NMM {

  class AVIWriteNode :
    public GenericSinkNode,
    public ISinkFileHandlerImpl,
    public ISetBufferLabelImpl,
    public ITrackImpl,
    public IKeyframeImpl,
    public IFrameCountImpl
  {

  private:
    muxer_t* avimux;
    muxer_stream_t* mux_a;
    muxer_stream_t* mux_v;
    int out_file_format;
    FILE* avifile;
    long int bytes_written;
    long int frames_written;
    string filename;
    string __file_extension;
    string __working_directory;
    bool closed;
    bool is_key_frame;
    unsigned int labelnum_video;
    unsigned int labelnum_audio;
    ThreadMutex avw_mutex;
    
    bool first_audio;
    bool first_video;
    Timestamp first_audio_ts;
    Timestamp first_video_ts;
    
  protected:
    // Node interface
    virtual Result doInit();
    virtual Result doDeinit();
    virtual Result doActivate();
    virtual Result doDeactivate();
    
    // ITrack interface
    virtual Result startTrack(const string& filename);
    virtual Result endTrack();
    virtual Result finished();
    
    // IKeyframe interface
    virtual Result setKeyframe();
    
    // ISetBufferLabel interface
    virtual Result setBufferLabel(const string& stream, const int& label);
    
    // ISinkFileHandler interface
    virtual void setWorkingDirectory(const string& directory);
    virtual string getWorkingDirectory() ;
    virtual void setFileExtension(const string& extension) ;
    virtual string getFileExtension();
    
    // IFileHandler interface
    virtual void setFilename(const string& filename);

    // IFrameCount interface
    virtual int getFrameCount() const;

    virtual Message* processBuffer(Buffer *in_buffer); 
    
  public: 
    AVIWriteNode(const char *name = "AVIWriteNode", 
		 const StreamQueue::Mode queue_mode = StreamQueue::MODE_SUSPEND, 
		 const int queue_size = 1);
    ~AVIWriteNode();
  };
  
} // class AVIWriteNode

#endif // NMM_AVIWRITENODE_HPP
