/* NMM - Network-Integrated Multimedia Middleware
 *
 * Copyright (C) 2002-2005
 *                    NMM work group,
 *                    Computer Graphics Lab,
 *                    Saarland University, Germany
 *                    http://www.networkmultimedia.org
 *
 * Maintainer:        
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or (at your option) any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 * 
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307
 * USA
 */


#include <fcntl.h>
#include "KfirReadNode.hpp"

#include "nmm/base/registry/Plugin.hpp"
#include "nmm/utils/SecurityCheck.hpp"

/*********************************************/
/* ReadNode for KFir MPEG2 Encoder Boards    */
/*********************************************/

NMM_REGISTER_PLUGIN(NMM::KfirReadNode, "KfirReadNode")

namespace NMM {

  KfirReadNode::KfirReadNode(const char *name /* = "KfirReadNode" */,
			     StreamQueue::Mode mode /* = StreamQueue::MODE_SUSPEND */,
			     int queue_size /* = StreamQueue::DEFAULT_SIZE */)
    : GenericSourceNode(name, mode, queue_size), devicename(""), fd(-1)
  {

  }
	
  KfirReadNode::~KfirReadNode()
  {
    close (fd);
  }
  
  void KfirReadNode::setInputDevice(const char *devicename_)
  {
    devicename = devicename_;
  }
  
  
  Result KfirReadNode::doInit()
  {
    if(!SSecurityCheck::getInstance().checkFilenameForReading(devicename)) {
      ERROR_STREAM("Security check not passed for file " << devicename << ", check .nmmrc for options" << endl);
      return FAILURE;
    }
    GLOBAL_DEBUG_STREAM( "KfirReadNode::doInit(): open " << devicename << endl);
    fd = open(devicename.c_str(), O_RDONLY);
    if (fd < 0)
    {
      return FAILURE;
      close(fd);
    }
    return SUCCESS;
  }
  
  Buffer *KfirReadNode::processBuffer(Buffer *) {
    Buffer *outBuf;
    int bytesReadOk;
        
    if ((outBuf = getNewBuffer( BUFFER_SIZE )) != 0)
    {
      bytesReadOk = read( fd, outBuf->getData(), BUFFER_SIZE );
      if ( bytesReadOk > 0 )
      {
        outBuf->setUsed( bytesReadOk );
	return(outBuf);
      }
      else setProducingFlag(false);
    }
    return 0;
  }
}
