/* NMM - Network-Integrated Multimedia Middleware
 *
 * Copyright (C) 2002-2005
 *                    NMM work group,
 *                    Computer Graphics Lab,
 *                    Saarland University, Germany
 *                    http://www.networkmultimedia.org
 *
 * Maintainer:        Michael Repplinger <repplinger@cs.uni-sb.de>
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or (at your option) any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 * 
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307
 * USA
 */


#include "Factory1394.hpp"
#include "Camera1394.hpp"
#include <l1394/l1394_session.h>

namespace NMM{

Factory1394::Factory1394(NodeFactory* fac)
  : NodeFactory("NodeFactory1394", fac), session(0), is_init(false)
{
  session = L1394::SSession::getInstance();
}

Factory1394::~Factory1394()
{
  for (list<GenericFireWireNode*>::iterator it = device_list.begin(); it != device_list.end(); ++it)
    delete *it;

}

void Factory1394::releaseNode(const Node& )
  throw (const FactoryException& ) {

}

Node& Factory1394::createNode(const FactoryTicket& fp)
       throw (const FactoryException& )
{
  MutexGuard m(mutex);

  L1394::Device* device = session->findDevice(fp.getObjectID());
  if (!device) {
    cout << "A Device with guid " << hex << fp.getObjectID() << dec << " is not available" << endl;
    throw FactoryException("A Device with guid is not available");
  }

  setTicketInvalid(fp);
  switch(device->getDeviceType()) {
    case L1394::_Camera        : return createCamera(fp.getObjectID()); break;
    case L1394::_Vcr           : throw FactoryException("VCR currently not supported"); break;
    case L1394::_VideoMonitor  : throw FactoryException("VideoMonitor currently not supported"); break;
    case L1394::_Disc          : throw FactoryException("Disc currently not supported"); break;
    case L1394::_Tuner         : throw FactoryException("Tuner currently not supported"); break;
  }
  throw FactoryException("Unknown error");
}



Camera1394& Factory1394::createCamera(const u_int64_t& guid)
      throw (const FactoryException& )
{
  L1394::Camera* cam_device = session->findCamera(guid);
  if (!cam_device)
    throw FactoryException("A camera with given guid does not exist");

  Camera1394* camera = new Camera1394("Camera1394");
  camera->setDevice(cam_device);


  return *camera;
}



} //end namespace

