/* NMM - Network-Integrated Multimedia Middleware
 *
 * Copyright (C) 2002-2005
 *                    NMM work group,
 *                    Computer Graphics Lab,
 *                    Saarland University, Germany
 *                    http://www.networkmultimedia.org
 *
 * Maintainer:        NMM work group,
 *                    http://www.networkmultimedia.org
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or (at your option) any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 * 
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307
 * USA
 */



#ifndef NMM_SOURCENETNODE_HPP
#define NMM_SOURCENETNODE_HPP

#include "nmm/base/GenericSourceNode.hpp"

#include "nmm/utils/net/INetAddr.hpp"
#include "nmm/utils/net/Sock.hpp"
#include "nmm/utils/net/SockConnector.hpp"

#include "nmm/interfaces/net/INetClientImpl.hpp"

namespace NMM {

  class SourceNetNode
    : public GenericSourceNode
    , public INetClientImpl
  {
  public: 

    /** Constructor
     */
    SourceNetNode(const char* = "SourceNetNode",
                  StreamQueue::Mode = StreamQueue::MODE_SUSPEND,
                  int = StreamQueue::DEFAULT_SIZE);

    /** Destructor
     */
    ~SourceNetNode();

    /**
     * INetClient interface.
     */
    void setPort(const int& value);
    void setHostname(const string& value);
    void setProtocol(const string& value);
    int getPort() const;
    string getHostname() const;
    string getProtocol() const;
    void connectionLost();
    
  protected:
    virtual Buffer* processBuffer(Buffer *);
    
    virtual Result doInit();
    virtual Result doInitOutput();
    virtual Result doDeactivate();
    
  private:
    Sock          *s_stream;
    SockConnector  s_connect;
    INetAddr       addr;
    bool           is_connected;
    string         local_host;
  };
  
}//end namespace

#endif
