/* NMM - Network-Integrated Multimedia Middleware
 *
 * Copyright (C) 2002-2005
 *                    NMM work group,
 *                    Computer Graphics Lab,
 *                    Saarland University, Germany
 *                    http://www.networkmultimedia.org
 *
 * Maintainer:        Marco Lohse <mlohse@cs.uni-sb.de>
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or (at your option) any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 * 
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307
 * USA
 */


#ifndef NMM_DEVNULLNODE_HPP
#define NMM_DEVNULLNODE_HPP

#include "nmm/base/GenericSinkNode.hpp"

#include "nmm/interfaces/stats/IBufferCounterImpl.hpp"
#include "nmm/interfaces/stats/ISleepImpl.hpp"
#include "nmm/interfaces/stats/IBuffersPassedEventImpl.hpp"

namespace NMM {

  /**
   * DevNullNode is a Node which consumes any Buffers
   * passed to it. Its only use is for
   * statistical purposes.
   */
  class DevNullNode
    : public GenericSinkNode
    , public IBufferCounterImpl
    , public ISleepImpl
    , public IBuffersPassedEventSenderImpl
  {
  private:
    unsigned int sleep_time; // used to simulate a blocking device
    long buffers_passed;
    long upstream_passed;
    long interval;
    
  protected:
    virtual void setSleepTime(const int& value);
    virtual int getSleepTime() const;
    
    virtual int getBuffersPassed() const;
    virtual void resetBufferCounter();
    
    virtual void setBufferCountInterval(const int& value);
    virtual int getBufferCountInterval() const;
    
    virtual Result doInit();
    virtual Result doInitOutput();
    
    virtual Message *processBuffer(Buffer *in_buffer);
    
  public:

    DevNullNode(const char* name = "DevNullNode",
		const StreamQueue::Mode queue_mode = StreamQueue::DEFAULT_MODE,
		const int size = StreamQueue::DEFAULT_SIZE);
    virtual ~DevNullNode() {};
  };
}

#endif //NMM_DEVNULLNODE_HPP
