/* NMM - Network-Integrated Multimedia Middleware
 *
 * Copyright (C) 2002-2005
 *                    NMM work group,
 *                    Computer Graphics Lab,
 *                    Saarland University, Germany
 *                    http://www.networkmultimedia.org
 *
 * Maintainer:        NMM work group,
 *                    http://www.networkmultimedia.org
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or (at your option) any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 * 
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307
 * USA
 */


#include "IdNode.hpp"
#include "nmm/base/format/Property.hpp"
#include "nmm/base/registry/Plugin.hpp"
#include "nmm/base/memorymgr/Buffer.hpp"
#include <unistd.h>

NMM_REGISTER_PLUGIN(NMM::IdNode, "IdNode")

namespace NMM {

  IdNode::IdNode(const char* name,
                 const StreamQueue::Mode queue_mode,
                 const int size)
    : GenericFilterNode(name, queue_mode, size) 
  {
    buffers_passed = 0;
    upstream_passed = 0;
    sleep_time = 0;
  }
  
  void IdNode::setSleepTime(const int& value) {
    sleep_time = value;
  }
  
  int IdNode::getSleepTime() const {
    return sleep_time;
  }
  
  int IdNode::getBuffersPassed() const {
    return buffers_passed;
  }
  
  void IdNode::resetBufferCounter() {
    buffers_passed = 0;
  }
  
  Result IdNode::doInit() {
    Format* format;
    
    format = getOwnInputProperty()->addNewFormat("*/*");
    getOwnInputProperty()->setDefaultFormat(format);
    
    format = getOwnOutputProperty()->addNewFormat("*/*");
    getOwnOutputProperty()->setDefaultFormat(format);
    
    return SUCCESS;
  }
  
  Result IdNode::doInitOutput() {
    const Format* in_format = getInputFormat();
    if (in_format == NULL) { return FAILURE; }
    Format* format;
    
    format = new Format( *in_format );
    getOwnWorkingInputProperty()->addNewFormat(format);
    getOwnWorkingInputProperty()->setDefaultFormat(format);
    
    format = new Format( *in_format );
    getOwnWorkingOutputProperty()->addNewFormat(format);
    getOwnWorkingOutputProperty()->setDefaultFormat(format);
    
    return SUCCESS;
  }
  
  Message* IdNode::processBuffer(Buffer *in_buffer) {
    buffers_passed++;
    if (sleep_time) {
      usleep(sleep_time);
    }
    return in_buffer;
  }
  
  Result IdNode::buffersPassed(const int& value) {
    upstream_passed += value;
    return SUCCESS;
  }
  
  int IdNode::getUpstreamBuffersPassed() const {
    return upstream_passed;
  }
  
  void IdNode::resetUpstreamBufferCounter() {
    upstream_passed = 0;
  }
}
