/* NMM - Network-Integrated Multimedia Middleware
 *
 * Copyright (C) 2002-2005
 *                    NMM work group,
 *                    Computer Graphics Lab,
 *                    Saarland University, Germany
 *                    http://www.networkmultimedia.org
 *
 * Maintainer:        NMM work group,
 *                    http://www.networkmultimedia.org
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or (at your option) any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 * 
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307
 * USA
 */


#include "SimpleProcessorNode.hpp"
#include "nmm/base/format/Property.hpp"
#include "nmm/base/registry/Plugin.hpp"
#include <unistd.h>

NMM_REGISTER_PLUGIN(NMM::SimpleProcessorNode, "SimpleProcessorNode")

namespace NMM {

  SimpleProcessorNode::SimpleProcessorNode(const char* name,
                 const StreamQueue::Mode queue_mode,
                 const int size)
    : GenericFilterNode(name, queue_mode, size), __method_call_counter(0)
  {
  }

  SimpleProcessorNode::~SimpleProcessorNode() {
 
  }

    void SimpleProcessorNode::noArgumentNoReturn() {
      ++__method_call_counter;
    }

    int SimpleProcessorNode::noArgumentIntReturn() {
      ++__method_call_counter;
      return 0;
    }


    int SimpleProcessorNode::oneArgumentInIntReturn(const int& ) {

      ++__method_call_counter;
      return 0;
    }
    int SimpleProcessorNode::twoArgumentInIntReturn(const int& , const int& ){

      ++__method_call_counter;
      return 0;
    }
    int SimpleProcessorNode::threeArgumentInIntReturn(const int& , const int& , const int& ){

      ++__method_call_counter;
      return 0;
    }
    int SimpleProcessorNode::fourArgumentInIntReturn(const int& , const int& , const int& , const int& ){

      ++__method_call_counter;
      return 0;
    }
    int SimpleProcessorNode::fiveArgumentInIntReturn(const int& , const int& , const int& , const int& , const int& )
    {

      ++__method_call_counter;
      return 0;
    }

    int SimpleProcessorNode::oneArgumentOutIntReturn(int& ){

      ++__method_call_counter;
      return 0;
    }
    int SimpleProcessorNode::twoArgumentOutIntReturn(int& , int& ){

      ++__method_call_counter;
      return 0;
    }
    int SimpleProcessorNode::threeArgumentOutIntReturn(int& , int& , int& ){

      ++__method_call_counter;
      return 0;
    }
    int SimpleProcessorNode::fourArgumentOutIntReturn(int& , int& , int& , int& ){

      ++__method_call_counter;
      return 0;
    }
    int SimpleProcessorNode::fiveArgumentOutIntReturn(int& , int& , int& , int& , int& ){

      ++__method_call_counter;
      return 0;
    }

    int SimpleProcessorNode::oneArgumentInOutIntReturn(int& ){

      ++__method_call_counter;
      return 0;
    }
    int SimpleProcessorNode::twoArgumentInOutIntReturn(int& , int& ){

      ++__method_call_counter;
      return 0;
    }
    int SimpleProcessorNode::threeArgumentInOutIntReturn(int& , int& , int& ){

      ++__method_call_counter;
      return 0;
    }
    int SimpleProcessorNode::fourArgumentInOutIntReturn(int& , int& , int& , int& ){

      ++__method_call_counter;
      return 0;
    }
    int SimpleProcessorNode::fiveArgumentInOutIntReturn(int& , int& , int& , int& , int& ){
      
      ++__method_call_counter;
      return 0;
    }

    void SimpleProcessorNode::stringInArgument(const string& ) {
      ++__method_call_counter;
    }
  
  void SimpleProcessorNode::stringOutArgument(string& ) {
    ++__method_call_counter;
  }

  void SimpleProcessorNode::stringInOutArgument(string& ) {
    ++__method_call_counter;
  }

  Result SimpleProcessorNode::doInit() {
    Format* format;

    format = getOwnInputProperty()->addNewFormat("*/*");
    getOwnInputProperty()->setDefaultFormat(format);

    format = getOwnOutputProperty()->addNewFormat("*/*");
    getOwnOutputProperty()->setDefaultFormat(format);

    return SUCCESS;
  }

  Result SimpleProcessorNode::doInitOutput() {
    const Format* in_format = getInputFormat();
    if (in_format == NULL) { return FAILURE; }
    Format* format;

    format = new Format( *in_format );
    getOwnWorkingInputProperty()->addNewFormat(format);
    getOwnWorkingInputProperty()->setDefaultFormat(format);

    format = new Format( *in_format );
    getOwnWorkingOutputProperty()->addNewFormat(format);
    getOwnWorkingOutputProperty()->setDefaultFormat(format);

    return SUCCESS;
  }

  Message* SimpleProcessorNode::processBuffer(Buffer *in_buffer) {
    return in_buffer;
  }

}
