/* NMM - Network-Integrated Multimedia Middleware
 *
 * Copyright (C) 2002-2005
 *                    NMM work group,
 *                    Computer Graphics Lab,
 *                    Saarland University, Germany
 *                    http://www.networkmultimedia.org
 *
 * Maintainer:        Michael Repplinger <repplinger@cs.uni-sb.de>
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or (at your option) any later version.
 *
 * This library is distributed the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307
 * USA
 */


#ifndef NMM_SIMPLEPROCESSORNODE_HPP
#define NMM_SIMPLEPROCESSORNODE_HPP

#include "nmm/base/GenericFilterNode.hpp"

#include "nmm/interfaces/stats/IMethodCallImpl.hpp"

namespace NMM {

  /**
   * IdNode is a Node which performs the identity function, i.e.
   * passes on any buffers unaltered. Its only use is for
   * statistical purposes.
   */
  class SimpleProcessorNode
    : public GenericFilterNode
    , public IMethodCallImpl
  {
  public:
    SimpleProcessorNode(const char* name = "IdNode",
           const StreamQueue::Mode queue_mode = StreamQueue::DEFAULT_MODE,
           const int size = StreamQueue::DEFAULT_SIZE);
    ~SimpleProcessorNode();
  protected:
    Result doInit() ;

    Result doInitOutput();

    virtual void noArgumentNoReturn();

    virtual int noArgumentIntReturn();

    virtual int oneArgumentInIntReturn(const int& value1);
    virtual int twoArgumentInIntReturn(const int& value1, const int& value2);
    virtual int threeArgumentInIntReturn(const int& value1, const int& value2, const int& value3);
    virtual int fourArgumentInIntReturn(const int& value1, const int& value2, const int& value3, const int& value4);
    virtual int fiveArgumentInIntReturn(const int& value1, const int& value2, const int& value3, const int& value4, const int& value5);

    virtual int oneArgumentOutIntReturn(int& value1);
    virtual int twoArgumentOutIntReturn(int& value1, int& value2);
    virtual int threeArgumentOutIntReturn(int& value1, int& value2, int& value3);
    virtual int fourArgumentOutIntReturn(int& value1, int& value2, int& value3, int& value4);
    virtual int fiveArgumentOutIntReturn(int& value1, int& value2, int& value3, int& value4, int& value5);

    virtual int oneArgumentInOutIntReturn(int& value1);
    virtual int twoArgumentInOutIntReturn(int& value1, int& value2);
    virtual int threeArgumentInOutIntReturn(int& value1, int& value2, int& value3);
    virtual int fourArgumentInOutIntReturn(int& value1, int& value2, int& value3, int& value4);
    virtual int fiveArgumentInOutIntReturn(int& value1, int& value2, int& value3, int& value4, int& value5);

    virtual void stringInArgument(const string&);
    virtual void stringOutArgument(string&);
    virtual void stringInOutArgument(string&);

    virtual int getMethodCallCounter() { return __method_call_counter; }

    virtual void resetMethodCallCounter() { __method_call_counter = 0; }
    virtual Message *processBuffer(Buffer *in_buffer);
  private:
    int __method_call_counter;
  };
}

#endif //NMM_IDNODE_HPP
