/* NMM - Network-Integrated Multimedia Middleware
 *
 * Copyright (C) 2002-2005
 *                    NMM work group,
 *                    Computer Graphics Lab,
 *                    Saarland University, Germany
 *                    http://www.networkmultimedia.org
 *
 * Maintainer:        Florian Winter <fw@graphics.cs.uni-sb.de>
 *                     
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or (at your option) any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 * 
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307
 * USA
 */


#ifndef NMM_GLDISPLAY_HPP
#define NMM_GLDISPLAY_HPP NMM_GLDISPLAY_HPP
      
#include "nmm/misc/gui/XVirtualImg.hpp"

#include <GL/glx.h>

namespace NMM {

  /**
   * This is a class for writing video frames to a X Display using
   * OpenGL calls.
   */
  class GLDisplay
    : public XVirtualImg {

  private:
    XVisualInfo* __gl_visual_info;
    
    //! The GLX Window context as replacemant for the XVirtualImg's X
    //  GraphicsContext _gc
    GLXContext _ctx;

    //! indicated, that the incoming data is in RGBA channel order,
    //  which implies, that prepareOutput() must convert it to BGRA
    bool rgb_channel_order;

    //! width of the texture image
    const int tex_img_width;
    //! height of the texture image
    const int tex_img_height;

    /**
     * This is a small private helper method with some GL code needed
     * in multiple method
     */
    void resizeGL();

  protected:
    /**
     * override virtual method from XVirtualImage, init XVisualInfo
     * with glX method
     */
    virtual void initVisual();
    
    virtual void deinitVisual();
    
    /**
     * override virtual method from XVirtualImage, init graphics context
     * with glX method
     */
    virtual void initContext();

    /**
     * override virtual method from XVirtualImage, free graphics context
     * with glX method
     */
    virtual void deinitContext();


    // implementation of pure virtual methods from XVirtualImg
    virtual bool initImage();
    virtual void deinitImage();


  public:

    /**
     * Constructor:
     * \param parent Node* to the DisplayNode, this Img is running in
     * \param width The width of the video frames, to be presented
     * \param height The width of the video frames, to be presented
     * \param title The title used for the X-Window to be created
     * \param rgb Flag to indicate that the upcoming image data is in
     *            RGBA channel order
     */
    GLDisplay(Node* parent,
              int width, int height,
              string title,
              bool rgb);

    /**
     * Destructor
     */
    ~GLDisplay();

    // implementation of pure virtual methods from XVirtualImg
    void prepareOutput(NMM::Buffer*);
    void update( NMM::Buffer*);

    char* getImage() { return NULL; }
  };         

}
#endif //GLDISPLAY_HPP
