/* NMM - Network-Integrated Multimedia Middleware
 *
 * Copyright (C) 2002-2005
 *                    NMM work group,
 *                    Computer Graphics Lab,
 *                    Saarland University, Germany
 *                    http://www.networkmultimedia.org
 *
 * Maintainer:        Florian Winter <fw@graphics.cs.uni-sb.de>
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or (at your option) any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 * 
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307
 * USA
 */


#ifndef NMM_XIMG_HPP
#define NMM_XIMG_HPP NMM_XIMG_HPP

#include "nmm/misc/gui/XVirtualImg.hpp"

namespace NMM {

  /**
   * This is a class for writing video frames to a X Display using
   * pure Xlib calls.
   */
  class XImg
    : public XVirtualImg {

  protected:
    //! pointer to the XImage used for putting the incomming images in
    XImage* ximage;

    // implementation of pure virtual methods from XVirtualImg
    virtual bool initImage();
    virtual void deinitImage();

  public:

    /**
     * Constructor:
     * \param parent Node* to the DisplayNode, this Img is running in
     * \param width The width of the video frames, to be presented
     * \param height The width of the video frames, to be presented
     * \param title The title used for the X-Window to be created
     */
    XImg(Node* parent,
         int width, int height,
         string title);

    /**
     * Destructor
     */
    virtual ~XImg();

    // implementation of pure virtual methods from XVirtualImg
    virtual void prepareOutput(Buffer*);
    virtual void update(Buffer*);

    virtual char* getImage();
  };

}

#endif //NMM_XIMG_HPP
