/* NMM - Network-Integrated Multimedia Middleware
 *
 * Copyright (C) 2002-2005
 *                    NMM work group,
 *                    Computer Graphics Lab,
 *                    Saarland University, Germany
 *                    http://www.networkmultimedia.org
 *
 * Maintainer:        
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or (at your option) any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 * 
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307
 * USA
 */


#ifndef _NMM_DIVX4DECODENODE_HPP_
#define _NMM_DIVX4DECODENODE_HPP_ _NMM_DIVX4DECODENODE_HPP_

#include "nmm/base/GenericProcessorNode.hpp"

#include "nmm/interfaces/file/ITrackImpl.hpp"
#include "nmm/interfaces/video/IVideoDelayImpl.hpp"
#include "nmm/interfaces/video/IKeyframeImpl.hpp"

extern "C" {
#ifndef LINUX
#define LINUX
#define DIVX4DECODENODE_DEFINED_LINUX
#endif
#include <decore.h>
#ifdef DIVX4DECODENODE_DEFINED_LINUX
#undef LINUX
#undef DIVX4DECODENODE_DEFINED_LINUX
#endif
}


namespace NMM {

  /**
   * This class takes divx4-encoded Video Frames in YUV colorspace
   * and converts them to yuv-raw-frames by using the opendivx-library from
   * http://www.projectmayo.com or the DivX4-Library from
   * http://www.divx.com
   *
   * This Node has one Input Stream and one Output Stream.
   */
  class DivX4DecodeNode :
    public GenericProcessorNode,
    public ITrackImpl,
    public IVideoDelayImpl,
    public IKeyframeImpl
  {
  private:
    Result releaseDecoder(); ///< Release the Decoder-library
    
    // variables taken from DEC_PARAM struct from decore.h
    int x_dim;		     ///< the x dimension of the frames to be decoded
    int y_dim;		     ///< the y dimension of the frames to be decoded
    int output_format;       ///< the format of the image frame
    double framerate;	     ///< the frame rate of the sequence to be decoded

    DEC_FRAME decframe;      ///< structure from decore.h used to decode frames
    DEC_FRAME_INFO frameinfo;///< structure from decore.h used to get informations about the decoded frame (e.g. key-frame?)

    int postproc_level;      ///< level for postprocessing, range from 0 to 100
    long handle;             ///< decore needs a unique handle to the calling entity

    Timestamp timestamp;     //! The timestamp to put in the next outgoing buffer.
    Timestamp in_buf_ts; 

    int vid_delay;           ///< delay for video in milli-seconds

    bool released;           ///< flag: is the decoder already released?

    string compressor;       ///< what is the compressor string ("DIVX", "DIV3", ...)
    
    ThreadMutex divx_mutex;  ///< Thread-Mutex
    
    bool skip_until_keyframe;

    Timestamp save_ts;       ///< Timestamp which stores the latest valid Timestamp


  protected:
    virtual Result doActivate();     ///< The standard NMM doActivate()-Method
    virtual Result doInit();         ///< The standard NMM doInit()-Method
    virtual Result doDeinit();       ///< The standard NMM doDeinit()-Method
    virtual Result doInitOutput();   ///< The standard NMM doInitOutput()-Method
    virtual Result doFlush();        ///< The standard NMM doFlush()-Method
    virtual Result doDeactivate();
    /**
     * ITrack interface.
     * Instream event handlers for start and end of tracks.
     */
    virtual Result startTrack(const string& filename);
    virtual Result endTrack();
    virtual Result finished();
    
    /**
     * IVideoDelay interface.
     * Instream event handler for setting the video delay.
     */
    virtual Result setVideoDelay(const int& delay);


    /** IKeyframe interface. */
    virtual Result setKeyframe();
    
  public:

    DivX4DecodeNode(const char* name = "DivX4DecodeNode",
		    StreamQueue::Mode qmode = StreamQueue::MODE_SUSPEND,
		    int qsize = StreamQueue::DEFAULT_SIZE);
    ///< Constructor; takes the Nodename, the Mode of the Stream-Queue and the size of the Stream-Queue as arguments (all optional)
    ~DivX4DecodeNode(); ///< Destructor
    
    virtual Message* processBuffer(Buffer*); ///< The standard NMM processBuffer method. Here, the divx4 video buffers are decoded
  };

}

#endif //_NMM_DIVX4DECODENODE_HPP_
