/* NMM - Network-Integrated Multimedia Middleware
 *
 * Copyright (C) 2002-2005
 *                    NMM work group,
 *                    Computer Graphics Lab,
 *                    Saarland University, Germany
 *                    http://www.networkmultimedia.org
 *
 * Maintainer:         
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or (at your option) any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307
 * USA
 */


#ifndef _NMM_DOUBLELISTUNIT_HPP_
#define _NMM_DOUBLELISTUNIT_HPP_ _NMM_DOUBLELISTUNIT_HPP_

#include "OSDElement.hpp"

#include "nmm/interfaces/video/osd/manager/ITextView.hpp"
#include "nmm/interfaces/video/osd/manager/IPlaneDecorator.hpp"
#include "nmm/interfaces/video/osd/manager/IBorderDecorator.hpp"
#include "nmm/interfaces/video/osd/manager/IHeaderDecorator.hpp"

namespace NMM {
  class DoubleListUnit : public OSDElement {
  public:
    typedef vector<string> line_t;
    
    DoubleListUnit( IOSDManager* manager, IWidgetFactory* factory, string font );
    ~DoubleListUnit();
    
    void setColors( unsigned int win_active_color, unsigned int win_inactive_color,
                    unsigned int sel_active_color, unsigned int sel_active_color2, unsigned int sel_inactive_color );
    
    void selectWindow( unsigned int mode=0 );
    void deSelectWindow();
    void setHeader( string h );
    
    void setFirstColumnWidth( unsigned int w );
    
    unsigned int addLine( string, line_t, string url="" );
    
    string getLineUrl( unsigned int l );
    string getLineContent( unsigned int l );
    void chooseLineContent( unsigned int a, unsigned int b );
    void chooseLineContent( unsigned int a, string b );
    unsigned int getSelectedLine();
    void changeSelectedLineContent( string b );
    void changeLineContent( unsigned int l, string b );

    void changeSelectedLineSubject(string b);
    void changeLineSubject(unsigned int l, string b);
    string getLineSubject(unsigned int l);

    unsigned int getNumLines();
    void reset();
    void selectLine(const unsigned int& l);
    void selectLine2(const unsigned int& l);
    
    void deleteLine( unsigned int pos );
    void deleteAllLines();
    
    Result selectUp();
    Result selectDown();
    Result selectLeft();
    Result selectRight();
    
    
  protected:
    unsigned int  win_active_color;
    unsigned int  win_inactive_color;
    unsigned int  sel_active_color;
    unsigned int  sel_active_color2;
    unsigned int  sel_inactive_color;
    
    unsigned int  column_width;
    
    void deleteWidgets();
    string font;
    
    ITextView* getSubList( unsigned int line );
    
    ITextView_var	  first;
    IWidgetComposite_var  composite;
    IPlaneDecorator_var   plane;
    IBorderDecorator_var  border;
    IHeaderDecorator_var  header;
  };
  
}

#endif //_NMM_DOUBLELISTUNIT_HPP_
