/* NMM - Network-Integrated Multimedia Middleware
 *
 * Copyright (C) 2002-2005
 *                    NMM work group,
 *                    Computer Graphics Lab,
 *                    Saarland University, Germany
 *                    http://www.networkmultimedia.org
 *
 * Maintainer:         
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or (at your option) any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 * 
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307
 * USA
 */

#include "listunit.hpp"
#include <sstream>

namespace NMM {
  
  ListUnit::ListUnit( IOSDManager* manager, IWidgetFactory* factory, string _font ) : OSDElement(manager, factory),
                                                                                      font(_font) {
    setColors( 0xff000000, 0xff100000, 0xff707070, 0xff303030, 0xff101010 );
    
    text.reset(factory->createTextView());
    text->setFont(font);
    
    text->setSelectionColor( sel_inactive_color );
    
    scroll.reset(factory->createScrollDecorator( *text ));
    plane.reset(factory->createPlaneDecorator( *scroll ));
    plane->setPlaneColor( win_inactive_color );
    border.reset(factory->createBorderDecorator( *plane ));
    header.reset(factory->createHeaderDecorator( *border ));
    header->setFont(font);
    
    win->setContent( *header );
  }
  
  ListUnit::~ListUnit() {
    deleteWidgets();
  }
  
  void ListUnit::deleteWidgets() {
    if ( text.get() ) {
      manager->deleteWidget(*text);
      text.reset(NULL);
    }

    if ( plane.get() ) {
      manager->deleteWidget(*plane);
      plane.reset(NULL);
    }

    if ( border.get() ) {
      manager->deleteWidget(*border);
      border.reset(NULL);
    }

    if ( scroll.get() ) {
      manager->deleteWidget(*scroll);
      scroll.reset(NULL);
    }

    if ( header.get() ) {
      manager->deleteWidget(*header);
      header.reset(NULL);
    }
  }
  
  void ListUnit::setColors( unsigned int _win_active_color, unsigned int _win_inactive_color,
                            unsigned int _sel_active_color, unsigned int _sel_active_color2, unsigned int _sel_inactive_color ) {

    win_active_color   = _win_active_color;
    win_inactive_color = _win_inactive_color;
    sel_active_color   = _sel_active_color;
    sel_active_color2  = _sel_active_color2;
    sel_inactive_color = _sel_inactive_color;
  }
  
  void ListUnit::selectWindow( unsigned int mode ) {
    if ( mode == 1 ) {
      text->setSelectionColor( sel_active_color2 );
    } else {
      text->setSelectionColor( sel_active_color );
    }
    plane->setPlaneColor( win_active_color );
  }

  void ListUnit::deSelectWindow() {
    text->setSelectionColor( sel_inactive_color );
    plane->setPlaneColor( win_inactive_color );
  }
  
  void ListUnit::setHeader( string h ) {
    header->setText( h );
  }
  
  unsigned int ListUnit::getSelectedLine(){
    if(text.get()){
      return text->getSelectedLine();
    }else{
      return 0;
    }
  }

  string ListUnit::getLineURL(const unsigned int i) {
    return text->getLineURL(i);
  }

  void ListUnit::selectLine(const unsigned int& i){
    text->selectLine(i);
  }

  string ListUnit::getLineContent(int i){
    return text->getLineContent(i);
  }

  void ListUnit::deleteAllLines(){
    text->deleteAllLines();
  }

  int ListUnit::addLine(string c, string url){
    return text->addLine(c,url);
  }

  int ListUnit::addLine(string c){
    return text->addLine(c);
  }

  void ListUnit::addMultiLine(string c){
    stringstream s(c);
    unsigned int linelength = 0;
    string tmp;
    string line;
    while(s>>tmp)
      {
        linelength += tmp.length() *20; //length of word in pixel (approx)
        
        if(tmp == "/"){
          text->addLine(line);
          line="";
          linelength = 0;
        }else if(tmp == ":"){
          line += tmp + " ";
          text->addLine(line);
          line="";
          linelength = 0; 
        }else if(tmp == "..."){
          line += tmp + " ";
          text->addLine(line);
          text->addLine(string());
          line="";
          linelength = 0;
        }else{
          if(linelength < static_cast<unsigned int>(width))
            {
              line += tmp + " ";
            }else{  
              line += tmp + " ";
              text->addLine(line);
              line="";
              linelength = 0;
            }
        }
      }
    if (line!="") text->addLine(line);
  }
  
  void ListUnit::deSelectLine() {
    text->deSelectLine();
  }

  Result ListUnit::selectUp() {
   text->selectUp();
   return SUCCESS;
  }

  Result ListUnit::selectDown() {
    text->selectDown();
    return SUCCESS;
  }

  void ListUnit::markLine(unsigned int line){
    text->markLine(line);
  }

  void ListUnit::unmarkLine(unsigned int line){
    text->unmarkLine(line);
  }

  list<unsigned int> ListUnit::getMarkedLines(){
    return text->getMarkedLines();
  }

}
