/* NMM - Network-Integrated Multimedia Middleware
 *
 * Copyright (C) 2002-2005
 *                    NMM work group,
 *                    Computer Graphics Lab,
 *                    Saarland University, Germany
 *                    http://www.networkmultimedia.org
 *
 * Maintainer:         
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or (at your option) any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307
 * USA
 */

#include "imageview.hpp"
#include "nmm/misc/overlay/overlay.h"

namespace NMM {

  /*****************************************************************************************/
  /* ImageView									         */
  /*****************************************************************************************/
  ImageView::ImageView() : bitmapreader(NULL) {
  }
  
  ImageView::~ImageView() {
  }

  void ImageView::setBitmapReader( BitmapReader* br ) {
    bitmapreader = br;
  }

  void ImageView::setBitmapID( const string& id ) {
    bitmapid = id;
  }

  void ImageView::overlayBitmap( string id, workspace_t* ws ) {
    unsigned char* destination;
    unsigned int   bitmapwidth, bitmapheight;
    unsigned int   v_width, v_height;

    /* pointer to pixel data (source) */
    unsigned char* source = bitmapreader->getImage( id, &bitmapwidth, &bitmapheight );
    if ( !source ) return;

    /* adjust width and height so that they don't extend the workspace dimension */
    v_width  = adjustDimension( xpos, width, ws->width );
    v_height = adjustDimension( ypos, height, ws->height );

    /* make sure v_width/v_height isn't larger than bitmapwidth/bitmapheight*/
    if ( v_width > bitmapwidth  ) v_width = bitmapwidth;
    if ( v_height > bitmapheight ) v_height = bitmapheight;

    /* pointer to destination */
    destination = (unsigned char*) (ws->data + ypos*(ws->stride<<2) + (xpos<<2));

    /* overlay the two bitmaps */
    RGBAoverlay( source, bitmapwidth, v_width, v_height,
		 destination, ws->stride );
  }

  void ImageView::draw( workspace_t* ws ) {
    overlayBitmap( bitmapid, ws );
  }

}

