/* NMM - Network-Integrated Multimedia Middleware
 *
 * Copyright (C) 2002-2005
 *                    NMM work group,
 *                    Computer Graphics Lab,
 *                    Saarland University, Germany
 *                    http://www.networkmultimedia.org
 *
 * Maintainer:         
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or (at your option) any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307
 * USA
 */

#include "textdraw.hpp"
#include "nmm/misc/overlay/overlay.h"

namespace NMM {

  bool TextDraw::draw( const char* text, BitmapFont* font, unsigned int* frame, unsigned int stride,
		       unsigned int xstart, unsigned int ystart,
		       unsigned int xend, unsigned int yend ) {

    char c;	       // actual character
    unsigned int cw, ch; // character width and height
    unsigned int c_stride;
    int linelength;      // number of chars in the actual line
    unsigned char* data; // pointer to character pixel data (source)
    unsigned char* pixel; // pointer to character pixel data (source)
    bool x_clip, y_clip;

    int i;

    i = 0;
    linelength = strlen( text );

    /* get one character out of the string */
    while( i < linelength ) {
      c = text[i];

      /* check width and height of the character */
      font->checkDimension( c, &cw, &ch );
      /* get pointer to character pixel data */
      data = font->getPixelData( c );
      // if no pixel data are available, produce a space of 10 pixels
      if ( !data ) cw = 10;

      y_clip = (ystart+ch) >= yend;
      if ( y_clip ) {
	if ( yend > ystart ) ch = yend-ystart;
	else ch = 0;
      }

      c_stride = cw;
      x_clip = (xstart+cw) >= xend;
      if ( x_clip ) {
	if ( xend > xstart ) cw = xend-xstart;
	else cw = 0;
      }

      pixel = (unsigned char*) (frame + ystart*stride + xstart);

      RGBAoverlay( data, c_stride, cw, ch,
		   pixel, stride );

      /* index of next character of the string */
      i++;

      if ( (x_clip || i>=linelength) && y_clip ) return( true );
      xstart+=cw; // next character start position

    }

    return( false );

  }

}
