/* NMM - Network-Integrated Multimedia Middleware
 *
 * Copyright (C) 2002-2005
 *                    NMM work group,
 *                    Computer Graphics Lab,
 *                    Saarland University, Germany
 *                    http://www.networkmultimedia.org
 *
 * Maintainer:        
 *                    
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version 2
 * of the License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307
 * USA
 */


/**
 * This NMM node was developed in the context of 
 * the multimedia course in WS2001/02 at 
 * University of Saarbruecken/Germany
 */


#ifndef _NMM_TVCARDREADNODE_HPP_
#define _NMM_TVCARDREADNODE_HPP_ _NMM_TVCARDREADNODE_HPP_

#include "nmm/interfaces/video/tv/ITVCardImpl.hpp"
#include "nmm/interfaces/device/IDeviceImpl.hpp"
#include "nmm/base/sync/GenericSyncSourceNode.hpp"
#include "nmm/base/format/Property.hpp"
#include "nmm/base/sync/StreamTimer.hpp"

extern "C"
{
#include <linux/videodev.h>
}

namespace NMM {

  class TVCardReadNode
    : public GenericSyncSourceNode,
      public ITVCardImpl
  {
  private:
    StreamTimer* timer;

  protected:
    string device;
    int device_fd;

    int width, height;
    int framesize;
    float frame_rate;
    int int_frame_rate;
    const int frame_rate_base;

    bool sendEvent; //status flag whether to send an event
    string filename;
    int fd;
    size_t buffer_size;
    long int seek;
    long int filesize;
    int _size;
    int _frame;
    int picture_size, offset1, offset2, uv_size;
    int grab_format;
    
    unsigned char* _mmap_buffer;
    video_capability _capability;
    video_buffer _vid_buffer;
    video_mbuf _vid_mbuf;
    video_mmap _vid_mmap;
    video_picture _vid_picture;
    video_window _vid_win;
    video_channel _vid_channel;
    video_tuner _vid_tuner;

    ThreadMutex mutex_v4l;

    Result setOutputFormat(int&);

    //    Result setFrquency();
    //    Result getFrequency();
    //    Result searchFrequency();

    // virtual functions inherited from GenericSyncSourceNode
    virtual Time getLocalTime(u_int64_t);

    //TVCardImpl functions
    virtual void setDevice(const string&) throw(const DeviceNotFoundException&);
    virtual string getDevice() const;
    virtual void setResolution(const int&, const int&);
    virtual void getResolution(int&, int&);
    virtual Result setChannel(const int &);
    virtual Result getChannel(int &,string &);
    virtual Result hasChannel(int& c1, const string& c2){return FAILURE;}

  public:
    
    TVCardReadNode(const char *name = "TVCardReadNode",
		   StreamQueue::Mode = StreamQueue::MODE_SUSPEND,
		   int = StreamQueue::DEFAULT_SIZE);
    virtual ~TVCardReadNode();

    Result doInit();
    Result doDeinit();
    Result doInitOutput();
    Result doActivate();
    virtual Message* produceBuffer(Buffer *);
  };

}

#endif
