/* NMM - Network-Integrated Multimedia Middleware
 *
 * Copyright (C) 2002-2005
 *                    NMM work group,
 *                    Computer Graphics Lab,
 *                    Saarland University, Germany
 *                    http://www.networkmultimedia.org
 *
 * Maintainer:        Marco Lohse <mlohse@cs.uni-sb.de>
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or (at your option) any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 * 
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307
 * USA
 */


#ifndef _BENCHMARK_HPP_
#define _BENCHMARK_HPP_

#include "utils.hpp"

namespace NMM {

  /**
   * Utility, which can be used to measure bytes/frame, frames-per-second, and bytes/s
   * Typical use:
   *
   * Initialization:
   * Benchmark benchmark;
   * benchmark.setEnabled(true);
   * benchmark.setNameObject(this);
   * benchmark.setMessage("compression performance");
   *
   * Every frame (typically Node::run()-method):
   * benchmark.benchmark(out->getUsed());
   * where out is a Buffer.
   * 
   */
  class Benchmark {
  protected:
    long frame_count;
    long total_bytes;
    long fps_looptimer_diff;
    looptimer fps_looptimer1;
    looptimer fps_looptimer2;
    
    bool enabled;
    string message;

    float fps;
    long bpf;
    long bps;

    NamedObject *master;
  public:

    /**
     * Constructor.
     * \param obj the NamedObject-Object
     * \param msg the message which will be printed out every 10th frame.
     */
    Benchmark(NamedObject* obj = 0, const char* msg = 0);

    ~Benchmark(); 
      
    /**
     * Sets the status of the object.
     * \param enabled true to enable benchmarking, falso to disable
     */
    void setEnabled(const bool enabled); 

    /**
     * Sets the message which will be displayed every 10 frames.
     * If no message was set, 'Benchmark' will be printed out.
     * \param msg the message which will be printed out every 10th frame.
     */
    void setMessage(const char *str);

    /**
     * Sets the NamedObject-Object of this Benchmark-Object
     * \param obj the NamedObject-Object
     */
    void setNamedObject(NamedObject *obj);

    /**
     * This method should be called every frame
     * \param usedBytes the bytes which were produced/consumed for this frame
     */
    void benchmark(const long usedBytes);

    inline const NamedObject* getNamedObject() const {return master;}
    inline const string& getMessage() const{return message;}
    inline const bool getEnabled() const{return enabled;}
    inline const float getFPS() const {return fps;}
    inline const long getBPF() const {return bpf;}
    inline const long getBPS() const {return bps;}
    inline const long getFrameCount() const {return frame_count; }
  };
}

#endif
