/* NMM - Network-Integrated Multimedia Middleware
 *
 * Copyright (C) 2002-2005
 *                    NMM work group
 *                    Computer Graphics Lab,
 *                    Saarland University, Germany
 *                    http://www.networkmultimedia.org
 *
 * Maintainer:        Michael Reppliner <repplinger@cs.uni-sb.de>
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or (at your option) any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 * 
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307
 * USA
 */



#ifndef _EVENTPLAYER_HPP_
#define _EVENTPLAYER_HPP_

#include "nmm/base/ISender.hpp"
#include "nmm/base/IReceiver.hpp"
#include "nmm/base/sync/TimedElement.hpp"
#include "nmm/misc/timer/TimerEvents.hpp"

#include <string>

class istream;

namespace NMM {

class IValueStream;
class StdIStream;

/** An instance of this class can be used to send events, recorded with an EventRecorder, to an object.
  * A TimerProducer is used to send the events with the same time-interval as they were recorded.
  */
class EventPlayer  {
public:
/** Constructor
  * \param input_file Specify the input file to read the events.
  * \param delay Specifies an delay in seconds that is added to each event.
  */
  EventPlayer(const string& input_file, long delay = 0);

/** Destructor
  */
  virtual ~EventPlayer();

/** returns the timer producer
  */
  TimerProducer& getTimerProducer() { return __producer; }
  
private:
  void readEvents();

private: //member
  IValueStream*         __value_stream;  /// used ValueStream for deserialization
  StdIStream*           __std_i_stream;  /// wrapper for the std::istream
  std::istream*         __i_stream;      /// used istream to read from the file
  TimerProducer         __producer;      /// TimerProducer to send the Events at a specific time
  std::list<CEvent*>    __event_list;    /// stores the CEvents, read from the file
  long                  __delay;         /// delay that is added to each event.
};



}

#endif

