/* NMM - Network-Integrated Multimedia Middleware
 *
 * Copyright (C) 2002-2005
 *                    NMM work group,
 *                    Computer Graphics Lab,
 *                    Saarland University, Germany
 *                    http://www.networkmultimedia.org
 *
 * Maintainer:        Christoph Wellner <chwellner@graphics.cs.uni-sb.de>
 *
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version 2
 * of the License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307
 * USA
 */

#ifndef FORK_HPP
#define FORK_HPP FORK_HPP

#include <stdio.h>
#include <stdlib.h>
#include <ctype.h>
#include <unistd.h>
#include <sys/types.h>
#include <sys/socket.h>


#include <string>
#include <vector>

#include "nmm/utils/Singleton.hpp"
#include "nmm/utils/thread/ThreadedObject.hpp"

using namespace std;

namespace NMM{

  class ProcessDescription{
  private:
    ThreadMutex mutex;
    ThreadCondition c;
  public:
    int id;
    int pid;
    bool finished;
    int returnValue;
  public:
    ProcessDescription()
      :mutex(),
       c(mutex)
    {}
    
    ~ProcessDescription(){
      mutex.unlock();
    }

    const ThreadMutex& getThreadMutex() const {return mutex;}
    const ThreadCondition& getThreadCondition() const {return c;}
  };


  class Fork : public ThreadedObject {
    friend class CreateUsingNew<Fork>;
    
  public:

  private:
    ThreadMutex executemutex;

    //int-array for the 2 filde-descriptors
    int sv[2];
    //pipe to get the Child-ID
    int childPid;
        
    int internalProcessID;
    
    map<int, ProcessDescription*> runningChilds;
     
    void start();

    bool stopThread;

  private:
    Fork();
    Fork(const Fork&);
    ~Fork();
    
  public:
    
    int getReadSocket();
    int getWriteSocket();
    
    int executeCommand(string command);
    int killall();

    void run();
  };    
    
  /* Typdef to access the ForkHelper */
  typedef Singleton<Fork, CreateUsingNew> SFork;
}

#endif
