/* NMM - Network-Integrated Multimedia Middleware
 *
 * Copyright (C) 2002-2005
 *                    NMM work group,
 *                    Computer Graphics Lab,
 *                    Saarland University, Germany
 *                    http://www.networkmultimedia.org
 *
 * Maintainer:        Michael Repplinger <repplinger@cs.uni-sb.de>
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version 2
 * of the License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307,
 * USA.
 */

#include <string>
#include <list>
#include "nmm/interfaces/base/INode.hpp"

#include "nmm/nmm_strstream.hpp"

#ifndef _NMM_PARSERSTRUCTS_HPP_
#define _NMM_PARSERSTRUCTS_HPP_


namespace NMM {
  class NodeDescription;
}

/** a NCElement consist in an event (as string) and a state */
struct NCElement {
  string              event_key;
  vector<string>      argument_list;
  NMM::INode::State   state;
};

/** stores */
class NodeConfiguration {
public:  
  /** Constructor */
  NodeConfiguration( ) : __nd(0) {}
  /** resets the internal attributes */
  void clear()                                               { __event_list.clear(); __nd = 0; }
  /** adds a new event that should be send to the node */
  void addNCElement(const NCElement& nc)                     { __event_list.push_back(nc); }
  /** sets the NodeDescription that should receive the events */
  void setNodeDescription(NMM::NodeDescription* nd)          { __nd = nd; }
  /** returns the list with all events and their corresponding states */  
  const list<NCElement>& getEventList()                const { return __event_list; }
  /** returns the list with all events and their corresponding states */
        list<NCElement>& getEventList()                      { return __event_list; }
  /** returns the NodeDescription that should receive the events */
  const NMM::NodeDescription* getNodeDescription()     const { return __nd; }
  /** returns the NodeDescription that should receive the events */
        NMM::NodeDescription* getNodeDescription()           { return __nd; }
  
private:
  list<NCElement>        __event_list;            ///< stores the events (and corresponding states) that should be send to a node
  NMM::NodeDescription*  __nd;                    ///< stores the NodeDescription to identify the node
};

#endif
