/*---------------------------------------------------------------
 * Copyright (c) 1999,2000,2001,2002,2003                              
 * The Board of Trustees of the University of Illinois            
 * All Rights Reserved.                                           
 *--------------------------------------------------------------- 
 * Permission is hereby granted, free of charge, to any person    
 * obtaining a copy of this software (Iperf) and associated       
 * documentation files (the "Software"), to deal in the Software  
 * without restriction, including without limitation the          
 * rights to use, copy, modify, merge, publish, distribute,        
 * sublicense, and/or sell copies of the Software, and to permit     
 * persons to whom the Software is furnished to do
 * so, subject to the following conditions: 
 *
 *     
 * Redistributions of source code must retain the above 
 * copyright notice, this list of conditions and 
 * the following disclaimers. 
 *
 *     
 * Redistributions in binary form must reproduce the above 
 * copyright notice, this list of conditions and the following 
 * disclaimers in the documentation and/or other materials 
 * provided with the distribution. 
 * 
 *     
 * Neither the names of the University of Illinois, NCSA, 
 * nor the names of its contributors may be used to endorse 
 * or promote products derived from this Software without
 * specific prior written permission. 
 * 
 * THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, 
 * EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES 
 * OF MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE AND 
 * NONINFRINGEMENT. IN NO EVENT SHALL THE CONTIBUTORS OR COPYRIGHT 
 * HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER LIABILITY, 
 * WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, 
 * ARISING FROM, OUT OF OR IN CONNECTION WITH THE
 * SOFTWARE OR THE USE OR OTHER DEALINGS IN THE SOFTWARE. 
 * ________________________________________________________________
 * National Laboratory for Applied Network Research 
 * National Center for Supercomputing Applications 
 * University of Illinois at Urbana-Champaign 
 * http://www.ncsa.uiuc.edu
 * ________________________________________________________________
 * main.cpp
 * by Mark Gates <mgates@nlanr.net>
 * &  Ajay Tirumala <tirumala@ncsa.uiuc.edu>
 * -------------------------------------------------------------------
 * main does initialization and creates the various objects that will
 * actually run the iperf program, then waits in the Joinall().
 * -------------------------------------------------------------------
 * headers
 * uses
 *   <stdlib.h>
 *   <string.h>
 *
 *   <signal.h>
 * ------------------------------------------------------------------- */

#define HEADERS()

#include "headers.h"

#include "Client.hpp"
#include "Settings.hpp"
#include "Locale.hpp"
#include "Condition.hpp"
#include "List.h"
#include "util.h"

#include <iostream>
#include <string>
#include <sstream>

#include "main.hpp"

/* -------------------------------------------------------------------
 * prototypes
 * ------------------------------------------------------------------- */

//void waitUntilQuit( void );

//void sig_quit( int inSigno );

void cleanup( void );

/* -------------------------------------------------------------------
 * global variables
 * ------------------------------------------------------------------- */
#define GLOBAL()

NMM::IPerf17::IPerf17() {
  client_running = false;

  theListener = NULL;
  theSpeaker  = NULL;
  gSettings_c = NULL;
}

void NMM::IPerf17::deinit() {

    // with threads, send a quit signal
    gQuit_cond.Signal();

    // wait for other (client, server) threads to complete
    Thread::Joinall();

    cleanup();
}

void NMM::IPerf17::startAsyncClient(const string host, uint port, uint time, uint parallel) {
    signal(SIGPIPE,SIG_IGN);
    stringstream str_port("");
    stringstream str_time("");
    stringstream str_parallel("");
    str_port << port;
    str_time << time;
    str_parallel << parallel;

    int argc = 9;
    const char* argv[9] = {"iperf","-p",str_port.str().c_str(),"-t",str_time.str().c_str(),"-P", str_parallel.str().c_str() ,"-c",host.c_str()} ;

    ext_Settings* ext_gSettings = new ext_Settings;

    // read settings from environment variables and command-line interface
    /*Settings* */ gSettings_c = new Settings( ext_gSettings );
    gSettings_c->ParseEnvironment();
    gSettings_c->ParseCommandLine( argc, const_cast<char**>(argv) );

    theSpeaker = new Speaker(ext_gSettings);
    theSpeaker->OwnSettings();
    theSpeaker->DeleteSelfAfterRun();
    theSpeaker->Start();
//    cerr << "started speaker (" << theSpeaker << ")\n";
    client_running = true;
}

void NMM::IPerf17::getClientSpeed(float& speed) {
    if (client_running) {

      // wait for answer;
      speed = theSpeaker->waitForAnswer();
//      cerr << "my speed: " << speed << "  on  " << theSpeaker <<  "  from " << this <<endl;
      DELETE_PTR( gSettings_c );  // modified by qfeng
      client_running = false;
   }
}

void NMM::IPerf17::startClient(float& speed, const string host, uint port, uint time, uint parallel) {
  startAsyncClient(host, port, time, parallel);
  getClientSpeed(speed);
}

void NMM::IPerf17::startServer(uint port) {
    signal(SIGPIPE,SIG_IGN);
    stringstream str_port("");
    str_port << port;
    int argc = 4;
    const char* argv[4];
    argv[0] = "iperf";
    argv[1] = "-p";
    argv[2] = str_port.str().c_str();
    argv[3] = "-s";

    ext_Settings* ext_gSettings = new ext_Settings;
    Settings* gSettings = NULL;

    // read settings from environment variables and command-line interface
    gSettings = new Settings( ext_gSettings );
    gSettings->ParseEnvironment();
    gSettings->ParseCommandLine( argc, const_cast<char**>(argv) );


    // start up a listener
    theListener = new Listener( ext_gSettings );
    theListener->DeleteSelfAfterRun();

    theListener->Start();

    if ( ext_gSettings->mThreads == 0 ) {
        theListener->SetDaemon();

        // the listener keeps going; we terminate on user input only
        waitUntilQuit();

        if ( Thread::NumUserThreads() > 0 ) {
            printf( wait_server_threads );
            fflush( 0 );
        }
    }
    DELETE_PTR( gSettings );  // modified by qfeng
}

/* -------------------------------------------------------------------
 * sets up signal handlers
 * parses settings from environment and command line
 * starts up server or client thread
 * waits for all threads to complete
 * ------------------------------------------------------------------- */
/*
int main( int argc, char **argv ) {
    ext_Settings* ext_gSettings = new ext_Settings;
    Settings* gSettings = NULL;

    // read settings from environment variables and command-line interface
    gSettings = new Settings( ext_gSettings );
    gSettings->ParseEnvironment();
    gSettings->ParseCommandLine( argc, argv );

    // start up client or server (listener)
    if ( gSettings->GetServerMode() == kMode_Server ) {
        // start up a listener
        theListener = new Listener( ext_gSettings );
        theListener->DeleteSelfAfterRun();

        // Start the server as a daemon
        if ( gSettings->GetDaemonMode() == true ) {
            theListener->runAsDaemon(argv[0],LOG_DAEMON);
        }

        theListener->Start();

        if ( ext_gSettings->mThreads == 0 ) {
            theListener->SetDaemon();

            // the listener keeps going; we terminate on user input only
            waitUntilQuit();
            if ( Thread::NumUserThreads() > 0 ) {
                printf( wait_server_threads );
                fflush( 0 );
            }
        }

    } else if ( gSettings->GetServerMode() == kMode_Client ) {
        theSpeaker = new Speaker(ext_gSettings);
        theSpeaker->OwnSettings();
        theSpeaker->DeleteSelfAfterRun();
        theSpeaker->Start();

        // wait for answer;
        theSpeaker->waitForAnswer();

    } else {
        // neither server nor client mode was specified
        // print usage and exit

        printf( usage_short, argv[0], argv[0] );
    }

    // wait for other (client, server) threads to complete
    Thread::Joinall();
    DELETE_PTR( gSettings );  // modified by qfeng
    // all done!
    return 0;
} // end main
*/
/* -------------------------------------------------------------------
 * Blocks the thread until a quit thread signal is sent
 * ------------------------------------------------------------------- */

void NMM::IPerf17::waitUntilQuit( void ) {

    // signal handlers send quit signal on ^C and kill
    gQuit_cond.Lock();
//    my_signal( SIGTERM, sig_quit );
//    my_signal( SIGINT,  sig_quit );


    // this sleep is a hack to get around an apparent bug? in IRIX
    // where pthread_cancel doesn't work unless the thread
    // starts up before the gQuit_cand.Wait() call below.
    // A better solution is to just use sigwait here, but
    // then I have to emulate that for Windows...
    usleep( 10 );

    // wait for quit signal
    gQuit_cond.Wait();
    gQuit_cond.Unlock();
} // end waitUntilQuit

/* -------------------------------------------------------------------
 * Sends a quit thread signal to let the main thread quit nicely.
 * ------------------------------------------------------------------- */
/*
void sig_quit( int inSigno ) {

    // if we get a second signal after 1/10 second, exit
    // some implementations send the signal to all threads, so the 1/10 sec
    // allows us to ignore multiple receipts of the same signal
    static Timestamp* first = NULL;
    if ( first != NULL ) {
        Timestamp now;
        if ( now.subSec( *first ) > 0.1 ) {
            sig_exit( inSigno );
        }
    } else {
        first = new Timestamp();
    }

    // with threads, send a quit signal
    gQuit_cond.Signal();

} // end sig_quit
*/
/* -------------------------------------------------------------------
 * Any necesary cleanup before Iperf quits. Called at program exit,
 * either by exit() or terminating main().
 * ------------------------------------------------------------------- */

void cleanup( void ) {
    extern Iperf_ListEntry *clients;
    Iperf_destroy ( &clients );
} // end cleanup







