/* NMM - Network-Integrated Multimedia Middleware
 *
 * Copyright (C) 2002-2005
 *                    NMM work group,
 *                    Computer Graphics Lab,
 *                    Saarland University, Germany
 *                    http://www.networkmultimedia.org
 *
 * Maintainer:         
 *                    Michael Repplinger <repplinger@cs.uni-sb.de>
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or (at your option) any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 * 
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307
 * USA
 */

#include "INetAddr.hpp"

#include <netdb.h>

#include <sys/socket.h>  // for inet_ntoa
#include <netinet/in.h>  // for inet_ntoa
#include <arpa/inet.h>   // for inet_ntoa

#include "nmm/utils/utils.hpp"

namespace NMM {

  /****************** class INetAddr ******************/
  INetAddr::INetAddr(unsigned long addr, unsigned int port)
    : SerializedValue("INetAddr", "NMM")
  {
    memset (&_address, 0, sizeof _address);
    _address.sin_family      = AF_INET;

    setPort(port);
    setAddress(addr);
    setTransportProtocol("tcp");

    registerAttributes();
  }

  INetAddr::INetAddr(const char* addr, unsigned int port)
    : SerializedValue("INetAddr", "NMM")
  {
    memset (&_address, 0, sizeof _address);
    _address.sin_family      = AF_INET;

    setPort(port);
    setAddress(addr);
    setTransportProtocol("tcp");

    registerAttributes();
  }

  INetAddr::INetAddr()
    : SerializedValue("INetAddr", "NMM")
  {
    memset (&_address, 0, sizeof _address);
    _address.sin_family      = AF_INET;

    setPort(0);
    setAddress(static_cast<unsigned long>(0)); //static_cast avoids ambigious overload with setAddress(const char*)
    setTransportProtocol("tcp");

    registerAttributes();
  }

  INetAddr::INetAddr(const INetAddr& addr)
    : SerializedValue(addr)
  {
    *this = addr;
    registerAttributes();
  }

  INetAddr::~INetAddr(){}

  void INetAddr::setAddress(const char* addr) {
    struct hostent* host     = gethostbyname(addr);
    if (host) {
      memcpy(&_address.sin_addr.s_addr, host->h_addr_list[0], sizeof(unsigned long));
      _ip_address = getIpAddress();
    }
  }

  void INetAddr::setAddress(unsigned long addr) {
    /* convert the given addr to the network byte order */
    _address.sin_addr.s_addr = htonl(addr);
    _ip_address = addr;
  }
  
  void INetAddr::setAddress(struct sockaddr_in* addr) {
    memcpy(&_address, addr, sizeof(_address));
    _ip_address = ntohl(_address.sin_addr.s_addr);
    _port = ntohs(_address.sin_port);
  }
  
  string INetAddr::toString() const {
    return string(inet_ntoa(_address.sin_addr)) + string(":") + _port.getRValue();
  }
  
  void INetAddr::updateAttributes() {
    setAddress(_ip_address.getRValue());
    setPort(_port.getRValue());
    setTransportProtocol(_trans_prot.getRValue().c_str());
  }

  void INetAddr::registerAttributes() {
    insertArgument(&_trans_prot);
    insertArgument(&_port);
    insertArgument(&_ip_address);
  }


}
