/* NMM - Network-Integrated Multimedia Middleware
 *
 * Copyright (C) 2002-2005
 *                    NMM work group,
 *                    Computer Graphics Lab,
 *                    Saarland University, Germany
 *                    http://www.networkmultimedia.org
 *
 * Maintainer:         
 *                    Michael Repplinger <repplinger@cs.uni-sb.de>
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or (at your option) any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 * 
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307
 * USA
 */

#ifndef NMM_NETEXCEPTIONS_HPP
#define NMM_NETEXCEPTIONS_HPP NMM_NETEXCEPTIONS_HPP

#include "nmm/NMMTypes.hpp"

namespace NMM {

  /** Exception for transmissions
   */
  class TransmissionException : public Exception {
  public:
    TransmissionException(const string& comment, int my_errno) : Exception(comment), errno_code(my_errno) {};
    ~TransmissionException() {};
    const int errno_code;
  };

  /** Exception for connections
   */
  class ConnectionException : public Exception {
  public:
    enum ConnectionMethod{socket = 0, connect, close, open, shutdown, accept, listen, bind};
    ConnectionException(const string& comment, int my_errno, ConnectionMethod cm) : Exception("ConnectionException >" + comment), errno_code(my_errno), method_code(cm) {}
    ~ConnectionException() {};
    const int errno_code;
    const ConnectionMethod method_code;
  };

  class ConnectionClosed : public Exception {
  public:
    ConnectionClosed(int _errno)
      : Exception("Connection Closed by remote host"), errno_code(_errno) {}

    ConnectionClosed(const string& msg, int _errno) : Exception (msg), errno_code(_errno) {}

    const int errno_code;
  };

}

#endif
