/* NMM - Network-Integrated Multimedia Middleware
 *
 * Copyright (C) 2002-2005
 *                    NMM work group,
 *                    Computer Graphics Lab,
 *                    Saarland University, Germany
 *                    http://www.networkmultimedia.org
 *
 * Maintainer:         
 *                    Michael Repplinger <repplinger@cs.uni-sb.de>
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or (at your option) any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 * 
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307
 * USA
 */

#include "Sock.hpp"

#include <errno.h>       // for errnos
// #include <netdb.h>       // for gethostbyname
// #include <unistd.h>      // for close
// #include <sys/socket.h>  // for inet_ntoa
// #include <netinet/in.h>  // for inet_ntoa
// #include <arpa/inet.h>   // for inet_ntoa
// #include <signal.h>      // for signals

namespace NMM {

  /****************** class Sock ******************/
  void Sock::createWriteTransmissionException(int my_errno){
    switch(my_errno) {
    case EBADF  : throw TransmissionException("fd is not a valid file descriptor or is not open for writing.", my_errno);

    case EINVAL : throw TransmissionException("fd is attached to an object which is unsuitable for writing.", my_errno);

    case EFAULT : throw TransmissionException("buf is outside your accessible address space.", my_errno);

    case EFBIG : throw TransmissionException("An attempt was made to write a file that exceeds the implementation-defined maximum file size or the process' file size limit, or to  write  at  a position past than the maximum allowed offset.", my_errno);

    case EPIPE  : throw TransmissionException("fd  is connected to a pipe or socket whose reading end is closed.", my_errno);

    case EAGAIN : throw TransmissionException("Non-blocking I/O has been selected using O_NONBLOCK and there was no room in the\
              pipe or socket connected to fd to write the data immediately.", my_errno);

    case EINTR  : throw TransmissionException("The call was interrupted by a signal before any data was written.", my_errno);

    case ENOSPC : throw TransmissionException("The device containing the file referred to by fd has no room for the data.", my_errno);

    case EIO    : throw TransmissionException("A low-level I/O error occurred while modifying the inode.", my_errno);

    default     : throw TransmissionException("Unknown error", my_errno);
    }
  }

  void Sock::createReadTransmissionException(int my_errno) {
    switch (my_errno) {
    case EINTR  :
      throw TransmissionException("The call was interrupted by a signal before any data was read.", my_errno);

    case EAGAIN :
      throw TransmissionException("Non-blocking  I/O has been selected using O_NONBLOCK and no data was immediately\
               available for reading.", my_errno);

    case EIO    :
      throw TransmissionException("I/O error.", my_errno);

    case EISDIR :
      throw TransmissionException("fd refers to a directory.", my_errno);

    case EBADF  :
      throw TransmissionException("fd is not a valid file descriptor or is not open for reading.", my_errno);

    case EINVAL :
      throw TransmissionException("fd is attached to an object which is unsuitable for reading.", my_errno);

    case EFAULT :
      throw TransmissionException("buf is outside your accessible address space.", my_errno);

    default:
      throw TransmissionException("Unknown error", my_errno);
    }
  }

}
