/* NMM - Network-Integrated Multimedia Middleware
 *
 * Copyright (C) 2002-2005
 *                    NMM work group,
 *                    Computer Graphics Lab,
 *                    Saarland University, Germany
 *                    http://www.networkmultimedia.org
 *
 * Maintainer:        
 *                    NMM work group,
 *                    http://www.networkmultimedia.org
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or (at your option) any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307
 * USA
 */

#ifndef _NMM_CPUMONITOR_HPP_
#define _NMM_CPUMONITOR_HPP_

#include "nmm/utils/NamedObject.hpp"

#include <string>
#include <fstream>
#include <unistd.h> // needed for pid_t

namespace NMM {

/**
 * This class reads infos about the cpu utilisation of a thread from proc
 */

class CPUMonitor : public NamedObject {

    /**
     * Checks if a thread exists by trying to open its file in proc
     */
    bool isThreadExisting();

    pid_t thread_pid;
    char proc_cpu_file[32];
    ifstream* cpu_info;

    enum modes_t {CPU,STAT} mode; // read usage data from /proc/.../cpu or /proc/.../stat

  public:

    CPUMonitor(pid_t thread_pid);
    ~CPUMonitor();

    /**
     * \returns how many jiffies the thread occudpied the cpu (user time) or 0 if there was no thread with the specified pid
     */
    const unsigned int getUserTime();

    /**
     * \returns how many jiffies the system was busy because of the thread (system time) or 0 if there was no thread with the specified pid
     */
    const unsigned int getSystemTime();

    /**
     * \returns system and user time of the monitored thread
     */
    const unsigned int getBusyTime() {
        //  (getSystemTime() + getUserTime())
        return (getSystemTime() + getUserTime());
    }

    const pid_t getPID() {
        return thread_pid;
    }

};

}

#endif
