/* NMM - Network-Integrated Multimedia Middleware
 *
 * Copyright (C) 2002-2005
 *                    NMM work group,
 *                    Computer Graphics Lab,
 *                    Saarland University, Germany
 *                    http://www.networkmultimedia.org
 *
 * Maintainer:        Michael Repplinger <repplinger@cs.uni-sb.de>
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or (at your option) any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 * 
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307
 * USA
 */

#ifndef NMM_THREADEDOBJECT_HPP
#define NMM_THREADEDOBJECT_HPP

#include "Thread.hpp"

namespace NMM {
  class ThreadedObject {
  public:
    /** Constructor
     */
    ThreadedObject() : running(false) {}
    
    /**
     */
    virtual ~ThreadedObject() { deactivateThread();}
    
    /** returns a refernce to the thread
     */
    inline const Thread& getThread() {return thread;}
    
  protected:
    /** Abstract run-method which is started in the thread
     */
    virtual void run() = 0;


    /** activates the thread and starts the run method
     */
    inline void activateThread()   { if (isRunning()) return; running = true; thread.start(static_run, static_cast<void *>(this)); }

    /** deactivates the thread
     */
    inline void deactivateThread() { if (isRunning()) {running = false; thread.join();}}

    inline bool isRunning() const  { return running; }
  private:
    static void* static_run(void* obj) { static_cast<ThreadedObject*>(obj)->run();  return 0;}
    Thread thread;
    bool running;
  };
}

#endif // NMM_THREADEDOBJECT_HPP
